/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrWrapLib
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECOrbFile_h
#define _CECOrbFile_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CyrWrapLib/CECOrbFolder.h>
#include <Repository/CyrStRepository.h>

///////////////////////////////////////////////////////////////////////////////
// @class   CECOrbFile | This class wraps up functionality available via the
//          CyrStFile interface.  It is intended purely to hide the CORBA
//          mechanisms from products such as Impact.  This class is a borrowed
//          item.  This means that the Factory is constantly trying to destroy
//          it and it must be placed in a borrowing thread to keep it alive.
// @base    public | CECOrbLink
class CECOrbFile : public CECOrbLink
{
    // @access Public Members
public:
    // @cmember Constructor used to retrieve a File
    CECOrbFile( CECOrbFolder* pParent, const char* name );
    // @cmember Constructor used to store a file
    CECOrbFile( CECOrbFolder* pParent, const char* name, const char* desc );

    string GetName() const;      // @cmember Returns File name
    long GetCurrentStatus() const;  // @cmember Returns current file status

    bool Lock();    // @cmember Locks File access
    void Unlock();  // @cmember Unlocks File access

    // @cmember Retrieve File contents from database
    void CheckOut( const char* destPath, bool lock );
    // @cmember Transfer File contents to database
    void CheckIn( const char* sourcePath, bool lock );

    CyrStFile_ptr GetServant();  // @cmember Returns the underlying servant object
    CECOrbServer*  GetParentServer();
    CECOrbProject* GetParentProject();
    CECOrbFolder*  GetParentFolder();

    // @access Protected Members
protected:
    CyrStFile_var m_servant;  // @cmember Reference to underlying CORBA servant

    // @cmember Returns underlying servant reference as an item for linking.
    CyrStLink_ptr GetLinkServant()
    {
        return GetServant();
    }

    CECOrbFolder* m_folder;
    string     m_cacheName;      // @cmember cached Name to save on CORBA calls
}
;

#endif
