/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyrFFApi
// @module  CECFFTestResultsFlusher Implementation | This module contains the include
//          of the CECFFTestResultsFlusher Implementation
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECFFTestResultsFlusher_h
#define _CECFFTestResultsFlusher_h

///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Standard includes
#include <direct.h>
#include <iostream>
#include <CyrWrapLib/CECTestOutput.h>
#include <CyrffApi/CECFFFolder.h>

///////////////////////////////////////////////////////////////////////////////
// Omnithreading
#include <omnithread.h>

///////////////////////////////////////////////////////////////////////////////
// Standard includes
#include <functional>
#include <map>
#include <deque>

///////////////////////////////////////////////////////////////////////////////
// Typedefs
typedef std::deque<string> OutputList;
typedef std::map<string,OutputList*,std::less<string> > OutputTable;
typedef OutputList::iterator OutputListIt;
typedef std::map<string,OutputListIt, std::less<string> > OutputItTable;

class CECFFTestResultsFlusher
{
public:
    CECFFTestResultsFlusher();
    virtual ~CECFFTestResultsFlusher();

    void kickoff(string rwFilepath, string rwFilename, OutputList outList);

    // Virtual Run Function which kicks off the flushing mechanism
    virtual void Run();
public:
    // readers to tell us the state of the thread
    bool isRunning()
    {
        return m_bRunning;
    }
    bool isIdle()
    {
        return !m_bRunning;
    }
    bool useIfIdle();
    void setIdle();


private:

    // @cmember Flushes all available Test Output to a file
    void flushList();


    // @cmember tidy up any allocated memory during the flushing process
    void deleteLists();

    omni_thread* flusher;       // @cmember Our internal Flusher Thread

    OutputList      m_OutList;      // @cmember List of all output related to run.
    OutputItTable   m_iterator;     // @cmember Table of current iterator statuses.
    string      m_rwOutType;    // @cmember Output Type we are to flush
    bool            m_bRunning;     // @cmember Flag to indicate if the thread is running
    string       m_rwFilename;  // @cmember Output list we are to write to
    string      m_rwFullpath;   // @cmember Path to the output list
}
;
#endif
