/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc Registry
///////////////////////////////////////////////////////////////////////////////

#ifndef _RegistryProvider_h
#define _RegistryProvider_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Factory/Provider_i.h>
#include <Objects/Registry_i.h>

///////////////////////////////////////////////////////////////////////////////
// @class   RegistryProvider | This Provider is responsible for providing
//          access to a machine's Windows registry settings.  Unlike other
//          Providers this class does not create any CORBA objects in the New
//          method.  Instead a single registry class is created in the
//          constructor and this is always returned a requesting class.  This
//          object then provides access to the machine's registry settings.
// @base    public | ProviderImpl
class RegistryProvider : public ProviderImpl
{
    // @access Public Members
public:
    // @cmember Constructor
    RegistryProvider( CORBAServer* pServer, const char* name, const char* owner );
    virtual void Shutdown();

    // @cmember Create a new instance of a class
    CORBA::Object_ptr New( ParamList_ptr params );

    // @access Private Members
private:
    RegistryImpl* m_registry;  // @cmember Pointer to actual registry class
}
;

#endif
