 /* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "OSTAORB.h"
int CORBAServer::refcount=0;
CORBAServer::CORBAServer(int argc,char** argv)
{
    refcount++;
    //Initialise ORB
    m_orb=CORBA::ORB_init(argc,argv,"omniORB3");
    //Get reference to Root POA
    CORBA::Object_var obj = m_orb->resolve_initial_references("RootPOA");
    m_poa=PortableServer::POA::_narrow(obj);
    //Activate POA manager
    m_manager=m_poa->the_POAManager();
    m_manager->activate();
    // Obtain a reference to the root context of the Name service:
    // This wrapper is very specific and we never use CORBA without the naming service
    CORBA::Object_var nameobj;
    nameobj = m_orb->resolve_initial_references("NameService");
    // Narrow the reference returned.
    m_rootContext = CosNaming::NamingContext::_narrow(nameobj);
    if( CORBA::is_nil(m_rootContext) )
	throw CyrCORBAException("Failed to narrow the root naming context.","CORBAServer::CORBAServer",CYR_CORBA_OBJECT_NOT_FOUND);

}
CORBAServer::CORBAServer(const CORBAServer& srv)
{

    refcount++;
    m_orb=srv.m_orb;
    CORBA::Object_var obj = m_orb->resolve_initial_references("RootPOA");
    m_poa=PortableServer::POA::_narrow(obj);
    //Activate POA manager
    m_manager=m_poa->the_POAManager();
    m_manager->activate();
    // Obtain a reference to the root context of the Name service:
    // This wrapper is very specific and we never use CORBA without the naming service
    CORBA::Object_var nameobj;
    nameobj = m_orb->resolve_initial_references("NameService");
    // Narrow the reference returned.
    m_rootContext = CosNaming::NamingContext::_narrow(nameobj);
    if( CORBA::is_nil(m_rootContext) )
	throw CyrCORBAException("Failed to narrow the root naming context.","CORBAServer::CORBAServer",CYR_CORBA_OBJECT_NOT_FOUND);

}
CORBAServer::~CORBAServer()
{
    //possibly do some shutdown mechanisms
    refcount--;
}
void CORBAServer::Run()
{
    m_orb->run();
}
PortableServer::ObjectId* CORBAServer::ActivateObject(PortableServer::Servant p_servant)
{
    return m_poa->activate_object(p_servant);
}
void CORBAServer::DeActivateObject(const PortableServer::ObjectId& oid)
{
    m_poa->deactivate_object(oid);
}
void CORBAServer::NameObject(CORBA::Object_ptr objref,const char* classname,const char* regname)
{

    try 
    {
	CosNaming::Name cosName;
	cosName.length( 1 );
	CosNaming::NameComponent cosNameComponent;
	cosNameComponent.id   = CORBA::string_dup(regname);   // string copied
	cosNameComponent.kind = CORBA::string_dup(classname); // string copied
	cosName[0]=cosNameComponent;

	try
	{
#ifdef WIN32
	    	m_rootContext->bind( cosName, objref );
#endif
#ifdef SUN
	    // Try to bind object in NameService
		try
		{
			FindObjectWithName( classname,regname) ;
	    		m_rootContext->rebind( cosName, objref );
		}
		catch(...)
		{
	    		m_rootContext->bind( cosName, objref );
		}
#endif
	}
	catch( CosNaming::NamingContext::AlreadyBound& )
	{
	    // We might fail because the object is already bound.  If so rebind the
	    // object and continue as normal as this is not strictly an error.
	    m_rootContext->rebind( cosName, objref );
	}
    }
    catch( CORBA::Exception& )
    {
	// Any other CORBA exception is trapped here.  The most likely cause of
	// this problem will be due to the NameService being unavailable.
	throw CyrCORBAException("Could not bind object.  Is NameService active?","CORBAServer::NameObject",CYR_CORBA_OBJECT_NOT_FOUND);
    }
    catch( ... )
    {
	throw CyrCORBAException("Unknown exception","CORBAServer::NameObject",CYR_CORBA_OBJECT_NOT_FOUND);
    }

}
void CORBAServer::UnNameObject(const char* classname,const char* regname)
{

    CosNaming::Name cosName;
    cosName.length( 1 );
    CosNaming::NameComponent cosNameComponent;
    cosNameComponent.id   = CORBA::string_dup(regname);   // string copied
    cosNameComponent.kind = CORBA::string_dup(classname); // string copied
    cosName[0]=cosNameComponent;
    try
    {
	// Remove name from NameService
	m_rootContext->unbind( cosName );
    }
    catch( CORBA::Exception& )
    {
	// If we catch a CORBA exception here then it is likely that it we
	// are trying to unbind an obect that hasn't been bound.
	throw CyrCORBAException("Could not unbind object.  Is object registered?","CORBAServer::UnNameObject",CYR_CORBA_OBJECT_NOT_FOUND);
    }
    catch( ... )
    {
	// Catch any other exceptions and complain bitterly.
	throw CyrCORBAException("Unknown exception","CORBAServer::UnNameObject",CYR_CORBA_OBJECT_NOT_FOUND);
    }

}
CORBA::Object_ptr CORBAServer::FindObjectWithName(const char* classname,const char* regname)
{

    // Construct name object
    CosNaming::Name cosName;
    cosName.length( 1 );
    CosNaming::NameComponent cosNameComponent;
    cosNameComponent.id   = CORBA::string_dup(regname);   // string copied
    cosNameComponent.kind = CORBA::string_dup(classname); // string copied
    cosName[0]=cosNameComponent;

    try
    {
	// Resolve name object
	return m_rootContext->resolve( cosName );
    }
    catch( CORBA::Exception& )
    {
	// If it is a CORBA exception then the error is that the requested
	// object cannot be found.
	throw CyrCORBAException( "Could not resolve object", "CORBAServer::FindObjectWithName", CYR_CORBA_OBJECT_NOT_FOUND);
    }
    catch( ... )
    {
	// Catch all other exceptions
	throw CyrCORBAException( "Unknown exception", "CORBAServer::FindObjectWithName", CYR_CORBA_OBJECT_NOT_FOUND);
    }

    return CORBA::Object::_nil();

}
CORBAServer::NameList_t CORBAServer::GetListOfNames(const char* type)
{
    NameList_t namelist;

    // Get a list of all contexts on NameService
    CosNaming::BindingIterator_var bi;
    CosNaming::BindingList_var bl;
    CosNaming::Binding_var b;

    m_rootContext->list( 0, bl, bi );

    // Loop through all contexts and get all objects
    while ( bi->next_one(b) )
    {
	RegName rn;
	// If type is NULL then add name AND type to list
        if ( type == NULL ) 
	{
            // Append name to list
	    rn.Id = b->binding_name[0].id;
            rn.Kind = b->binding_name[0].kind;
	    namelist.push_back(rn);
	}
	else if ( strcmp( type, b->binding_name[0].kind ) == 0 )
        {
            // Append name to list
	    rn.Id = b->binding_name[0].id;
            rn.Kind = b->binding_name[0].kind;
	    namelist.push_back(rn);
        }
    }
    // Tidy up
    bi->destroy();

    return namelist;
}
