 /* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include <exception>
#include "CECInetInfo.h"
#include "mtsafe.hpp"
CECInetInfo::CECInetInfo()
{
#ifdef WIN32
    WORD wVersionRequested = MAKEWORD( 2, 2 );
    WSADATA wsaData;
    int err = WSAStartup( wVersionRequested, &wsaData );
    if ( err != 0 )
    {
        throw exception();
    }
#endif
}
CECInetInfo::~CECInetInfo()
{
#ifdef WIN32
    WSACleanup();
#endif
}
string CECInetInfo::GetIPAddressAsString(const string& name)const
{
    string hname(name);//copy of const string
    if(hname.empty())hname=GetHostName();
    hostent* host=CTMTSafe::gethostbyname(hname.c_str());
    if(host)
    {
        in_addr* addr;
        addr=((struct in_addr*)((host->h_addr_list)[0]));
        string ipadd=inet_ntoa(*addr);
        std::string::iterator si;
        //replace dots with underscores!!!
        std::string retstr="";
        for(si=ipadd.begin();si!=ipadd.end();si++)
        {
            if(*si=='.')retstr+='_';
            else retstr+= *si;
        }
        return retstr;
    }
    else
    {
        return "";
    }
}
string CECInetInfo::GetHostName()const
{
    char hostname[256];
    int iret=gethostname(hostname,256);
    if (iret==0)return string(hostname);
    else return string("");
}
long CECInetInfo::GetIPAddress(const string& name)const
{
    string hname(name);//copy of const string
    if(hname.empty())hname=GetHostName();
    hostent* host=gethostbyname(hname.c_str());
    if(host)
    {
        in_addr* addr;
        addr=((struct in_addr*)((host->h_addr_list)[0]));
        long ipadd=addr->s_addr;
        return ipadd;
    }
    else
    {
        return 0;
    }
}
