/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4514)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __CYRTOOLS_BINARY__
#  include "cbinary.hpp"
#endif  // __CYRTOOLS_BINARY__


//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////// Class CTBinary //
///////////////////////////////////////////////////////////////////////////////

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTBinary::CTBinary
//
// Default constructor.
//
// pBuffer : pointer to a buffer to copy from.
// iLen    : length of the buffer.
// Return  : none.
//-----------------------------------------------------------------------------
CTBinary::CTBinary (void* pBuffer, int iLen)
        : pBuffer_ (NULL), iLen_ (iLen)
{
    if (iLen_)
    {
        pBuffer_ = (void*) new char [iLen_] ;
        if (pBuffer)
        {
            memcpy (pBuffer_, pBuffer, iLen_) ;
        }
        else
        {
            memset (pBuffer_, 0, iLen_) ;
        }
    }
} // CTBinary::CTBinary

//================================================================< Operators >
//-----------------------------------------------------------------------------
// CTBinary::operator=
//
// Assign this object to a binary object.
//
// binary : reference to a binary object.
// Return : *this.
//-----------------------------------------------------------------------------
CTBinary DLLSPEC_BINARY operator+ (const CTBinary& binary1, const CTBinary& binary2)
{
    CTBinary binary (binary1) ;
    binary += binary2 ;
    return binary ;
} // operator+

//=========================================================< Control methodes >
//-----------------------------------------------------------------------------
// CTBinary::reset
//
// Reset this object.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTBinary::reset (void)
{
    if (pBuffer_)
    {
        delete [] pBuffer_ ;
        pBuffer_ = NULL ;
    }
    iLen_ = 0 ;
} // CTBinary::reset

//-----------------------------------------------------------------------------
// CTBinary::append
//
// Concatenate a binary object to this object.
//
// binary : reference to a binary object.
// Return : *this.
//-----------------------------------------------------------------------------
CTBinary& CTBinary::append (const CTBinary& binary)
{
    if (iLen_ + binary.iLen_)
    {
        char*   pNewBuffer = new char [iLen_ + binary.iLen_] ;
        if (iLen_)
        {
            memcpy (pNewBuffer, pBuffer_, iLen_) ;
        }
        if (binary.iLen_)
        {
            memcpy (&pNewBuffer[iLen_], binary.pBuffer_, binary.iLen_) ;
        }
        delete [] pBuffer_ ;
        pBuffer_ = pNewBuffer ;
        iLen_ += binary.iLen_ ;
    }

    return *this ;
} // CTBinary::CTBinary

//-----------------------------------------------------------------------------
// CTBinary::copy
//
// Assign this object to a binary object.
//
// binary : reference to a binary object.
// Return : *this.
//-----------------------------------------------------------------------------
CTBinary& CTBinary::copy (const CTBinary& binary)
{
    iLen_ = binary.iLen_ ;
    if (binary.pBuffer_)
    {
        pBuffer_ = (void*) new char [iLen_] ;
        memcpy (pBuffer_, binary.pBuffer_, iLen_) ;
    }
    else
    {
        pBuffer_ = NULL ;
    }

    return *this ;
} // CTBinary::copy

//-----------------------------------------------------------------------------
// CTBinary::index
//
// Search for the first occurence of a buffer in this object.
// Return the position of the first occurence or -1 if not found.
//
// pBuffer : pointer to a buffer.
// iLen    : length of the buffer.
// iStart  : offset to start from.
// Return  : iIndex.
//-----------------------------------------------------------------------------
int CTBinary::index (void* pBuffer, int iLen, int iStart /* = 0 */)
{
    int iIndex = -1 ;

    if (iStart < 0)
    {
        iStart = 0 ;
    }
    if ( (pBuffer) && (iLen > 0) )
    {
        for (int i = iStart ; i < iLen_ ; ++ i)
        {
            if ( ( (char*) pBuffer_)[i] == ( (char*) pBuffer)[0])
            {
                for (int j = 0 ; j < iLen ; ++ j)
                {
                    if ( ( (char*) pBuffer_)[i + j] == ( (char*) pBuffer)[j])
                    {
                        iIndex = i ;
                        break ;
                    }
                }
                if (iIndex == i)
                {
                    break ;
                }
            }
        }
    }

    return iIndex ;
} // CTBinary::index

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// End of Class CTBinary //
///////////////////////////////////////////////////////////////////////////////
