/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __CYRTOOLS_TIME__
#define __CYRTOOLS_TIME__

//=================================================================< Includes >

#include <time.h>

#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __MTSAFE__
#  include <mtsafe.hpp>
#endif  // __MTSAFE__

#include <string>
using std::string;
#ifdef WIN32
#include <windows.h>
#endif
//===================================================================< Macros >
#define DLLSPEC_TIME    DLLSPEC (CYRTOOLS)


//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////// Class CTTime //
///////////////////////////////////////////////////////////////////////////////
// Class representing the time.
class CTTime ;
DLLSPEC_TIME CTTime operator- (const CTTime& time1, const CTTime& time2);

class DLLSPEC_TIME CTTime
{
    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CTTime (const unsigned long ulSeconds = 0L, const unsigned long ulMilliSeconds = 0L) ;
    CTTime (const CTTime& time) ;
    ~CTTime () ;

public:                         // Operators.
    DLLSPEC_TIME friend bool operator== (const CTTime& time1, const CTTime& time2) ;
    DLLSPEC_TIME friend bool operator< (const CTTime& time1, const CTTime& time2) ;
    DLLSPEC_TIME friend bool operator<= (const CTTime& time1, const CTTime& time2) ;
    DLLSPEC_TIME friend bool operator> (const CTTime& time1, const CTTime& time2) ;
    DLLSPEC_TIME friend bool operator>= (const CTTime& time1, const CTTime& time2) ;
    DLLSPEC_TIME friend CTTime  operator+ (const CTTime& time1, const CTTime& time2) ;
    DLLSPEC_TIME friend CTTime  operator- (const CTTime& time1, const CTTime& time2) ;
    CTTime&                     operator= (const CTTime& time) ;
    CTTime&                     operator= (const unsigned long ulMilliSeconds) ;
    CTTime&                     operator+= (const CTTime& time) ;
    CTTime&                     operator+= (const unsigned long ulMilliSeconds) ;
    CTTime&                     operator-= (const CTTime& time) ;
    CTTime&                     operator-= (const unsigned long ulMilliSeconds) ;
    CTTime&                     operator++ () ;
    CTTime&                     operator-- () ;
    operator unsigned long () const ;

public:                         // Get methodes.
    unsigned long               getSeconds () const
    {
        return ulSeconds_ ;
    }
    unsigned long               getMilliSeconds () const
    {
        return ulMilliSeconds_ ;
    }
    bool                    isNull () const
    {
        return ( (ulSeconds_ == 0L) && (ulMilliSeconds_ == 0L) ) ;
    }

public:                         // Control methodes.
    void                        reset ()
    {
        ulSeconds_ = ulMilliSeconds_ = 0L ;
    }
    void                        setSecondsAndMilliSeconds (unsigned long ulSeconds, unsigned long ulMilliSeconds)
    {
        setSeconds (ulSeconds) ;
        setMilliSeconds (ulMilliSeconds) ;
    }
    void                        setSeconds (unsigned long ulSeconds)
    {
        ulSeconds_ = ulSeconds ;
    }
    void                        setMilliSeconds (unsigned long ulMilliSeconds)
    {
        ulSeconds_ += (ulMilliSeconds / 1000L) ;
        (ulMilliSeconds_ = ulMilliSeconds % 1000L) ;
    }
    void                        now () ;
    char                        *getPrintable();
    string              Format(const string& fmtstr);

private:                        // Members.
    unsigned long               ulSeconds_ ;        // Number of seconds.
    unsigned long               ulMilliSeconds_ ;   // Number of milliseconds.

    //-----------------------------------------------------------------------------
}

; // CTTime

///////////////////////////////////////////////////////////////////////////////
// Wrapper for simple sleep call across platforms
inline void CyrSleep(unsigned long millisecs)
{
#if defined(WIN32)
    Sleep(millisecs);
#elif defined(Linux) || defined(HPUX) ||defined (SOLARIS)
    /* alanw */
    struct timespec req;
    req.tv_sec = millisecs / 1000;
    req.tv_nsec = (millisecs - req.tv_sec * 1000) * 1000 * 1000;
    nanosleep(& req, NULL);
#endif
}

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// End of Classes CTTime //
///////////////////////////////////////////////////////////////////////////////

#endif  // __CYRTOOLS_TIME__
