/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4514)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __LOGFILE__
#  include "logfile.hpp"
#endif // __LOGFILE__


//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////// Class CTOLogFileList //
///////////////////////////////////////////////////////////////////////////////

//================================================================< Operators >
//-----------------------------------------------------------------------------
// CTFunctor::operator<<
//
// zStr   : pointer to a string.
// Return : *this.
//-----------------------------------------------------------------------------
CTOLogFileList& CTOLogFileList::operator<< (const char* zStr)
{
    size_t i ;
    for (i = 0 ; i < size() ; ++ i)
    {
        switch ( (*this)[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
            {
                ofstream& oFStream = (ofstream&) ( (*this)[i].getLogFile () ) ;
                oFStream << zStr ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTREAM :
            {
                ostream& oStreamWithAssign = (ostream&) ( (*this)[i].getLogFile () ) ;
                oStreamWithAssign << zStr ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            {
                ostrstream& oStrStream = (ostrstream&) ( (*this)[i].getLogFile () ) ;
                oStrStream << zStr ;
            }
            break ;
            /*
                        case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) ( (*this)[i].getLogFile () ) ;
                                oEventLogStream << zStr ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("operator<<: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return *this ;
} // CTOLogFileList::operator<<

//-----------------------------------------------------------------------------
// CTFunctor::operator<<
//
// c      : char.
// Return : *this.
//-----------------------------------------------------------------------------
CTOLogFileList& CTOLogFileList::operator<< (char c)
{
    size_t i ;
    for (i = 0 ; i < size () ; ++ i)
    {
        switch ( (*this)[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
            {
                ofstream& oFStream = (ofstream&) ( (*this)[i].getLogFile () ) ;
                oFStream << c ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTREAM :
            {
                ostream& oStreamWithAssign = (ostream&) ( (*this)[i].getLogFile () ) ;
                oStreamWithAssign << c ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            {
                ostrstream& oStrStream = (ostrstream&) ( (*this)[i].getLogFile () ) ;
                oStrStream << c ;
            }
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) ( (*this)[i].getLogFile () ) ;
                                oEventLogStream << c ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("operator<<: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return *this ;
} // CTOLogFileList::operator<<

//-----------------------------------------------------------------------------
// CTFunctor::operator<<
//
// uc     : unsigned char.
// Return : *this.
//-----------------------------------------------------------------------------
CTOLogFileList& CTOLogFileList::operator<< (unsigned char uc)
{
    size_t i ;
    for (i = 0 ; i < size () ; ++ i)
    {
        switch ( (*this)[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
            {
                ofstream& oFStream = (ofstream&) ( (*this)[i].getLogFile () ) ;
                oFStream << uc ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTREAM :
            {
                ostream& oStreamWithAssign = (ostream&) ( (*this)[i].getLogFile () ) ;
                oStreamWithAssign << uc ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            {
                ostrstream& oStrStream = (ostrstream&) ( (*this)[i].getLogFile () ) ;
                oStrStream << uc ;
            }
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) ( (*this)[i].getLogFile () ) ;
                                oEventLogStream << uc ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("operator<<: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return *this ;
} // CTOLogFileList::operator<<

//-----------------------------------------------------------------------------
// CTFunctor::operator<<
//
// d      : int.
// Return : *this.
//-----------------------------------------------------------------------------
CTOLogFileList& CTOLogFileList::operator<< (int d)
{
    size_t i ;
    for (i = 0 ; i < size () ; ++ i)
    {
        switch ( (*this)[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
            {
                ofstream& oFStream = (ofstream&) ( (*this)[i].getLogFile () ) ;
                oFStream << d ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTREAM :
            {
                ostream& oStreamWithAssign = (ostream&) ( (*this)[i].getLogFile () ) ;
                oStreamWithAssign << d ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            {
                ostrstream& oStrStream = (ostrstream&) ( (*this)[i].getLogFile () ) ;
                oStrStream << d ;
            }
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) ( (*this)[i].getLogFile () ) ;
                                oEventLogStream << d ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("operator<<: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return *this ;
} // CTOLogFileList::operator<<

//-----------------------------------------------------------------------------
// CTFunctor::operator<<
//
// ui     : unsigned int.
// Return : *this.
//-----------------------------------------------------------------------------
CTOLogFileList& CTOLogFileList::operator<< (unsigned int ui)
{
    size_t i ;
    for (i = 0 ; i < size () ; ++ i)
    {
        switch ( (*this)[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
            {
                ofstream& oFStream = (ofstream&) ( (*this)[i].getLogFile () ) ;
                oFStream << ui ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTREAM :
            {
                ostream& oStreamWithAssign = (ostream&) ( (*this)[i].getLogFile () ) ;
                oStreamWithAssign << ui ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            {
                ostrstream& oStrStream = (ostrstream&) ( (*this)[i].getLogFile () ) ;
                oStrStream << ui ;
            }
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) ( (*this)[i].getLogFile () ) ;
                                oEventLogStream << ui ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("operator<<: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return *this ;
} // CTOLogFileList::operator<<

//-----------------------------------------------------------------------------
// CTFunctor::operator<<
//
// l      : long.
// Return : *this.
//-----------------------------------------------------------------------------
CTOLogFileList& CTOLogFileList::operator<< (long l)
{
    size_t i ;
    for (i = 0 ; i < size () ; ++ i)
    {
        switch ( (*this)[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
            {
                ofstream& oFStream = (ofstream&) ( (*this)[i].getLogFile () ) ;
                oFStream << l ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTREAM :
            {
                ostream& oStreamWithAssign = (ostream&) ( (*this)[i].getLogFile () ) ;
                oStreamWithAssign << l ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            {
                ostrstream& oStrStream = (ostrstream&) ( (*this)[i].getLogFile () ) ;
                oStrStream << l ;
            }
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) ( (*this)[i].getLogFile () ) ;
                                oEventLogStream << l ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("operator<<: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return *this ;
} // CTOLogFileList::operator<<

//-----------------------------------------------------------------------------
// CTFunctor::operator<<
//
// ul     : unsigned long.
// Return : *this.
//-----------------------------------------------------------------------------
CTOLogFileList& CTOLogFileList::operator<< (unsigned long ul)
{
    size_t i ;
    for (i = 0 ; i < size () ; ++ i)
    {
        switch ( (*this)[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
            {
                ofstream& oFStream = (ofstream&) ( (*this)[i].getLogFile () ) ;
                oFStream << ul ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTREAM :
            {
                ostream& oStreamWithAssign = (ostream&) ( (*this)[i].getLogFile () ) ;
                oStreamWithAssign << ul ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            {
                ostrstream& oStrStream = (ostrstream&) ( (*this)[i].getLogFile () ) ;
                oStrStream << ul ;
            }
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) ( (*this)[i].getLogFile () ) ;
                                oEventLogStream << ul ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("operator<<: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return *this ;
} // CTOLogFileList::operator<<

//-----------------------------------------------------------------------------
// CTFunctor::operator<<
//
// s      : short.
// Return : *this.
//-----------------------------------------------------------------------------
CTOLogFileList& CTOLogFileList::operator<< (short s)
{
    size_t i ;
    for (i = 0 ; i < size () ; ++ i)
    {
        switch ( (*this)[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
            {
                ofstream& oFStream = (ofstream&) ( (*this)[i].getLogFile () ) ;
                oFStream << s ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTREAM :
            {
                ostream& oStreamWithAssign = (ostream&) ( (*this)[i].getLogFile () ) ;
                oStreamWithAssign << s ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            {
                ostrstream& oStrStream = (ostrstream&) ( (*this)[i].getLogFile () ) ;
                oStrStream << s ;
            }
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) ( (*this)[i].getLogFile () ) ;
                                oEventLogStream << s ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("operator<<: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return *this ;
} // CTOLogFileList::operator<<

//-----------------------------------------------------------------------------
// CTFunctor::operator<<
//
// us     : unsigned short.
// Return : *this.
//-----------------------------------------------------------------------------
CTOLogFileList& CTOLogFileList::operator<< (unsigned short us)
{
    size_t i ;
    for (i = 0 ; i < size () ; ++ i)
    {
        switch ( (*this)[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
            {
                ofstream& oFStream = (ofstream&) ( (*this)[i].getLogFile () ) ;
                oFStream << us ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTREAM :
            {
                ostream& oStreamWithAssign = (ostream&) ( (*this)[i].getLogFile () ) ;
                oStreamWithAssign << us ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            {
                ostrstream& oStrStream = (ostrstream&) ( (*this)[i].getLogFile () ) ;
                oStrStream << us ;
            }
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) ( (*this)[i].getLogFile () ) ;
                                oEventLogStream << us ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("operator<<: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return *this ;
} // CTOLogFileList::operator<<

//-----------------------------------------------------------------------------
// CTFunctor::operator<<
//
// f      : float.
// Return : *this.
//-----------------------------------------------------------------------------
CTOLogFileList& CTOLogFileList::operator<< (float f)
{
    size_t i ;
    for (i = 0 ; i < size () ; ++ i)
    {
        switch ( (*this)[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
            {
                ofstream& oFStream = (ofstream&) ( (*this)[i].getLogFile () ) ;
                oFStream << f ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTREAM :
            {
                ostream& oStreamWithAssign = (ostream&) ( (*this)[i].getLogFile () ) ;
                oStreamWithAssign << f ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            {
                ostrstream& oStrStream = (ostrstream&) ( (*this)[i].getLogFile () ) ;
                oStrStream << f ;
            }
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) ( (*this)[i].getLogFile () ) ;
                                oEventLogStream << f ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("operator<<: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return *this ;
} // CTOLogFileList::operator<<

//-----------------------------------------------------------------------------
// CTFunctor::operator<<
//
// d      : double.
// Return : *this.
//-----------------------------------------------------------------------------
CTOLogFileList& CTOLogFileList::operator<< (double d)
{
    size_t i ;
    for (i = 0 ; i < size () ; ++ i)
    {
        switch ( (*this)[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
            {
                ofstream& oFStream = (ofstream&) ( (*this)[i].getLogFile () ) ;
                oFStream << d ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTREAM :
            {
                ostream& oStreamWithAssign = (ostream&) ( (*this)[i].getLogFile () ) ;
                oStreamWithAssign << d ;
            }
            break ;
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            {
                ostrstream& oStrStream = (ostrstream&) ( (*this)[i].getLogFile () ) ;
                oStrStream << d ;
            }
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) ( (*this)[i].getLogFile () ) ;
                                oEventLogStream << d ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("operator<<: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return *this ;
} // CTOLogFileList::operator<<

//-----------------------------------------------------------------------------
// CTFunctor::operator<<
//
// str      : a RWCstring object.
// Return : *this.
//-----------------------------------------------------------------------------
CTOLogFileList& CTOLogFileList::operator<< (const string& str)
{
    *this << str.c_str() ;
    return *this;
} // CTOLogFileList::operator<<


//=============================================================< Manipulators >

//-----------------------------------------------------------------------------
// dec
//
// logFileList : list of log files.
// Return      : logFileList.
//-----------------------------------------------------------------------------
CTOLogFileList& dec (CTOLogFileList& logFileList)
{
    size_t i ;
    for (i = 0 ; i < logFileList.size () ; ++ i)
    {
        switch (logFileList[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            //          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
            {
                ostream& oStream =  logFileList[i].getLogFile () ;
                oStream.setf (ios::dec, ios::basefield) ;
            }
            break ;
        default :
            throw CTLogFileException ("dec: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return logFileList ;
} // dec

//-----------------------------------------------------------------------------
// hex
//
// logFileList : list of log files.
// Return      : logFileList.
//-----------------------------------------------------------------------------
CTOLogFileList& hex (CTOLogFileList& logFileList)
{
    size_t i ;
    for (i = 0 ; i < logFileList.size () ; ++ i)
    {
        switch (logFileList[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            //          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
            {
                ostream& oStream =  logFileList[i].getLogFile () ;
                oStream.setf (ios::hex, ios::basefield) ;
            }
            break ;
        default :
            throw CTLogFileException ("hex: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return logFileList ;
} // hex

//-----------------------------------------------------------------------------
// oct
//
// logFileList : list of log files.
// Return      : logFileList.
//-----------------------------------------------------------------------------
CTOLogFileList& oct (CTOLogFileList& logFileList)
{
    size_t i ;
    for (i = 0 ; i < logFileList.size () ; ++ i)
    {
        switch (logFileList[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            //          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
            {
                ostream& oStream =  logFileList[i].getLogFile () ;
                oStream.setf (ios::oct, ios::basefield) ;
            }
            break ;
        default :
            throw CTLogFileException ("oct: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return logFileList ;
} // oct

//-----------------------------------------------------------------------------
// flush
//
// logFileList : list of log files.
// Return      : logFileList.
//-----------------------------------------------------------------------------
CTOLogFileList& flush (CTOLogFileList& logFileList)
{
    size_t i ;
    for (i = 0 ; i < logFileList.size () ; ++ i)
    {
        switch (logFileList[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            //          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
            {
                ostream& oStream =  logFileList[i].getLogFile () ;
                oStream.flush () ;
            }
            break ;
        default :
            throw CTLogFileException ("flush: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return logFileList ;
} // flush

//-----------------------------------------------------------------------------
// endl
//
// logFileList : list of log files.
// Return      : logFileList.
//-----------------------------------------------------------------------------
CTOLogFileList& endl (CTOLogFileList& logFileList)
{
    size_t i ;
    for (i = 0 ; i < logFileList.size () ; ++ i)
    {
        switch (logFileList[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            //          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
            {
                ostream& oStream =  logFileList[i].getLogFile () ;
                oStream << '\n' << std::flush ;
            }
            break ;
        default :
            throw CTLogFileException ("endl: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return logFileList ;
} // endl

//-----------------------------------------------------------------------------
// ends
//
// logFileList : list of log files.
// Return      : logFileList.
//-----------------------------------------------------------------------------
CTOLogFileList& ends (CTOLogFileList& logFileList)
{
    size_t i ;
    for (i = 0 ; i < logFileList.size () ; ++ i)
    {
        switch (logFileList[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            //          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
            {
                ostream& oStream =  logFileList[i].getLogFile () ;
                oStream << char ('\0') ;
            }
            break ;
        default :
            throw CTLogFileException ("ends: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return logFileList ;
} // ends

//-----------------------------------------------------------------------------
// error
//
// logFileList : list of log files.
// Return      : logFileList.
//-----------------------------------------------------------------------------
CTOLogFileList& error (CTOLogFileList& logFileList)
{
    size_t i ;
    for (i = 0 ; i < logFileList.size () ; ++ i)
    {
        switch (logFileList[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) (logFileList[i].getLogFile () ) ;
                                oEventLogStream.setEventType (EVENTLOGSTREAM_ERROR_TYPE) ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("error: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return logFileList ;
} // error

//-----------------------------------------------------------------------------
// warning
//
// logFileList : list of log files.
// Return      : logFileList.
//-----------------------------------------------------------------------------
CTOLogFileList& warning (CTOLogFileList& logFileList)
{
    size_t i ;
    for (i = 0 ; i < logFileList.size () ; ++ i)
    {
        switch (logFileList[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) (logFileList[i].getLogFile () ) ;
                                oEventLogStream.setEventType (EVENTLOGSTREAM_WARNING_TYPE) ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("warning: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return logFileList ;
} // warning

//-----------------------------------------------------------------------------
// information
//
// logFileList : list of log files.
// Return      : logFileList.
//-----------------------------------------------------------------------------
CTOLogFileList& information (CTOLogFileList& logFileList)
{
    size_t i ;
    for (i = 0 ; i < logFileList.size () ; ++ i)
    {
        switch (logFileList[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) (logFileList[i].getLogFile () ) ;
                                oEventLogStream.setEventType (EVENTLOGSTREAM_INFORMATION_TYPE) ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("information: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return logFileList ;
} // information

//-----------------------------------------------------------------------------
// auditSuccess
//
// logFileList : list of log files.
// Return      : logFileList.
//-----------------------------------------------------------------------------
CTOLogFileList& auditSuccess (CTOLogFileList& logFileList)
{
    size_t i ;
    for (i = 0 ; i < logFileList.size () ; ++ i)
    {
        switch (logFileList[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) (logFileList[i].getLogFile () ) ;
                                oEventLogStream.setEventType (EVENTLOGSTREAM_AUDIT_SUCCESS) ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("auditSuccess: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return logFileList ;
} // auditSuccess

//-----------------------------------------------------------------------------
// auditFailure
//
// logFileList : list of log files.
// Return      : logFileList.
//-----------------------------------------------------------------------------
CTOLogFileList& auditFailure (CTOLogFileList& logFileList)
{
    size_t i ;
    for (i = 0 ; i < logFileList.size () ; ++ i)
    {
        switch (logFileList[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) (logFileList[i].getLogFile () ) ;
                                oEventLogStream.setEventType (EVENTLOGSTREAM_AUDIT_FAILURE) ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("auditFailure: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return logFileList ;
} // auditFailure

//-----------------------------------------------------------------------------
// __listBinaryData
//
// logFileList : list of log files.
// pBuf        : pointer to binary data.
// iBufLen     : length of binary data.
// Return      : logFileList.
//-----------------------------------------------------------------------------
CTOLogFileList& __listBinaryData (CTOLogFileList& logFileList, void* pBuf, int iBufLen)
{
    size_t i ;
    for (i = 0 ; i < logFileList.size () ; ++ i)
    {
        switch (logFileList[i].getType () )
        {
        case CTOLogFileIterator::LOGFILE_OFSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTREAM :
        case CTOLogFileIterator::LOGFILE_OSTRSTREAM :
            break ;
            /*          case CTOLogFileIterator::LOGFILE_OEVENTLOGSTREAM :
                            {
                                CTOEventLogStream& oEventLogStream = (CTOEventLogStream&) (logFileList[i].getLogFile () ) ;
                                oEventLogStream.setBinaryData (pBuf, iBufLen) ;
                            }
                            break ;*/
        default :
            throw CTLogFileException ("auditFailure: an iterator is incorrect.", __FILE__, __LINE__) ;
        }
    }

    return logFileList ;
} // __listBinaryData

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////// End of Class CTOLogFileList //
///////////////////////////////////////////////////////////////////////////////
