/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4514)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __MEMORY_VIEW__
#  include "memview.hpp"
#endif  // __MEMORY_VIEW__

#ifndef __XALLOC__
#  include "xalloc.hpp"
#endif  // __XALLOC__

#include <stdio.h>
#include <ctype.h>


//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////// Class CTMemView //
///////////////////////////////////////////////////////////////////////////////

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTMemView::CTMemView
//
// Constructor.
//
// Arguments : pBuffer   : Pointer to binary information.
//             lBufLen   : Number of bytes in the binary information.
//             label     : Label to put at the begining of the line.
//             iWidth    : Number of byte to put per line.
//             sepAscii  : Separator of binay and ascii.
//             cDefChar  : Default character to print.
//             endOfLine : Label to put at the end of the line.
// Return    : none.
//-----------------------------------------------------------------------------
CTMemView::CTMemView (const void* pBuffer, const long lBufLen, string label /* = "" */, int iWidth /* = 16 */, string sepAscii /* = " " */, char cDefChar /* = ' ' */, string endOfLine /* = "" */)
        : pBinary_ (NULL), lBinaryLen_ (0L)
{
    if ( (pBuffer) && (lBufLen > 0) )
    {   // Save the binary buffer.
        try
        {
            pBinary_ = (void*) new char [lBufLen] ;
        }
        catch (CTxalloc&)
        {
            throw CTMemViewException ("CTMemView: new char [lBufLen] failed.", __FILE__, __LINE__) ;
        }
        lBinaryLen_ = lBufLen ;
        memcpy (pBinary_, pBuffer, lBufLen) ;
    }

    for (long i = 0 ; i < lBufLen ; i += (long) iWidth)
    {
        string  address ;
        if (label == "")
        {   // Get address.
            char    zAddr[6] ;
            sprintf (zAddr, "%04.4X ", (short int) i) ;
            address = zAddr ;
        }
        else
        {
            address = label ;
        }
        // Get the first part of the line.
        string  hex ;
        string  ascii ;
        for (long j = 0 ; j < (long) iWidth ; ++ j)
        {
            if (j < lBufLen - i)
            {
                ascii += ( ( (char*) pBuffer) [i + j]) ;
                char    zHex[4] ;
                sprintf (zHex, "%02.2hX ", (short int) (0x00FF & ascii[(size_t) j]) ) ;
                hex += zHex ;
                if (isprint (ascii[(size_t) j]) == 0)
                {
                    ascii[(size_t) j] = '.' ;
                }
            }
            else
            {
                ascii += (cDefChar) ;
                hex += "00 " ;
            }
        }

        // Save the line ;
        ascii_ += address + hex + sepAscii + ascii + endOfLine + "\n" ;
    }
} // CTMemView::CTMemView

//-----------------------------------------------------------------------------
// CTMemView::~CTMemView
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
CTMemView::~CTMemView ()
{
    if (pBinary_)
    {
        // Delete the binary object.
        delete [] (char*) pBinary_ ;
        pBinary_ = NULL ;
        lBinaryLen_ = 0L ;
    }
} // CTMemView::~CTMemView

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////// End of Class CTMemView //
///////////////////////////////////////////////////////////////////////////////
