/* Copyright (C) 2000 Cyrano SA

   This file is part of OpenSTA

   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.

   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html

   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//=================================================================< Includes >
#ifndef	__DEFINITION__
#  include "definition.h"
#endif	// __DEFINITION__

#ifdef	WIN32
#  ifndef	_WINDOWS_
#    define	WIN32_LEAN_AND_MEAN
#    define	NOGDICAPMASKS
#    define	NOVIRTUALKEYCODES
#    define	NOWINMESSAGES
#    define	NOWINSTYLES
#    define	NOSYSMETRICS
#    define	NOMENUS
#    define	NOICONS
#    define	NOKEYSTATES
#    define	NOSYSCOMMANDS
#    define	NORASTEROPS
#    define	NOSHOWWINDOW
#    define	OEMRESOURCE
#    define	NOATOM
#    define	NOCLIPBOARD
#    define	NOCOLOR
#    define	NOCTLMGR
#    define	NODRAWTEXT
#    define	NOGDI
#    define	NOKERNEL
//#    define	NOUSER
#    define	NONLS
#    define	NOMB
#    define	NOMEMMGR
#    define	NOMETAFILE
#    define	NOMINMAX
#    define	NOMSG
#    define	NOOPENFILE
#    define	NOSCROLL
#    define	NOSERVICE
#    define	NOSOUND
#    define	NOTEXTMETRIC
#    define	NOWH
#    define	NOWINOFFSETS
#    define	NOCOMM
#    define	NOKANJI
#    define	NOHELP
#    define	NOPROFILER
#    define	NODEFERWINDOWPOS
#    define	NOMCX
#    pragma warning(disable : 4201)
#    include <windows.h>
#    pragma warning(default : 4201)
#  endif	// _WINDOWS_
#  include <process.h>
#endif	// WIN32

#ifdef	DGUX
#  ifndef	_POSIX4A_DRAFT6_SOURCE
#    define	_POSIX4A_DRAFT6_SOURCE
#  endif	// _POSIX4A_DRAFT6_SOURCE
#  include <pthread.h>
#  include <sched.h>
#  include <signal.h>
#endif	// DGUX

#if defined(AIX)
#  include <pthread.h>
#  include <sys/sched.h>
#  include <signal.h>
#endif	// AIX

#if defined(SUN)
#  include <pthread.h>
#  include <sched.h>
#  include <signal.h>
#  include <errno.h>
#endif	// HPUX

#if defined(HPUX) 
#  include <pthread.h>
#  include <sched.h>
#  include <signal.h>
#  undef	sleep
#endif	// HPUX

#ifndef	__MTHREAD__
#  include "mthread.hpp"
#endif	// __MTHREAD__

#ifndef	__MUTEX__
#  include <mutex.hpp>
#endif	// __MUTEX__

#ifndef __XALLOC__
#  include "xalloc.hpp"
#endif	// __XALLOC__
#include <list>
using std::list;

//============================================================< Define Macros >
#if defined	(WIN32)
#  define	thread_yield()	::Sleep (0)
#endif	// WIN32
#ifdef	DGUX
#  define	thread_yield()	::pthread_yield (NULL)
#endif	// DGUX
#if	defined (AIX) || defined (HPUX)
#  define	thread_yield()	::pthread_yield ()
#endif	// AIX || HPUX
#if defined	(SUN)
#  define	thread_yield()	::usleep (10)
#endif	// WIN32


//================================================================< Variables >
#ifdef	WIN32
DWORD					GdwTLSIndex = TLS_OUT_OF_INDEXES ;	// TLS Index.
#endif	// WIN32
#if	defined (DGUX) || defined (AIX) || defined (SUN)
pthread_key_t			GptThreadKey = NULL ;			// Data key.
int						GiNbThreads = 0 ;				// Number of knows threads.
#endif	// DGUX || AIX
#ifdef	HPUX
pthread_key_t			GptThreadKey = NULL ;			// Data key.
#endif	// HPUX


//======================================================< Definition of class >

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////// Class CTThreadInfo //
///////////////////////////////////////////////////////////////////////////////
// Base class for the threads.
typedef list<void *> voidList ;
class CTThreadInfo : virtual public CTMTSafe
{
//-----------------------------------------------------------------------------
public:						// Types
	enum enumInitState
	{
		INIT_ZERO = 0,
		INIT_PARTIAL,
		INIT_TOTAL,
		INIT_END
	} ;

public:						// Constructor(s) and destructor.
	CTThreadInfo () ;
	~CTThreadInfo () ;

public:						// Get methodes.
	const enumThreadState	getState (void) const	{ return eState_ ; }

public:						// Control methodes.
	void					parentInitializeSize (void**, const CTFunctor&, fnThreadAbortRoutine, enumThreadState, enumThreadPriority) ;
	unsigned int			suspend (void) ;
	unsigned int			resume (void) ;
	void					kill (void* pStatus = NULL) ;
	void 					setPriority (enumThreadPriority) ;
	void					abort (void* pStatus = NULL) ;
	void					sleep (unsigned long ulMilliseconds = 0L) ;
	void					join (void** ppStatus = NULL) ;

public:						// Members.
	CTFunctor*				pFunctorStartRoutine_ ;		// Thread start routine.
	bool					bAbort_ ;					// Abort flag.
	fnThreadAbortRoutine	pAbortRoutine_ ;			// Thread abort routine.
	unsigned int			uSuspendCount_ ;			// Suspend count of the thread.
	enumThreadState			eState_ ;					// State of the thread.
	enumInitState			eInitState_ ;				// Initialize state of the thread (INIT_ZERO, INIT_PARTIAL, INIT_TOTAL, INIT_END).
	void*					pData_ ;					// Data pointer.
	enumThreadPriority		ePriority_ ;				// Thread priority.
	void*					pKillStatus_ ;				// Exit status specified by a kill.
	void**					ppThreadInfoMember_ ;		// Pointer to thread object's thread information member.
	list<void*>		statusPointerList_ ;		// List of pointers to a pointer to the termination status of the thread to affect when aborting.
	CTMutex					mutexInfo_ ;				// Mutex specific to the thread.
#ifdef	WIN32
	unsigned				uThreadId_ ;				// Thread identifier.
	unsigned long			ulThread_ ;					// Thread handle.
#endif	// WIN32
#if	defined (DGUX) || defined (AIX) || defined (HPUX) || defined(SUN)
	pthread_t				threadId_ ;					// Thread identifier.
#endif	// DGUX || AIX || HPUX
#ifdef	HPUX
	CTCondition				condResume_ ;				// Condition for resume the thread.
#endif	// HPUX

//-----------------------------------------------------------------------------
private:					// Constructor(s) and destructor.
	CTThreadInfo (const CTThreadInfo&) ;

private:					// Operators.
	CTThreadInfo&			operator= (const CTThreadInfo&) ;

//-----------------------------------------------------------------------------
} ; // CTThreadInfo

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////// End of Class CTThreadInfo //
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////// Class CTPublicCurrentThread //
///////////////////////////////////////////////////////////////////////////////
// Class used to control the current thread with more methodes.
class CTPublicCurrentThread : public CTCurrentThread
{
//-----------------------------------------------------------------------------
public:						// Types.
#ifdef	WIN32
	typedef	unsigned (_stdcall *fnStartRoutine_t) (void*) ;
#endif	// WIN32
#if	defined (DGUX) || defined (AIX) || defined (HPUX) || defined (SUN)
	typedef	void* (*fnStartRoutine_t) (void*) ;
#endif	// DGUX || AIX || HPUX

public:						// Control methodes.
#ifdef	WIN32
	static bool			threadAttachOrDetach (DWORD) ;
	static void				threadCleanupRoutine (void) ;
	static unsigned			_stdcall threadStartRoutine (CTThreadInfo**) ;
#endif	// WIN32
#if	defined (DGUX) || defined (AIX) || defined (HPUX) || defined (SUN)
	static void				threadDataDestructor (CTThreadInfo*) ;
	static void				threadCleanupRoutine (CTThreadInfo*) ;
	static void*			threadStartRoutine (CTThreadInfo**) ;
#endif	// DGUX || AIX || HPUX
	static CTThreadInfo&	getCurrentThread (void) ;

//-----------------------------------------------------------------------------
private:					// Constructor(s) and destructor.
	CTPublicCurrentThread () ;
	CTPublicCurrentThread (const CTPublicCurrentThread&) ;

private:					// Operators.
	void*					operator new (size_t) ;
	void					operator delete (void*) ;
	CTPublicCurrentThread&	operator= (const CTPublicCurrentThread&) ;

//-----------------------------------------------------------------------------
} ; // CTPublicCurrentThread

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////// End of Class CTPublicCurrentThread //
///////////////////////////////////////////////////////////////////////////////


//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////// Class CTThreadInfo //
///////////////////////////////////////////////////////////////////////////////
// Base class for the threads.

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTThreadInfo::CTThreadInfo
//
// Default constructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
CTThreadInfo::CTThreadInfo ()
: pFunctorStartRoutine_ (NULL), bAbort_ (false), pAbortRoutine_ (NULL), uSuspendCount_ (0), eState_ (THREAD_CREATED), eInitState_ (INIT_ZERO), pData_ (NULL), ePriority_ (PRIORITY_NORMAL), pKillStatus_ (NULL), ppThreadInfoMember_ (NULL)
#if defined(HPUX)
, condResume_ (mutexInfo_)
#endif	// HPUX
{
	// Lock the informations.
	CTLockGuard<CTMutex>	guard (mutexInfo_) ;

#ifdef	WIN32
	uThreadId_ = GetCurrentThreadId () ;
	ulThread_ = 0 ;
	// Register the pointer of the thread object (so we'll can get the current thread object from anywhere).
	if (!TlsSetValue (GdwTLSIndex, this) )
	{	// Error from TlsSetValue ().
		char zBuf[255] ;
 		FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
		CharToOem (zBuf, zBuf) ;
		throw CTThreadException (string ("CTThreadInfo: TlsSetValue failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
	}
#endif	// WIN32
#if	defined (SUN)
	threadId_ = pthread_self () ;
	if (GptThreadKey == NULL)
	{
		// Create the data key.
		if (pthread_key_create (&GptThreadKey, (void (*)(void *))CTPublicCurrentThread::threadDataDestructor) == -1)
		{	// Error from pthread_key_create ().
			throw CTThreadException (string ("CTThreadInfo: pthread_key_create failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
		}
	}
	// Register the pointer of the thread object (so we'll can get the current thread object from anywhere).
	if (pthread_setspecific (GptThreadKey, this) == -1)
	{	// Error from pthread_setspecific ().
		throw CTThreadException (string ("CTThreadInfo: pthread_setspecific failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
	++ GiNbThreads ;
#endif	// SUN
#if	defined (DGUX) || defined (AIX)
	threadId_ = pthread_self () ;
	if (GptThreadKey == NULL)
	{
		// Create the data key.
		if (pthread_key_create (&GptThreadKey, CTPublicCurrentThread::threadDataDestructor) == -1)
		{	// Error from pthread_key_create ().
			throw CTThreadException (string ("CTThreadInfo: pthread_key_create failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
		}
	}
	// Register the pointer of the thread object (so we'll can get the current thread object from anywhere).
	if (pthread_setspecific (GptThreadKey, this) == -1)
	{	// Error from pthread_setspecific ().
		throw CTThreadException (string ("CTThreadInfo: pthread_setspecific failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
	++ GiNbThreads ;
#endif	// DGUX || AIX
#ifdef	HPUX
	threadId_ = pthread_self () ;
	if (GptThreadKey == NULL)
	{
		// Create the data key.
		if (pthread_keycreate (&GptThreadKey, (pthread_destructor_t) CTPublicCurrentThread::threadDataDestructor) == -1)
		{	// Error from pthread_keycreate ().
			throw CTThreadException (string ("CTThreadInfo: pthread_keycreate failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
		}
	}
	// Register the pointer of the thread object (so we'll can get the current thread object from anywhere).
	if (pthread_setspecific (GptThreadKey, this) == -1)
	{	// Error from pthread_setspecific ().
		throw CTThreadException (string ("CTThreadInfo: pthread_setspecific failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
#endif	// HPUX
} // CTThreadInfo::CTThreadInfo

//-----------------------------------------------------------------------------
// CTThreadInfo::~CTThreadInfo
//
// Donstructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
CTThreadInfo::~CTThreadInfo ()
{
	CTLockGuard<CTMutex>	guard (mutexInfo_) ;

	// Set the pointer to thread object's thread information member to NULL.
	if (ppThreadInfoMember_)
	{
		*ppThreadInfoMember_ = NULL ;
	}
	if (pFunctorStartRoutine_)
	{
		delete pFunctorStartRoutine_ ;
	}
#ifdef	WIN32
	// Unregister the pointer of the thread object (so we can't get the current thread object from anywhere).
	if (!TlsSetValue (GdwTLSIndex, NULL) )
	{	// Error from TlsSetValue ().
		char zBuf[255] ;
 		FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
		CharToOem (zBuf, zBuf) ;
		throw CTThreadException (string ("~CTThreadInfo: TlsSetValue failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
	}
#endif	// WIN32
#if	defined (DGUX) || defined (AIX) || defined (SUN)
	// Unregister the pointer of the thread object (so we can't get the current thread object from anywhere).
	if (pthread_setspecific (GptThreadKey, NULL) == -1)
	{	// Error from pthread_setspecific ().
		throw CTThreadException (string ("~CTThreadInfo: pthread_setspecific failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
	-- GiNbThreads ;
	if (GiNbThreads == 0)
	{
		pthread_key_delete (GptThreadKey) ;
		GptThreadKey = NULL ;
	}
#endif	// DGUX || AIX
#ifdef	HPUX
	// Unregister the pointer of the thread object (so we can't get the current thread object from anywhere).
	if (pthread_setspecific (GptThreadKey, NULL) == -1)
	{	// Error from pthread_setspecific ().
		throw CTThreadException (string ("~CTThreadInfo: pthread_setspecific failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
#endif	// HPUX
} // CTThreadInfo::~CTThreadInfo

//=========================================================< Control methodes >
//-----------------------------------------------------------------------------
// CTThreadInfo::parentInitializeSize
//
// Initialize size of the new thread by the parent.
//
// ppThreadInfoMember  : Pointer to thread object's thread information member.
// functorStartRoutine : Functor representing the start routine.
// pAbortRoutine       : The abort routine.
// eInitialState       : Initial state.
// ePriority           : Priority.
// Return              : none.
//-----------------------------------------------------------------------------
void CTThreadInfo::parentInitializeSize (void** ppThreadInfoMember, const CTFunctor& functorStartRoutine, fnThreadAbortRoutine pAbortRoutine, enumThreadState eInitialState, enumThreadPriority ePriority)
{
	// Suspend the thread.
#ifdef	WIN32
	if (SuspendThread ( (HANDLE) ulThread_) == (DWORD) -1)
	{	// Error from SuspendThread ().
		char zBuf[255] ;
 		FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
		CharToOem (zBuf, zBuf) ;
		throw CTThreadException (string ("parentInitializeSize: SuspendThread failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
	}
#endif	// WIN32
#if	defined (SUN)

/*	if (pthread_kill (threadId_, SIGTSTP) != 0)
	{	// Error from pthread_kill ().
		throw CTThreadException (string ("parentInitializeSize: pthread_kill failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
*/
#endif	// DGUX || AIX
#if	defined (DGUX) || defined (AIX)
	if (pthread_kill (threadId_, SIGSTOP) == -1)
	{	// Error from pthread_kill ().
		throw CTThreadException (string ("parentInitializeSize: pthread_kill failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
#endif	// DGUX || AIX
	uSuspendCount_ = 1 ;
#if defined(HPUX)
	// Hope to pass the hand to the thread.
	thread_yield () ;
#endif	// HPUX

	// Set members values.
	try
	{
		pFunctorStartRoutine_	= new CTFunctor (functorStartRoutine) ;
	}
	catch (CTxalloc&)
	{
		pFunctorStartRoutine_	= NULL ;
	}
	pAbortRoutine_				= pAbortRoutine ;
	ppThreadInfoMember_			= ppThreadInfoMember ;
	eState_						= eInitialState ;
	eInitState_					= INIT_PARTIAL ;

	if (eInitialState == THREAD_RUNNING)
	{
		// Resume the thread.
#ifdef	WIN32
		if (ResumeThread ( (HANDLE) ulThread_) == (DWORD) -1)
		{	// Error from ResumeThread ().
			char zBuf[255] ;
 			FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
			CharToOem (zBuf, zBuf) ;
			throw CTThreadException (string ("parentInitializeSize: ResumeThread failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
		}
#endif	// WIN32
#if defined (SUN)
/*
		if (pthread_kill (threadId_, SIGCONT) != 0)
		{	// Error from pthread_kill ().
			throw CTThreadException (string ("parentInitializeSize: pthread_kill failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
		}
*/
#endif // SUN
#if	defined (DGUX) || defined (AIX) 
		if (pthread_kill (threadId_, SIGCONT) == -1)
		{	// Error from pthread_kill ().
			throw CTThreadException (string ("parentInitializeSize: pthread_kill failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
		}
#endif	// DGUX || AIX
#ifdef	HPUX
		condResume_.signal () ;
#endif	// HPUX
		uSuspendCount_ = 0 ;
		while (eInitState_ != INIT_TOTAL)
		{
			thread_yield () ;
		}
		eInitState_ = INIT_END ;
	}

	// Set the priority.
	setPriority (ePriority) ;
} // CTThreadInfo::parentInitializeSize

//-----------------------------------------------------------------------------
// CTThreadInfo::suspend
//
// Suspend the thread.
//
// Arguments : none.
// Return    : uSuspendCount_ : Suspend count of the thread.
//-----------------------------------------------------------------------------
unsigned int CTThreadInfo::suspend (void)
{
	// Verify the thread state.
	if ( (eState_ != THREAD_RUNNING) && (eState_ != THREAD_SUSPENDED) )
	{	// Error from eState_.
		throw CTThreadException ("suspend: bad thread state (must be THREAD_RUNNING or THREAD_SUSPENDED).", __FILE__, __LINE__) ;
	}

	// All is ok, suspend.
	if (uSuspendCount_ == 0)
	{
#ifdef	WIN32
		if (SuspendThread ( (HANDLE) ulThread_) == (DWORD) -1)
		{	// Error from SuspendThread ().
			char zBuf[255] ;
 			FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
			CharToOem (zBuf, zBuf) ;
			throw CTThreadException (string ("suspend: SuspendThread failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
		}
#endif	// WIN32
#if	defined (DGUX) || defined (AIX) || defined(SUN)
		if (pthread_kill (threadId_, SIGSTOP) == -1)
		{	// Error from pthread_kill ().
			throw CTThreadException (string ("suspend: pthread_kill failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
		}
#endif	// DGUX || AIX || SUN
		eState_ = THREAD_SUSPENDED ;
	}

	// Incrementation of the suspend counter.
	++ uSuspendCount_ ;
#ifdef	HPUX
	// Hope to pass the hand to the thread.
	thread_yield () ;
#endif	// HPUX

	// Return the value of the suspend counter.
	return uSuspendCount_ ;
} // CTThreadInfo::suspend

//-----------------------------------------------------------------------------
// CTThreadInfo::resume
//
// Resume the thread.
//
// Arguments : none.
// Return    : uSuspendCount_ : Suspend count of the thread.
//-----------------------------------------------------------------------------
unsigned int CTThreadInfo::resume (void)
{
	// Verify the thread state.
	if ( (eState_ != THREAD_RUNNING) && (eState_ != THREAD_SUSPENDED) )
	{	// Error from eState_.
		throw CTThreadException ("resume: bad thread state (must be THREAD_RUNNING or THREAD_SUSPENDED).", __FILE__, __LINE__) ;
	}

	// All is ok, resume.
	if (uSuspendCount_ == 1)
	{
#ifdef	WIN32
		if (ResumeThread ( (HANDLE) ulThread_) == (DWORD) -1)
		{	// Error from ResumeThread ().
			char zBuf[255] ;
 			FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
			CharToOem (zBuf, zBuf) ;
			throw CTThreadException (string ("resume: ResumeThread failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
		}
#endif	// WIN32
#if	defined (DGUX) || defined (AIX)
		if (pthread_kill (threadId_, SIGCONT) == -1)
		{	// Error from pthread_kill ().
			throw CTThreadException (string ("resume: pthread_kill failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
		}
#endif	// DGUX || AIX
#ifdef	HPUX
		condResume_.signal () ;
#endif	// HPUX
		eState_ = THREAD_RUNNING ;
		if (eInitState_ == INIT_PARTIAL)
		{
			while (eInitState_ != CTThreadInfo::INIT_TOTAL)
			{
				thread_yield () ;
			}
			eInitState_ = CTThreadInfo::INIT_END ;
		}
		else if (eInitState_ == CTThreadInfo::INIT_TOTAL)
		{
			eInitState_ = CTThreadInfo::INIT_END ;
		}
	}

	// Decrementation of the suspend counter.
	if (uSuspendCount_ > 0)
	{
		-- uSuspendCount_ ;
	}

	// Return the value of the suspend counter.
	return uSuspendCount_ ;
} // CTThreadInfo::resume

//-----------------------------------------------------------------------------
// CTThreadInfo::kill
//
// Kill the thread and specify its exit status.
//
// pStatus : exit status when aborting.
// Return  : none.
//-----------------------------------------------------------------------------
void CTThreadInfo::kill (void* pStatus /* = NULL */)
{
	// Verify the thread state.
	if ( (eState_ != THREAD_RUNNING) && (eState_ != THREAD_SUSPENDED) )
	{	// Error from eState_.
		throw CTThreadException ("kill: bad thread state (must be THREAD_RUNNING or THREAD_SUSPENDED).", __FILE__, __LINE__) ;
	}

	// Resume the thread if necessary.
	while (eState_ == THREAD_SUSPENDED)
	{
		resume () ;
	}

	// Specify that thread is killed.
	pKillStatus_ = pStatus ;
	eState_ = THREAD_KILLED ;

	// Kill the thread.
#ifdef	WIN32
	if (SuspendThread ( (HANDLE) ulThread_) == (DWORD) -1)
	{	// Error from SuspendThread ().
		char zBuf[255] ;
 		FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
		CharToOem (zBuf, zBuf) ;
		throw CTThreadException (string ("kill: SuspendThread failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
	}

	// Get the thread's current CPU registers.
	CONTEXT	context ;
	context.ContextFlags = CONTEXT_CONTROL ;

	if (!GetThreadContext ( (HANDLE) ulThread_, &context) )
	{	// Error from GetThreadContext ().
		char zBuf[255] ;
 		FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
		CharToOem (zBuf, zBuf) ;
		throw CTThreadException (string ("kill: GetThreadContext failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
	}

	// Change the instruction pointer to the clean-up function.
#ifdef	_X86_
	context.Eip = (DWORD) CTPublicCurrentThread::threadCleanupRoutine ;
#endif	// _X86_
#ifdef	_ALPHA_
	context.Fir = (DWORD) CTPublicCurrentThread::threadCleanupRoutine ;
#endif	// _X86_

	if (!SetThreadContext ( (HANDLE) ulThread_, &context) )
	{	// Error from SetThreadContext ().
		char zBuf[255] ;
 		FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
		CharToOem (zBuf, zBuf) ;
		throw CTThreadException (string ("kill: SetThreadContext failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
	}

	if (ResumeThread ( (HANDLE) ulThread_) == (DWORD) -1)
	{	// Error from ResumeThread ().
		char zBuf[255] ;
 		FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
		CharToOem (zBuf, zBuf) ;
		throw CTThreadException (string ("kill: ResumeThread failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
	}
#endif	// WIN32
#if	defined (DGUX) || defined (AIX) || defined (HPUX) || defined (SUN)
	if (pthread_cancel (threadId_) == -1)
	{	// Error from pthread_cancel ().
		throw CTThreadException (string ("kill: pthread_cancel failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
#endif	// DGUX || AIX || HPUX
} // CTThreadInfo::kill

//-----------------------------------------------------------------------------
// CTThreadInfo::setPriority
//
// Set the priority of the thread.
//
// ePriority : Priority to set to the thread.
// Return    : none.
//-----------------------------------------------------------------------------
void CTThreadInfo::setPriority (enumThreadPriority ePriority)
{
	// Verify the thread state.
	if ( (eState_ != THREAD_RUNNING) && (eState_ != THREAD_SUSPENDED) )
	{	// Error from eState_.
		throw CTThreadException ("setPriority: bad thread state (must be THREAD_RUNNING or THREAD_SUSPENDED).", __FILE__, __LINE__) ;
	}

	// Set the priority to the thread.
#ifdef	WIN32
	int	iPriority ;
	// Determinate the priority value.
	switch (ePriority)
	{
		case PRIORITY_HIGH :
			iPriority = THREAD_PRIORITY_HIGHEST ;
			break ;
		case PRIORITY_NORMAL :
			iPriority = THREAD_PRIORITY_NORMAL ;
			break ;
		case PRIORITY_LOW :
			iPriority = THREAD_PRIORITY_LOWEST ;
			break ;
		case PRIORITY_ERROR :
		default :
			iPriority = -1 ;
			break ;
	}
	// Set the thread's priority.
	if (!SetThreadPriority ( (HANDLE) ulThread_, iPriority) )
	{	// Error from SetThreadPriority ().
		char zBuf[255] ;
 		FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
		CharToOem (zBuf, zBuf) ;
		throw CTThreadException (string ("setPriority: SetThreadPriority failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
	}
#endif	// WIN32
#ifdef	DGUX
	// Get the thread's attributes.
	pthread_attr_t attributes ;
	if (pthread_getschedattr (threadId_, &attributes) == -1)
	{	// Error from pthread_getschedattr ().
		throw CTThreadException (string ("setPriority: pthread_getschedattr failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
	// Get the scheduler.
	int iScheduler = pthread_attr_getsched (&attributes) ;
	if (iScheduler == -1)
	{	// Error from pthread_attr_getsched ().
		throw CTThreadException (string ("setPriority: pthread_attr_getsched failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
	// Get the minimum and maximum priority values.
	int iMinPriority = sched_get_priority_min (iScheduler) ;
	if (iMinPriority == -1)
	{	// Error from sched_get_priority_min ().
		throw CTThreadException (string ("setPriority: sched_get_priority_min failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
	int iMaxPriority = sched_get_priority_max (iScheduler) ;
	if (iMaxPriority == -1)
	{	// Error from sched_get_priority_max ().
		throw CTThreadException (string ("setPriority: sched_get_priority_max failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
	// Calculate the high, normal and low priority values.
	int iNormalPriority	= (iMaxPriority + iMinPriority + 1) / 2 ;
	int iHighPriority	= (iMaxPriority + iNormalPriority + 1) / 2 ;
	int iLowPriority	= (iMinPriority + iNormalPriority + 1) / 2 ;
	// Determinate the priority value.
	int iPriority ;
	switch (ePriority)
	{
		case PRIORITY_HIGH :
			iPriority = iHighPriority ;
			break ;
		case PRIORITY_NORMAL :
			iPriority = iNormalPriority ;
			break ;
		case PRIORITY_LOW :
			iPriority = iLowPriority ;
			break ;
		case PRIORITY_ERROR :
		default :
			iPriority = -1 ;
			break ;
	}
	// Set the attributes priority field.
	if (pthread_attr_setprio (&attributes, iPriority) == -1)
	{	// Error from pthread_attr_setprio ().
		throw CTThreadException (string ("setPriority: pthread_attr_setprio failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
	// Set the thread's attributes.
	if (pthread_setschedattr (threadId_, &attributes) == -1)
	{	// Error from pthread_setschedattr ().
		throw CTThreadException (string ("setPriority: pthread_setschedattr failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
#endif	// DGUX
#if defined(AIX)
	// Get the thread's scheduler and its parameters.
	int					iScheduler ;
	struct sched_param	schedulerParameters;
	if (pthread_getschedparam (threadId_, &iScheduler, &schedulerParameters) == -1)
	{	// Error from pthread_getschedparam ().
		throw CTThreadException (string ("setPriority: pthread_getschedparam failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
	// Calculate the high, normal and low priority values.
	int iNormalPriority	= (PRIORITY_MAX + PRIORITY_MIN + 1) / 2 ;
	int iHighPriority	= (PRIORITY_MAX + iNormalPriority + 1) / 2 ;
	int iLowPriority	= (PRIORITY_MIN + iNormalPriority + 1) / 2 ;
	// Determinate the priority value.
	switch (ePriority)
	{
		case PRIORITY_HIGH :
			schedulerParameters.sched_priority = iHighPriority ;
			break ;
		case PRIORITY_NORMAL :
			schedulerParameters.sched_priority = iNormalPriority ;
			break ;
		case PRIORITY_LOW :
			schedulerParameters.sched_priority = iLowPriority ;
			break ;
		case PRIORITY_ERROR :
		default :
			iPriority = -1 ;
			break ;
	}
	// Set the scheduler's parameters of the thread.
	if (pthread_setschedparam (threadId_, iScheduler, &schedulerParameters) == -1)
	{	// Error from pthread_setschedparam ().
		throw CTThreadException (string ("setPriority: pthread_setschedparam failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
#endif	// AIX
#ifdef	HPUX
	// Get the thread's scheduler.
	int iScheduler = pthread_getscheduler (threadId_) ;
	if (iScheduler == -1)
	{	// Error from pthread_getscheduler ().
		throw CTThreadException (string ("setPriority: pthread_getscheduler failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
	// Get the minimum and maximum priority values.
	int	iMinPriority ;
	int	iMaxPriority ;
	switch (iScheduler)
	{
		case SCHED_FIFO:
			iMinPriority = PRI_FIFO_MIN ;
			iMaxPriority = PRI_FIFO_MAX ;
			break ;
		case SCHED_RR:
			iMinPriority = PRI_RR_MIN ;
			iMaxPriority = PRI_RR_MAX ;
			break ;
		case SCHED_FG_NP:
			iMinPriority = PRI_FG_MIN_NP ;
			iMaxPriority = PRI_FG_MAX_NP ;
			break ;
		case SCHED_BG_NP:
			iMinPriority = PRI_BG_MIN_NP ;
			iMaxPriority = PRI_BG_MAX_NP ;
			break ;
	}
	// Calculate the normal priority value.
	int iNormalPriority	= (iMaxPriority + iMinPriority + 1) / 2 ;
	// Determinate the priority value.
	int iPriority ;
	switch (ePriority)
	{
		case PRIORITY_HIGH :
			iPriority = iMaxPriority ;
			break ;
		case PRIORITY_NORMAL :
			iPriority = iNormalPriority ;
			break ;
		case PRIORITY_LOW :
			iPriority = iMinPriority ;
			break ;
		case PRIORITY_ERROR :
		default :
			iPriority = -1 ;
			break ;
	}
	// Set the attributes priority field.
	if (pthread_setscheduler (threadId_, iScheduler, iPriority) == -1)
	{	// Error from pthread_setscheduler ().
		throw CTThreadException (string ("setPriority: pthread_setscheduler failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
#endif	// HPUX

	// Save the priority of the thread.
	ePriority_ = ePriority ;
} // CTThreadInfo::setPriority

//-----------------------------------------------------------------------------
// CTThreadInfo::abort
//
// Abort the thread.
//
// Arguments : pStatus.
// Return    : none.
//-----------------------------------------------------------------------------
void CTThreadInfo::abort (void* pStatus /* = NULL */)
{
	// Verify the state of the thread.
	if ( (eState_ != THREAD_RUNNING) && (eState_ != THREAD_KILLED) )
	{	// Error from eState_.
		throw CTThreadException ("abort: bad thread state (must be THREAD_RUNNING or THREAD_KILLED).", __FILE__, __LINE__) ;
	}

	// Thread is terminated.
	eState_ = THREAD_TERMINATED ;

	// If any, call the user abort routine.
	if (pAbortRoutine_)
	{
		pAbortRoutine_ (pStatus) ;
		pAbortRoutine_ = NULL ;
	}

	// Prevent the joined threads.
	void**	ppStatus ;
	while (!statusPointerList_.empty () )
	{
		ppStatus = &(statusPointerList_.front()) ;
		statusPointerList_.pop_front () ;
		*ppStatus = pStatus ;
	}

	// Destroy the objects and abort the thread.
#ifdef	WIN32
	unsigned long ulThread = ulThread_ ;
	// Destroy the thread information object.
	delete this ;
	// Free the system resources associated to the thread.
	CloseHandle ( (HANDLE) ulThread) ;
	// Exit the thread with the status code.
	_endthreadex ( (unsigned) pStatus) ;
#endif	// WIN32
#if	defined (DGUX) || defined (AIX) || defined (HPUX) || defined (SUN)
	// Destroy the thread information object.
	delete this ;
	// Exit the thread with the status code.
	pthread_exit (pStatus) ;
#endif	// DGUX || AIX || HPUX
} // CTThreadInfo::abort

//-----------------------------------------------------------------------------
// CTThreadInfo::sleep
//
// Make the thread sleeping.
//
// ulMilliseconds : Number of milliseconds to sleep.
// Return         : none.
//-----------------------------------------------------------------------------
void CTThreadInfo::sleep (unsigned long ulMilliseconds /* = 0L */)
{
	// Verify the state of the thread.
	if ( (eState_ != THREAD_RUNNING) && (eState_ != THREAD_KILLED) )
	{	// Error from eState_.
		throw CTThreadException ("sleep: bad thread state (must be THREAD_RUNNING or THREAD_KILLED).", __FILE__, __LINE__) ;
	}

	if (eState_ == THREAD_RUNNING)
	{
		// Make the thread sleep.
#ifdef	WIN32
		::Sleep ( (DWORD) ulMilliseconds) ;
#endif	// WIN32
#if defined(DGUX)
		::usleep (ulMilliseconds) ;
#endif	// DGUX
#if defined(SUN)
		::usleep (ulMilliseconds* 1000) ;
#endif	// DGUX
#if	defined (AIX) || defined (HPUX)
		struct timespec	interval ;
		interval.tv_sec = (int) (ulMilliseconds / 1000) ;
		interval.tv_nsec = (ulMilliseconds % 1000) * 1000 * 1000 ;
		::pthread_delay_np (&interval) ;
#endif	// AIX || HPUX
	}
	else
	{
		abort (pKillStatus_) ;
	}
} // CTThreadInfo::sleep

//-----------------------------------------------------------------------------
// CTThreadInfo::join
//
// Wait for the termination of the thread.
//
// ppStatus : Points to a pointer to the termination status of the thread (or null).
// Return   : none.
//-----------------------------------------------------------------------------
void CTThreadInfo::join (void** ppStatus /* = NULL */)
{
	// Verify the thread state.
	if ( (eState_ != THREAD_RUNNING) && (eState_ != THREAD_SUSPENDED) && (eState_ != THREAD_KILLED) && (eState_ != THREAD_TERMINATED) )
	{	// Error from eState_.
		throw CTThreadException ("join: bad thread state (must be THREAD_RUNNING or THREAD_SUSPENDED or THREAD_KILLED or THREAD_TERMINATED).", __FILE__, __LINE__) ;
	}
	// Specify the thread we want its status.
	if (ppStatus)
	{
		statusPointerList_.push_back (ppStatus) ;
	}

	// Wait for the thread termination.
#ifdef	WIN32
	unsigned long	ulThread = ulThread_ ;
	mutexInfo_.unlock () ;
	if (WaitForSingleObject ( (HANDLE) ulThread, INFINITE) == (DWORD) WAIT_FAILED)
	{	// Error from WaitForSingleObject ().
		char zBuf[255] ;
 		FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
		CharToOem (zBuf, zBuf) ;
		throw CTThreadException (string ("join: WaitForSingleObject failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
	}
#endif	// WIN32
#if	defined (SUN)
	pthread_t	threadId = threadId_ ;
	mutexInfo_.unlock () ;
	if (pthread_join (threadId, NULL) != 0)
	{	// Error from pthread_join ().
		throw CTThreadException (string ("join: pthread_join failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}

#endif	// SUN
#if	defined (DGUX) || defined (AIX) || defined (HPUX)
	pthread_t	threadId = threadId_ ;
	mutexInfo_.unlock () ;
	if (pthread_join (threadId, NULL) == -1)
	{	// Error from pthread_join ().
		throw CTThreadException (string ("join: pthread_join failed: ") + strerror (errno), __FILE__, __LINE__, errno) ;
	}
#endif	// DGUX || AIX || HPUX
} // CTThreadInfo::join

///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////// End of Class CTThreadInfo //
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////// Class CTThread //
///////////////////////////////////////////////////////////////////////////////
// Class used to create and control others threads than the current thread.

//============================================< Constructor(s) and destructor >
//-----------------------------------------------------------------------------
// CTThread::~CTThread
//
// Destructor.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
CTThread::~CTThread ()
{
	if (pThreadInfo_)
	{
		// Get a reference to the thread information object.
		CTThreadInfo& threadInfo = *( (CTThreadInfo*) pThreadInfo_) ;

		if (threadInfo.ppThreadInfoMember_ == &pThreadInfo_)
		{	// Set the pointer of the thread object's thread information member to NULL.
			threadInfo.ppThreadInfoMember_ = NULL ;
		}
	}
} // CTThread::~CTThread

//=============================================================< Get methodes >
//-----------------------------------------------------------------------------
// CTThread::getState
//
// Get the state of the thread.
//
// Arguments : none.
// Return    : threadInfo.eState_ : State of the thread.
//-----------------------------------------------------------------------------
const enumThreadState CTThread::getState (void) const
{
	enumThreadState	eState ;

	if (pThreadInfo_ == NULL)
	{
		// Thread must be terminated.
		eState = THREAD_TERMINATED ;
	}
	else
	{
		// Get a reference to the thread information object.
		CTThreadInfo& threadInfo = *( (CTThreadInfo*) pThreadInfo_) ;

		// Get the state of the thread.
		eState = threadInfo.eState_ ;
	}

	// Return the state of the thread.
	return eState ;
} // CTThread::getState

//-----------------------------------------------------------------------------
// CTThread::getPriority
//
// Get the priority of the thread.
//
// Arguments : none.
// Return    : threadInfo.ePriority_ : Priority of the thread.
//-----------------------------------------------------------------------------
const enumThreadPriority CTThread::getPriority (void) const
{
	enumThreadPriority	ePriority ;

	if (pThreadInfo_ == NULL)
	{
		// Thread must be terminated.
		ePriority = PRIORITY_ERROR ;
	}
	else
	{
		// Get a reference to the thread information object.
		CTThreadInfo& threadInfo = *( (CTThreadInfo*) pThreadInfo_) ;

		// Get the priority of the thread.
		ePriority = threadInfo.ePriority_ ;
	}

	// Return the priority of the thread.
	return ePriority ;
} // CTThread::getPriority

//=========================================================< Control methodes >
//-----------------------------------------------------------------------------
// CTThread::create
//
// Create the new thread.
//
// functorStartRoutine : Functor representing the start routine.
// pAbortRoutine          : Functor representing the abort routine.
// eInitialState          : Initial state.
// ePriority              : Level of priority.
// Return                 : none.
//-----------------------------------------------------------------------------
void CTThread::create (const CTFunctor& functorStartRoutine, fnThreadAbortRoutine pAbortRoutine, enumThreadState eInitialState, enumThreadPriority ePriority)
{
	if ( (eInitialState != THREAD_RUNNING) && (eInitialState != THREAD_SUSPENDED) )
	{	// Error from eInitialState.
		throw CTThreadException ("create: eInitialState is not valid (must be THREAD_RUNNING or THREAD_SUSPENDED).", __FILE__, __LINE__) ;
	}
	if ( (ePriority != PRIORITY_HIGH) && (ePriority != PRIORITY_NORMAL) && (ePriority != PRIORITY_LOW) )
	{	// Error from ePriority.
		throw CTThreadException ("create: ePriority is not valid (must be PRIORITY_HIGH or PRIORITY_NORMAL or PRIORITY_LOW).", __FILE__, __LINE__) ;
	}

	// Create the new thread.
#ifdef	WIN32
	unsigned int	uThreadAddress ;
	unsigned long	ulThread ;
	ulThread = _beginthreadex (NULL, 0, (CTPublicCurrentThread::fnStartRoutine_t) CTPublicCurrentThread::threadStartRoutine, &pThreadInfo_, 0, &uThreadAddress) ;
	if (ulThread == -1)
	{	// Error from _beginthreadex ().
		throw CTThreadException (string ("create: _beginthreadex failed: ") + strerror (errno) , __FILE__, __LINE__, errno) ;
	}
#endif	// WIN32
#ifdef	DGUX
	// Initialize the thread's attributes.
	pthread_attr_t	attributes ;
	if (pthread_attr_init (&attributes) == -1)
	{	// Error from pthread_attr_init ().
		throw CTThreadException (string ("create: pthread_attr_init failed: ") + strerror (errno) , __FILE__, __LINE__, errno) ;
	}
	pthread_t threadId ;
	if (pthread_create (&threadId, &attributes, (CTPublicCurrentThread::fnStartRoutine_t) CTPublicCurrentThread::threadStartRoutine, &pThreadInfo_) == -1)
	{	// Error from pthread_create ().
		throw CTThreadException (string ("create: pthread_create failed: ") + strerror (errno) , __FILE__, __LINE__, errno) ;
	}
	if (pthread_attr_destroy (&attributes) == -1)
	{	// Error from pthread_attr_destroy ().
		throw CTThreadException (string ("create: pthread_attr_destroy failed: ") + strerror (errno) , __FILE__, __LINE__, errno) ;
	}
#endif	// DGUX
#if defined(AIX)  || defined (SUN)
	// Initialize the thread's attributes.
	pthread_attr_t	attributes ;
	if (pthread_attr_init (&attributes) == -1)
	{	// Error from pthread_attr_init ().
		throw CTThreadException (string ("create: pthread_attr_init failed: ") + strerror (errno) , __FILE__, __LINE__, errno) ;
	}
	pthread_t threadId ;
	if (pthread_create (&threadId, &attributes, (CTPublicCurrentThread::fnStartRoutine_t) CTPublicCurrentThread::threadStartRoutine, &pThreadInfo_) == -1)
	{	// Error from pthread_create ().
		throw CTThreadException (string ("create: pthread_create failed: ") + strerror (errno) , __FILE__, __LINE__, errno) ;
	}
	if (pthread_attr_destroy (&attributes) == -1)
	{	// Error from pthread_attr_destroy ().
		throw CTThreadException (string ("create: pthread_attr_destroy failed: ") + strerror (errno) , __FILE__, __LINE__, errno) ;
	}
#endif	// AIX
#ifdef	HPUX
	// Initialize the thread's attributes.
	pthread_attr_t	attributes ;
	if (pthread_attr_create (&attributes) == -1)
	{	// Error from pthread_attr_create ().
		throw CTThreadException (string ("create: pthread_attr_create failed: ") + strerror (errno) , __FILE__, __LINE__, errno) ;
	}
	pthread_t threadId ;
	if (pthread_create (&threadId, attributes, (CTPublicCurrentThread::fnStartRoutine_t) CTPublicCurrentThread::threadStartRoutine, &pThreadInfo_) == -1)
	{	// Error from pthread_create ().
		throw CTThreadException (string ("create: pthread_create failed: ") + strerror (errno) , __FILE__, __LINE__, errno) ;
	}
	if (pthread_attr_delete (&attributes) == -1)
	{	// Error from pthread_attr_delete ().
		throw CTThreadException (string ("create: pthread_attr_delete failed: ") + strerror (errno) , __FILE__, __LINE__, errno) ;
	}
#endif	// HPUX

	// Wait for thread information object creation.
	while (pThreadInfo_ == NULL)
	{
		thread_yield () ;
	}
	if (pThreadInfo_ == (void*) -1)
	{	// Error from pThreadInfo_.
		pThreadInfo_ = NULL ;
		throw CTThreadException ("create: new in system start routine failed.", __FILE__, __LINE__) ;
	}

	// Get a reference to the thread information object.
	CTThreadInfo& threadInfo = *( (CTThreadInfo*) pThreadInfo_) ;

#ifdef	WIN32
	// Save the thread handle.
	threadInfo.ulThread_ = ulThread ;
#endif	// WIN32

	// Initialize the new thread.
	threadInfo.parentInitializeSize (&pThreadInfo_, functorStartRoutine, pAbortRoutine, eInitialState, ePriority) ;
} // CTThread::create

//-----------------------------------------------------------------------------
// CTThread::suspend
//
// Suspend the thread.
//
// Arguments : none.
// Return    : uSuspendCount : Suspend count of the thread.
//-----------------------------------------------------------------------------
unsigned int CTThread::suspend (void)
{
	unsigned int	uSuspendCount ;

	if (pThreadInfo_ == NULL)
	{
		// Thread must be terminated.
		uSuspendCount = (unsigned int) -1 ;
	}
	else
	{
		// Get a reference to the thread information object.
		CTThreadInfo& threadInfo = *( (CTThreadInfo*) pThreadInfo_) ;

		// All is ok, suspend.
		uSuspendCount = threadInfo.suspend () ;
	}

	// Return the value of the suspend counter.
	return uSuspendCount ;
} // CTThread::suspend

//-----------------------------------------------------------------------------
// CTThread::resume
//
// Resume the thread.
//
// Arguments : none.
// Return    : uSuspendCount : Suspend count of the thread.
//-----------------------------------------------------------------------------
unsigned int CTThread::resume (void)
{
	unsigned int	uSuspendCount ;

	if (pThreadInfo_ == NULL)
	{
		// Thread must be terminated.
		uSuspendCount = (unsigned int) -1 ;
	}
	else
	{
		// Get a reference to the thread information object.
		CTThreadInfo& threadInfo = *( (CTThreadInfo*) pThreadInfo_) ;

		// All is ok, resume.
		uSuspendCount = threadInfo.resume () ;
	}

	// Return the value of the suspend counter.
	return uSuspendCount ;
} // CTThread::resume

//-----------------------------------------------------------------------------
// CTThread::kill
//
// Kill the thread and specify its exit status.
//
// pStatus : exit status when aborting.
// Return  : none.
//-----------------------------------------------------------------------------
void CTThread::kill (void* pStatus /* = NULL */)
{
	if (pThreadInfo_ != NULL)
	{
		// Get a reference to the thread information object.
		CTThreadInfo& threadInfo = *( (CTThreadInfo*) pThreadInfo_) ;

		// Kill the thread.
		threadInfo.kill (pStatus) ;
	}
} // CTThread::kill

//-----------------------------------------------------------------------------
// CTThread::setPriority
//
// Set the priority of the thread.
//
// ePriority : Priority to set to the thread.
// Return    : none.
//-----------------------------------------------------------------------------
void CTThread::setPriority (enumThreadPriority ePriority)
{
	if (pThreadInfo_ != NULL)
	{
		// Get a reference to the thread information object.
		CTThreadInfo& threadInfo = *( (CTThreadInfo*) pThreadInfo_) ;

		// Verify the priority parameter.
		if ( (ePriority != PRIORITY_HIGH) && (ePriority != PRIORITY_NORMAL) && (ePriority != PRIORITY_LOW) )
		{	// Error from ePriority.
			throw CTThreadException ("setPriority: bad priority parameter (must be PRIORITY_HIGH or PRIORITY_NORMAL or PRIORITY_LOW).", __FILE__, __LINE__) ;
		}

		// Set the priority to the thread.
		threadInfo.setPriority (ePriority) ;
	}
} // CTThread::setPriority

//-----------------------------------------------------------------------------
// operator==
//
// thread1 : thread 1.
// thread2 : thread 2.
// Return  : TRUE or FALSE.
//-----------------------------------------------------------------------------
int operator== (const CTThread& thread1, const CTThread& thread2)
{
    return (thread1.getThreadId () == thread2.getThreadId () ) ;
} // operator==


///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// End of Class CTThread //
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////// Class CTCurrentThread //
///////////////////////////////////////////////////////////////////////////////
// Class used to control the current thread.

//============================================< Constructor(s) and destructor >

//=============================================================< Get methodes >
//-----------------------------------------------------------------------------
// CTCurrentThread::getThreadId
//
// Get the identifier of the thread.
//
// Arguments : none.
// Return    : pThreadInfo : Identifier of the thread.
//-----------------------------------------------------------------------------
void* CTCurrentThread::getThreadId (void)
{
	// Get the thread informations reference.
	CTThreadInfo& threadInfo = CTPublicCurrentThread::getCurrentThread () ;

	// Return the identifier of the thread.
	return &threadInfo ;
} // CTCurrentThread::getThreadId

//-----------------------------------------------------------------------------
// CTCurrentThread::getState
//
// Get the state of the thread.
//
// Arguments : none.
// Return    : threadInfo.eState_ : State of the thread.
//-----------------------------------------------------------------------------
enumThreadState CTCurrentThread::getState (void)
{
	// Get the thread informations reference.
	CTThreadInfo& threadInfo = CTPublicCurrentThread::getCurrentThread () ;

	// Return the state of the thread.
	return threadInfo.eState_ ;
} // CTCurrentThread::getState

//-----------------------------------------------------------------------------
// CTCurrentThread::getData
//
// Get the data pointer associated to the current thread.
//
// Arguments : none.
// Return    : threadInfo.pData_ : Data pointer associated to the current thread.
//-----------------------------------------------------------------------------
void* CTCurrentThread::getData (void)
{
	// Get the thread informations reference.
	CTThreadInfo& threadInfo = CTPublicCurrentThread::getCurrentThread () ;

	// Return the state of the thread.
	return threadInfo.pData_ ;
} // CTCurrentThread::getData

//-----------------------------------------------------------------------------
// CTCurrentThread::getPriority
//
// Get the priority of the current thread.
//
// Arguments : none.
// Return    : threadInfo.ePriority_ : Priority of the thread.
//-----------------------------------------------------------------------------
enumThreadPriority CTCurrentThread::getPriority (void)
{
	// Get the thread informations reference.
	CTThreadInfo& threadInfo = CTPublicCurrentThread::getCurrentThread () ;

	// Return the priority of the thread.
	return threadInfo.ePriority_ ;
} // CTCurrentThread::getPriority

//=========================================================< Control methodes >
//-----------------------------------------------------------------------------
// CTCurrentThread::setAbortRoutine
//
// Set the user abort routine.
//
// pAbortRoutine : Abort routine.
// Return        : none.
//-----------------------------------------------------------------------------
void CTCurrentThread::setAbortRoutine (fnThreadAbortRoutine pAbortRoutine)
{
	// Get the thread informations reference.
	CTThreadInfo& threadInfo = CTPublicCurrentThread::getCurrentThread () ;

	// Set the user abort routine.
	threadInfo.pAbortRoutine_ = pAbortRoutine ;
} // CTCurrentThread::setAbortRoutine

//-----------------------------------------------------------------------------
// CTCurrentThread::setData
//
// Associate a data pointer to the current thread.
//
// pData  : Data pointer to associate to the current thread.
// Return : none.
//-----------------------------------------------------------------------------
void CTCurrentThread::setData (void* pData)
{
	// Get the thread informations reference.
	CTThreadInfo& threadInfo = CTPublicCurrentThread::getCurrentThread () ;

	// Set the user abort routine.
	threadInfo.pData_ = pData ;
} // CTCurrentThread::setData

//-----------------------------------------------------------------------------
// CTCurrentThread::abort
//
// Abort the thread.
//
// Arguments : pStatus.
// Return    : none.
//-----------------------------------------------------------------------------
void CTCurrentThread::abort (void* pStatus /* = NULL */)
{
	// Get the thread informations reference.
	CTThreadInfo& threadInfo = CTPublicCurrentThread::getCurrentThread () ;

	// Abort the thread.
	threadInfo.abort (pStatus) ;
} // CTCurrentThread::abort

//-----------------------------------------------------------------------------
// CTCurrentThread::sleep
//
// Make the thread sleeping.
//
// ulMilliseconds : Number of milliseconds to sleep.
// Return         : none.
//-----------------------------------------------------------------------------
void CTCurrentThread::sleep (unsigned long ulMilliseconds /* = 0L */)
{
	// Get the thread informations reference.
	CTThreadInfo& threadInfo = CTPublicCurrentThread::getCurrentThread () ;

	// Sleep.
	threadInfo.sleep (ulMilliseconds) ;
} // CTCurrentThread::sleep

//-----------------------------------------------------------------------------
// CTCurrentThread::join
//
// Wait for the termination of the specified thread.
//
// thread   : reference to a thread.
// ppStatus : Points to a pointer to the termination status of the thread (or null).
// Return   : none.
//-----------------------------------------------------------------------------
void CTCurrentThread::join (CTThread& thread, void** ppStatus /* = NULL */)
{
	// Get the current thread informations reference.
	CTThreadInfo& currentThreadInfo = CTPublicCurrentThread::getCurrentThread () ;

	// Get the target thread informations pointer.
	CTThreadInfo* pTargetThreadInfo = (CTThreadInfo*) thread.getThreadId () ;

	if (pTargetThreadInfo != NULL)
	{
		// Lock the thread info.
		pTargetThreadInfo->mutexInfo_.lock () ;

		// Get a reference to the target thread information object.
		CTThreadInfo& targetThreadInfo = *( (CTThreadInfo*) pTargetThreadInfo) ;

		// Join the target thread.
		targetThreadInfo.join (ppStatus) ;
	}
} // CTCurrentThread::join

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////// End of Class CTCurrentThread //
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////// Class CTPublicCurrentThread //
///////////////////////////////////////////////////////////////////////////////
// Class used to control the current thread with more methodes.

//============================================< Constructor(s) and destructor >

//=========================================================< Control methodes >
#ifdef	WIN32
//-----------------------------------------------------------------------------
// CTPublicCurrentThread::threadAttachOrDetach
//
// Entry point for module multi-thread.
//
// dwReason	   : Reason for calling function.
// Return      : bOk : Status (true or false).
//-----------------------------------------------------------------------------
bool CTPublicCurrentThread::threadAttachOrDetach (DWORD dwReason)
{
	CTThreadInfo*	pThreadInfo ;
	bool			bOk ;

	switch (dwReason)
	{
		case DLL_PROCESS_ATTACH :
			GdwTLSIndex = TlsAlloc () ;
			bOk = (GdwTLSIndex != TLS_OUT_OF_INDEXES) ;
			break ;
		case DLL_THREAD_ATTACH :
			bOk = true ;
			break ;
		case DLL_THREAD_DETACH :
			pThreadInfo = (CTThreadInfo*) TlsGetValue (GdwTLSIndex) ;
			if (pThreadInfo != NULL)
			{
				try
				{
					delete pThreadInfo ;
				}
				catch (CTThreadException&)
				{
					pThreadInfo = NULL ;
				}
			}
			bOk = true ;
			break ;
		case DLL_PROCESS_DETACH :
			if (GdwTLSIndex != TLS_OUT_OF_INDEXES)
			{
				pThreadInfo = (CTThreadInfo*) TlsGetValue (GdwTLSIndex) ;
				if (pThreadInfo != NULL)
				{
					try
					{
						delete pThreadInfo ;
					}
					catch (CTThreadException&)
					{
						pThreadInfo = NULL ;
					}
				}
				TlsFree (GdwTLSIndex) ;
				GdwTLSIndex = TLS_OUT_OF_INDEXES ;
			}
			bOk = true ;
			break ;
		default :
			bOk = false ;
	}

	return (bOk) ;
} // CTPublicCurrentThread::threadAttachOrDetach

//-----------------------------------------------------------------------------
// CTPublicCurrentThread::threadCleanupRoutine
//
// Destructor of specific data of a thread.
//
// Arguments : none.
// Return    : none.
//-----------------------------------------------------------------------------
void CTPublicCurrentThread::threadCleanupRoutine (void)
{
	CTThreadInfo*	pThreadInfo  = (CTThreadInfo*) TlsGetValue (GdwTLSIndex) ;
	if (pThreadInfo)
	{
		pThreadInfo->abort (pThreadInfo->pKillStatus_) ;
	}
} // CTPublicCurrentThread::threadCleanupRoutine
#endif	// WIN32

#if	defined (DGUX) || defined (AIX) || defined (HPUX) || defined (SUN)
//-----------------------------------------------------------------------------
// CTPublicCurrentThread::threadDataDestructor
//
// Destructor of specific object associated with a thread.
//
// pThreadInfo : thread information object pointer.
// Return      : none.
//-----------------------------------------------------------------------------
void CTPublicCurrentThread::threadDataDestructor (CTThreadInfo* pThreadInfo)
{
	if (pThreadInfo != NULL)
	{
		try
		{
			delete pThreadInfo ;
		}
		catch (CTThreadException&)
		{
			pThreadInfo = NULL ;
		}
	}
} // CTPublicCurrentThread::threadDataDestructor

//-----------------------------------------------------------------------------
// CTPublicCurrentThread::threadCleanupRoutine
//
// Destructor of specific data of a thread.
//
// pThreadInfo : thread information object pointer.
// Return      : none.
//-----------------------------------------------------------------------------
void CTPublicCurrentThread::threadCleanupRoutine (CTThreadInfo*)
{
	CTThreadInfo*	pThreadInfo ;
#if	defined (DGUX) || defined (HPUX)
	if ( (GptThreadKey == NULL) || (pthread_getspecific (GptThreadKey, (void**) &pThreadInfo) == -1) )
	{
		pThreadInfo = NULL ;
	}
#endif	// DGUX || HPUX
#if defined(AIX) || defined (SUN)
	pThreadInfo = (CTThreadInfo*) ( (GptThreadKey == NULL) ? NULL : pthread_getspecific (GptThreadKey) ) ;
#endif	// AIX
	if (pThreadInfo)
	{
		pThreadInfo->eState_ = THREAD_TERMINATED ;
		if (pThreadInfo->pAbortRoutine_)
		{
			pThreadInfo->pAbortRoutine_ (pThreadInfo->pKillStatus_) ;
			pThreadInfo->pAbortRoutine_ = NULL ;
		}

		// Prevent the joined threads.
		void**	ppStatus ;
		for (voidList::iterator itr = pThreadInfo->statusPointerList_.begin(); itr!=pThreadInfo->statusPointerList_.end(); itr++)
		{
			ppStatus = &(*itr);
			*ppStatus = pThreadInfo->pKillStatus_ ;
		}
	}
} // CTPublicCurrentThread::threadCleanupRoutine
#endif	// DGUX || AIX || HPUX

//-----------------------------------------------------------------------------
// CTPublicCurrentThread::threadStartRoutine
//
// Start routine of a new thread in the calling process.
//
// ppThreadInfo : Must point to a pointer to the thread information object.
// Return       : Depending of the system.
//-----------------------------------------------------------------------------
#ifdef	WIN32
unsigned _stdcall CTPublicCurrentThread::threadStartRoutine (CTThreadInfo** ppThreadInfo)
#endif	// WIN32
#if	defined (DGUX) || defined (AIX) || defined (HPUX) || defined (SUN)
void* CTPublicCurrentThread::threadStartRoutine (CTThreadInfo** ppThreadInfo)
#endif	// DGUX || AIX || HPUX
{
	void*			pStatus = (void*) -1 ;
	try
	{
		CTThreadInfo*	pThreadInfo ;

		// Create a new thread information object.
		try
		{
			pThreadInfo = new CTThreadInfo ;
		}
		catch (CTxalloc&)
		{
			pThreadInfo = NULL ;
		}
		if (pThreadInfo != NULL)
		{
			*ppThreadInfo = pThreadInfo ;
			// Wait for initialization of the thread.
			while (pThreadInfo->eInitState_ != CTThreadInfo::INIT_PARTIAL)
			{
				thread_yield () ;
#ifdef	HPUX
				// Verify if thread must suspended.
				if (pThreadInfo->uSuspendCount_ > 0)
				{
					try
					{
						if (pthread_setcancel (CANCEL_OFF) == -1)
						{	// Error from pthread_setcancel ().
							throw CTThreadException (string ("threadStartRoutine: pthread_setcancel failed.") + strerror (errno), __FILE__, __LINE__, errno) ;
						}
						{
							CTLockGuard<CTMutex>	guard (pThreadInfo->mutexInfo_) ;
							pThreadInfo->condResume_.wait () ;
						}
						if (pthread_setcancel (CANCEL_ON) == -1)
						{	// Error from pthread_setcancel ().
							throw CTThreadException (string ("threadStartRoutine: pthread_setcancel failed.") + strerror (errno), __FILE__, __LINE__, errno) ;
						}
					}
					catch (CTThreadException&)
					{
						pThreadInfo = NULL ;
					}
				}
#endif	// HPUX
			}

#if	defined (DGUX) || defined (AIX) || defined (HPUX) || defined (SUN)
			pthread_cleanup_push (CTPublicCurrentThread::threadCleanupRoutine, NULL) ;
#endif	// DGUX || AIX || HPUX
			// Initialization terminated.
			pThreadInfo->eInitState_ = CTThreadInfo::INIT_TOTAL ;
			// Wait for parent ack.
			while (pThreadInfo->eInitState_ != CTThreadInfo::INIT_END)
			{	
				thread_yield () ;
			}
			// Call the user start routine.
			pThreadInfo->pFunctorStartRoutine_->call () ;
			// Get the result pointer.
			pStatus = pThreadInfo->pFunctorStartRoutine_->getResultPointer () ;

			// Set thread state as TERMINATED.
			pThreadInfo->eState_ = THREAD_TERMINATED ;
#if	defined (DGUX) || defined (AIX) || defined (HPUX) || defined (SUN)
			pthread_cleanup_pop (0) ;
#endif	// DGUX || AIX || HPUX

			// Prevent the joined threads.
			void**	ppStatus ;
			while (!pThreadInfo->statusPointerList_.empty () )
			{
				// ????? 
				ppStatus = &(pThreadInfo->statusPointerList_.front());
				pThreadInfo->statusPointerList_.pop_front () ;
//				ppStatus = pThreadInfo->statusPointerList_.get () ;
				*ppStatus = pStatus ;
			}

#ifdef	WIN32
			unsigned long ulThread = pThreadInfo->ulThread_ ;
#endif	// WIN32

			// Destroy the information object.
			try
			{
				delete pThreadInfo ;
			}
			catch (CTThreadException&)
			{
				pThreadInfo = NULL ;
			}

#ifdef	WIN32
			// Free the system resources associated to the thread.
			CloseHandle ( (HANDLE) ulThread) ;
#endif	// WIN32
		}
		else
		{
			*ppThreadInfo = (CTThreadInfo*) -1 ;
		}
	}
	catch (...)
	{
	}

	// Exit the thread.
#ifdef	WIN32
	// Exit the thread with the status.
	_endthreadex ( (unsigned) pStatus) ;
	// Just a return statement for the compiler.
	return (unsigned) pStatus ;
#endif	// WIN32
#if	defined (DGUX) || defined (AIX) || defined (HPUX) || defined (SUN)
	// Exit the thread with the status.
	pthread_exit (pStatus) ;
	// Just a return statement for the compiler.
	return pStatus ;
#endif	// DGUX || AIX || HPUX
} // CTPublicCurrentThread::threadStartRoutine

//-----------------------------------------------------------------------------
// CTPublicCurrentThread::getCurrentThread
//
// Get the current thread.
//
// Arguments : none.
// Return    : reference to the current thread informations.
//-----------------------------------------------------------------------------
CTThreadInfo& CTPublicCurrentThread::getCurrentThread (void)
{
	CTThreadInfo*	pThreadInfo ;

#ifdef	WIN32
	pThreadInfo = (CTThreadInfo*) ( (GdwTLSIndex == TLS_OUT_OF_INDEXES) ? NULL : TlsGetValue (GdwTLSIndex) ) ;
#endif	// WIN32
#if	defined (DGUX) || defined (HPUX)
	if ( (GptThreadKey == NULL) || (pthread_getspecific (GptThreadKey, (void**) &pThreadInfo) == -1) )
	{
		pThreadInfo = NULL ;
	}
#endif	// DGUX || HPUX
#if defined(AIX) || defined (SUN)
	pThreadInfo = (CTThreadInfo*) ( (GptThreadKey == NULL) ? NULL : pthread_getspecific (GptThreadKey) ) ;
#endif	// AIX
	if (pThreadInfo == NULL)
	{
		// Create a new thread informations object.
		try
		{
			pThreadInfo = new CTThreadInfo ;
		}
		catch (CTxalloc&)
		{
			throw CTThreadException ("getCurrentThread: new CTThreadInfo failed.", __FILE__, __LINE__) ;
		}
#ifdef	WIN32
		// Save the thread handle.
		if (!DuplicateHandle (GetCurrentProcess (), GetCurrentThread (), GetCurrentProcess (), (LPHANDLE) &(pThreadInfo->ulThread_), 0, FALSE, DUPLICATE_SAME_ACCESS) )
		{	// Error from DuplicateHandle ().
			char zBuf[255] ;
 			FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
			CharToOem (zBuf, zBuf) ;
			throw CTThreadException (string ("getCurrentThread: DuplicateHandle failed: ") + zBuf, __FILE__, __LINE__, GetLastError () ) ;
		}
#endif	// WIN32

		// Set members initialization.
		pThreadInfo->eState_ = THREAD_RUNNING ;
		pThreadInfo->setPriority (PRIORITY_NORMAL) ;
		pThreadInfo->eInitState_ = CTThreadInfo::INIT_END ;
	}

#ifdef	HPUX
	// Verify if thread must suspended.
	if (pThreadInfo->uSuspendCount_ > 0)
	{
		if (pthread_setcancel (CANCEL_OFF) == -1)
		{	// Error from pthread_setcancel ().
			throw CTThreadException (string ("getCurrentThread: pthread_setcancel failed.") + strerror (errno), __FILE__, __LINE__, errno) ;
		}
		{
			CTLockGuard<CTMutex>	guard (pThreadInfo->mutexInfo_) ;
			pThreadInfo->condResume_.wait () ;
		}
		if (pthread_setcancel (CANCEL_ON) == -1)
		{	// Error from pthread_setcancel ().
			throw CTThreadException (string ("getCurrentThread: pthread_setcancel failed.") + strerror (errno), __FILE__, __LINE__, errno) ;
		}
	}
#endif	// HPUX

	// Return the pointer to the current thread informations.
	return (CTThreadInfo&) *pThreadInfo ;
} // CTPublicCurrentThread::getCurrentThread

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////// End of Class CTPublicCurrentThread //
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////// Globals Functions //
///////////////////////////////////////////////////////////////////////////////

#ifdef	WIN32
//-----------------------------------------------------------------------------
// DllMainMThread
//
// Entry point for module multi-thread.
//
// dwReason	   : Reason for calling function.
// Return      : bOk : Status (true or false).
//-----------------------------------------------------------------------------
bool WINAPI DllMainMThread (HANDLE, DWORD dwReason, PVOID)
{
	return CTPublicCurrentThread::threadAttachOrDetach (dwReason) ;
} // DllMainMThread
#endif	// WIN32

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// End of Globals Functions //
///////////////////////////////////////////////////////////////////////////////
