/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//AtvfRule.h
#ifndef _CYR_ATVFRULE_H
#define _CYR_ATVFRULE_H
/**
 * Class definition for an Atvf Rule Map. The class is used
 * for holding the record rules that describe a particular
 * Atvf format.  It can then be used for accessing an Input
 * stream and producing Records from the input.
 **/

#include <string>
#include <map>
#ifdef unix
#elif defined (__VMS)
#elif defined (WIN32)
#pragma warning(disable:4786)
using namespace std;
class AtvfRule;
ostream& operator<<(ostream& os, const AtvfRule& r);
#else //unknown
#error Unknown Platform
#endif // unknown

// local includes
#include "AtvfRecord.h"

typedef map<AtvfKey, SharedPtr<AtvfRecRule>, less<AtvfKey> > rule_map_t;

// class definition
class AtvfRule
{
    rule_map_t _rules;
    string _type;

public:
    //Constructors & destructor
    AtvfRule(const string& type);
    ~AtvfRule() {}

    //Accessors
    string* extractHead(AtvfInput& in) const;
    AtvfRecord* extractRecord(AtvfInput& in) const;
    void saveHead(const string& version, ostream& out) const;

    //Mutators
    void add(const SharedPtr<AtvfRecRule>& newRule);
    void clearRule();

    friend ostream& operator<<(ostream& os, const AtvfRule& r) ;
private:
    AtvfRule(const AtvfRule&);
    operator=(const AtvfRule&);
}
; //AtvfRule
#endif // _CYR_ATVFRULE_H
