////////////////////////////////////////////////////////////////////////
// RegExp.h
//
// This code has been derived from work by Henry Spencer.
// The main changes are
// 1. All char variables and functions have been changed to TCHAR
//    counterparts
// 2. Added GetFindLen() & GetReplaceString() to enable search
//    and replace operations.
// 3. And of course, added the C++ Wrapper
//
// The original copyright notice follows:
//
// Copyright (c) 1986, 1993, 1995 by University of Toronto.
// Written by Henry Spencer.  Not derived from licensed software.
//
// Permission is granted to anyone to use this software for any
// purpose on any computer system, and to redistribute it in any way,
// subject to the following restrictions:
//
// 1. The author is not responsible for the consequences of use of
// this software, no matter how awful, even if they arise
// from defects in it.
//
// 2. The origin of this software must not be misrepresented, either
// by explicit claim or by omission.
//
// 3. Altered versions must be plainly marked as such, and must not
// be misrepresented (by explicit claim or omission) as being
// the original software.
//
// 4. This notice must not be removed or altered.
/////////////////////////////////////////////////////////////////////////////

#ifndef _CREGEXP_H_
#define _CREGEXP_H_

#ifdef unix
#  define _strdup strdup
#  define _strupr strupr
#define TCHAR	char
#define _T
#define _tcslen strlen
#define _tcscspn strcspn
#define _tcsdup strdup
#define _tcsupr strupr
#define _tcsstr strstr
#define _tcsncpy strncpy
#define _tcschr strchr
#define _tcsncmp strncmp
#define _tcsspn strspn
#define _tcsnccpy strncpy
#elif defined (WIN32)
#else //unknown
#error Unknown Platform
#endif // unknown
#if defined (WIN32)
#ifdef __STDC__
#  undef __STDC__
#  include "TCHAR.H"
#  define __STDC__ 1
#else
#  include "TCHAR.H"
#endif
#endif
typedef int BOOL;

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

#define NSUBEXP  10

//
// LINUX Port Changes
//
#include <stdio.h>
#include <string.h>

class CRegExp
{
public:
    CRegExp();
    ~CRegExp();

    CRegExp *RegComp( const TCHAR *re );

    BOOL IsCompiled() const
    {
        return bCompiled;
    }

    void SetSrcPattern( const TCHAR *re )
    {
        delete srcPattern;
        srcPattern = _tcsdup(re);
    }

    const TCHAR* SrcPattern() const
    {
        return srcPattern;
    }

    int RegFind(const TCHAR *str, BOOL matchcase = TRUE);

    TCHAR* GetReplaceString( const TCHAR* sReplaceExp );

    int GetFindLen(size_t i = 0)
    {
        if( i >= NSUBEXP || startp[i] == NULL || endp[i] == NULL )
            return 0;

        return endp[i] - startp[i];
    }

    TCHAR* GetFindStart(size_t i = 0)
    {
        if( i >= NSUBEXP || startp[i] == NULL || endp[i] == NULL )
            return NULL;

        return startp[i];
    }

    TCHAR* operator[] (size_t i);


private:
    TCHAR *regnext(TCHAR *node);
    void reginsert(TCHAR op, TCHAR *opnd);

    int regtry(TCHAR *string);

    int regmatch(TCHAR *prog);
    size_t regrepeat(TCHAR *node);
    TCHAR *reg(int paren, int *flagp);
    TCHAR *regbranch(int *flagp);
    void regtail(TCHAR *p, TCHAR *val);
    void regoptail(TCHAR *p, TCHAR *val);
    TCHAR *regpiece(int *flagp);
    TCHAR *regatom(int *flagp);

    // Inline functions
private:
    TCHAR OP(TCHAR *p)
    {
        return *p;
    };
    TCHAR *OPERAND( TCHAR *p)
    {
        return (TCHAR*)((short *)(p+1)+1);
    };

    // regc - emit (if appropriate) a byte of code
    void regc(TCHAR b)
    {
        if (bEmitCode)
            *regcode++ = b;
        else
            regsize++;
    };

    // regnode - emit a node
    TCHAR * regnode(TCHAR op)
    {
        if (!bEmitCode)
        {
            regsize += 3;
            return regcode;
        }

        *regcode++ = op;
        *regcode++ = _T('\0');          /* Null next pointer. */
        *regcode++ = _T('\0');

        return regcode-3;
    };

    CRegExp& operator= (const CRegExp& rhs)
    {
        return *this;
    } // prevent assignment for time being


private:
    BOOL bEmitCode;
    BOOL bCompiled;
    TCHAR *sFoundText;
    TCHAR *srcPattern;		// Copy of orginal (src) pattern

    TCHAR *startp[NSUBEXP];
    TCHAR *endp[NSUBEXP];
    TCHAR regstart;         // Internal use only.
    TCHAR reganch;          // Internal use only.
    TCHAR *regmust;         // Internal use only.
    int regmlen;            // Internal use only.
    TCHAR *program;         // Unwarranted chumminess with compiler.

    TCHAR *regparse;        // Input-scan pointer.
    int regnpar;            // () count.
    TCHAR *regcode;         // Code-emit pointer; &regdummy = don't.
    TCHAR regdummy[3];      // NOTHING, 0 next ptr
    long regsize;           // Code size.

    TCHAR *reginput;        // String-input pointer.
    TCHAR *regbol;          // Beginning of input, for ^ check.
}
;

#endif // _CREGEXP_H_

