/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     TContext.cpp
// @module  TContext class | This module contains the implementation of the
//	    TContext and associated classes.
///////////////////////////////////////////////////////////////////////////////

#ifdef TH_POOL_VER
#   include "stdafx.h"
#endif

#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif

#include "TPLog.h"

#include "CORBARepository.h"
#ifndef CYRANO_TP_STANDALONE
#include <OSTAORB/OSTAORB.h>
#include "Objects/SpawnProc.h"
#endif
#include "TContext.h"
#include "mdbdelay.h"
#include "mdb_ctime.h"
#include "mdbparse.h"
#include "TExpression.h"

#ifndef TH_POOL_VER
#   include "threadcontrol.h"
#endif

#include "ociphases.h"
#include "CECTofTxt.h"

#ifdef TH_POOL_VER
#   include "AsyncVirtualUser.h"
#   include <strstream>
#endif

extern "C"
{
#   include <psl_macros.h>
#   include <psl_rtndef.h>
#   include <psldate_prmdef.h>
}


#ifdef unix
#   include <iostream.h>
#elif defined (__VMS)
#   include <iostream.h>
#elif defined (WIN32)
#   include <iostream>
#else //unknown
#   error Unknown Platform
#endif // unknown

unsigned  TContext::timefac = 100;

extern CORBAServer*    pCServer;

#ifdef TH_POOL_VER
#   include "AsyncVirtualUserController.h"
extern CAsyncVirtualUserController* _threadPoolController;
#endif

/**
* Constructor
**/
TContext::TContext()
        : intid(-1), verbose_mode(false),
        abort_flag(false),CurrentReturnValue("")
{
    m_ThreadID=GetFUID();//set up threadID

#ifndef TH_POOL_VER
    TContextData* ptdata=new TContextData();
    ptdata->oi=-1;
    ptdata->phasetxt="New Context started";
    ptdata->exit_called_flag=false;
    ptdata->m_have_valid_testcase=false;
    ptdata->flags=0;
    m_ContextDataStack.push(ptdata);
#endif
}

/**
* Destructor
**/
TContext::~TContext()
{
    cleanup();
    while(!m_ContextDataStack.empty())
    {
        delete m_ContextDataStack.top();
        m_ContextDataStack.pop();
    }

}

void TContext::cleanup(bool cflag)
{
    // Delete any left over commands
    for(cmds_map_t::iterator j = _cmds.begin(); j != _cmds.end(); ++j)
    {
        // Call command cleanup method if flag set
        if(cflag)
            (*j).second->cleanup();
        delete (*j).second;
    }
    _cmds.erase(_cmds.begin(), _cmds.end());

#ifndef TH_POOL_VER
    // And the same for connections
    for(cons_map_t::iterator i = _cons.begin(); i != _cons.end(); ++i)
    {
        // Call connection cleanup method if flag set
        if(cflag)
            (*i).second->cleanup();
        delete (*i).second;
    }
    _cons.erase(_cons.begin(), _cons.end());
#endif

    // Delete any left over handlers
    for(hnds_map_t::iterator k = _hnds.begin(); k != _hnds.end(); ++k)
    {
        delete (*k).second;
    }
    _hnds.erase(_hnds.begin(), _hnds.end());

    if(m_ContextDataStack.size() != 0)
    {
        // Delete any left over timer entries
        for(TimerMap_t::iterator iter = m_ContextDataStack.top()->TimerMap.begin(); iter != m_ContextDataStack.top()->TimerMap.end(); ++iter)
        {
            delete (*iter).second;
        }
        m_ContextDataStack.top()->TimerMap.erase(m_ContextDataStack.top()->TimerMap.begin(), m_ContextDataStack.top()->TimerMap.end());
    }
}

#ifndef TH_POOL_VER
/**
* Accessor to a connection in our map
**/
TConnect* TContext::connection(unsigned int conid) const
{
    cons_map_t::const_iterator i = _cons.find(conid);
    if(i == _cons.end())
        return NULL;
    return (*i).second;
}
#endif

/**
* Accessor to a command in our map
**/
TCommand* TContext::command(unsigned int cmdid) const
{
    cmds_map_t::const_iterator i = _cmds.find(cmdid);
    if(i == _cmds.end())
        return NULL;
    return (*i).second;
}



int	TContext::GetLabelIndex(const string& label) const
{
    return (m_ContextDataStack.top()->_script ? m_ContextDataStack.top()->_script->GetLabelIndex(label,m_ContextDataStack.top()->GetSubroutineData().SubroutineName) : -1);
}
int	TContext::GetNextEndTestCaseIndex() const
{
    return (Script() ? Script()->GetNextEndTestCaseIndex(m_ContextDataStack.top()->oi) : -1);
}

/**
* Accessor to a command in our map by id 
**/
TCommand* TContext::getCommandAtInd(unsigned int ind) const
{
    cmds_map_t::const_iterator j = _cmds.begin();
    for (unsigned i=0; i<ind; ++i)
    {
        if (j != _cmds.end())
            ++j;
    }
    if (j == _cmds.end()) return NULL;
    return (*j).second;
}

bool TContext::GetVerbose(int level)
{
    switch(level)
    {
    case TRACEMSG:
    case NOTEMSG:
        return logr != NULL;

        /*
            case HISTORYMSG:
            case REPORTMSG:
            case WRNMSG:
            case ERRMSG:
            case AUDITMSG:
        */
    default:
#ifndef CYRANO_TP_STANDALONE
        return Repository != NULL;
#else
        return logr != NULL;
#endif
    };
}

/**
* Accessor to perform logging, providing a level of importance
* that the log message is seen as.
* This should eventually use some sort of logging object passed
* into the constructor of the context.
**/
void TContext::log(const string& mesg, int level)
{
    string ThreadId = GetThreadID();
    string scriptname = ScriptName();
    int oind = m_ContextDataStack.top()->oi;

#ifdef TH_POOL_VER
    CTrace* pTrace = ((CAsyncVirtualUser *) this)->GetTrace();
    if(pTrace)
    {
        SYSTEMTIME st;
        GetLocalTime(&st);
        GetTraceStream(pTrace) << '[' << st.wMinute << ':' << st.wSecond << ':' << st.wMilliseconds << '|' << ThreadId << '|' << scriptname << '|' << oind << '|' << ((CAsyncVirtualUser *) this)->GetTRecordCallStep() << "]:" << mesg << "\r\n";
        ReleaseTraceStream();
    }
#endif

    switch(level)
    {
    case TRACEMSG:
        {
#ifdef __DLLSA__
            if(logr) logr->AddTrace(USERTRACE, TRACELEVEL, mesg.c_str());
#else
            string logstr;
            logstr=ThreadId+" :["+int_to_string(oind)+"]:"+mesg;
            if(logr) logr->AddTrace(USERTRACE,TRACELEVEL, logstr.c_str());
#endif
            break;
        }

    case HISTORYMSG:
        {
#ifndef CYRANO_TP_STANDALONE
            if(Repository) Repository->LogMessage(mesg,(int)HISTORYMSG,scriptname,ThreadId,oind);
#else
            if(logr) logr->AddTrace(USERTRACE, NOTELEVEL, mesg.c_str());
#endif  // CYRANO_TP_STANDALONE
            break;
        }

    case REPORTMSG:
        {
#ifndef CYRANO_TP_STANDALONE
            if(Repository) Repository->LogMessage(mesg,(int)REPORTMSG,scriptname,ThreadId,oind);
#else
            if(logr) logr->AddTrace(USERTRACE, NOTELEVEL, mesg.c_str());
#endif  // CYRANO_TP_STANDALONE
            break;
        }

    case NOTEMSG:
        {
            if(logr) logr->AddTrace(USERTRACE,NOTELEVEL,mesg.c_str());

            SetNoteText(mesg);
            break;
        }

    case WRNMSG:
        {
#ifndef CYRANO_TP_STANDALONE
            if(Repository) Repository->LogErrMessage(mesg,(int)AUDITMSG,scriptname,ThreadId,oind);
#else
            if(logr) logr->AddTrace(USERTRACE, WARNINGLEVEL, mesg.c_str());
#endif  // CYRANO_TP_STANDALONE
            break;
        }

    case ERRMSG:
        {
#ifndef CYRANO_TP_STANDALONE
            if(Repository) Repository->LogWrnMessage(mesg,(int)AUDITMSG,scriptname,ThreadId,oind);
#else
            if(logr) logr->AddTrace(USERTRACE,ERRORLEVEL, mesg.c_str());
#endif  // CYRANO_TP_STANDALONE
            break;
        }

    case AUDITMSG:
    default:
#ifndef CYRANO_TP_STANDALONE
        if(Repository) Repository->LogMessage(mesg,(int)AUDITMSG,scriptname,ThreadId,oind);
#else
        if(logr) logr->AddTrace(USERTRACE, AUDITLEVEL, mesg.c_str());
#endif  // CYRANO_TP_STANDALONE
        break;
    };

    return;
}

#ifndef TH_POOL_VER
/**
* Accessor to delay for a given number of milliseconds
* May be scaled or just completely ignored as per context set up.
**/
void TContext::delay(unsigned int ms)
{
    if(ms == 0)
        return; // just return if no delay to be done

    // Indicate transaction sleeping
    status = MDB_SLEEP;
    mdb_delay(ms, timefac);

    // Indicate transaction running again
    status = MDB_OK;
    return;
}

/**
* Mutator to add another connection to our map
**/
void TContext::addCon(unsigned int conid, TConnect* conP)
{
    cons_map_t::const_iterator i = _cons.find(conid);
    if(i != _cons.end())
    {	// already one with this id
        return;
    }
    _cons[conid] = conP;
    return;
}
#endif

/**
* Mutator to add another command to our map
**/
void TContext::addCmd(unsigned int cmdid, TCommand* cmdP)
{
    cmds_map_t::const_iterator i = _cmds.find(cmdid);
    if(i != _cmds.end())
    {	// already one with this id
        return;
    }
    _cmds[cmdid] = cmdP;
    return;
}

#ifndef TH_POOL_VER
/**
* Mutator to delete connection from our map
**/
void TContext::remCon(unsigned int conid)
{
    cons_map_t::iterator i = _cons.find(conid);
    if(i != _cons.end())
    {
        delete (*i).second;
        _cons.erase(i);
    }
    return;
}
#endif

/**
* Mutator to delete command from our map
**/
void TContext::remCmd(unsigned int cmdid)
{
    cmds_map_t::iterator i = _cmds.find(cmdid);
    if(i != _cmds.end())
    {
        delete (*i).second;
        _cmds.erase(i);
    }
    return;
}

// Extract declaration type (currently vardef) records from set.
int TContext::declarations(trecset_t _decls)
{
    string vdir;
    for(trecset_t::const_iterator it=_decls.begin() ; it != _decls.end() ; ++it)
    {
        // NOTE: case insensitive compare as long as constants (e.g. DECL_VARDEF)
        // are uppercase;
        string strtkn=(*it)->token();
        string_toupper(strtkn);
        if(strtkn == DECL_VARDEF)
        {
            if((m_ContextDataStack.top()->varlist.define_variable(ScriptName(), *it, vdir)) == NULL)
            {
                // Issue warning if volatile variable was attempting
                // to replace non-volatile one.
                if(m_ContextDataStack.top()->varlist.volatile_conflict(*it))
                {
                    if(GetVerbose(WRNMSG))
                    {
                        CECVar * vp = m_ContextDataStack.top()->varlist.get_variable_ptr(*it);
                        string wmsg  = "Variable conflict in file ";
                        wmsg        += m_ContextDataStack.top()->file;
                        wmsg        += ". ";
                        wmsg        += vp->get_name();
                        wmsg        += " is already defined with Thread/Private scope.";
                        log(wmsg, WRNMSG);
                    }
                }
            }
        }
        if(strtkn == DECL_FILDEF)
        {
            string  l_fileid;
            const TParam * par = (*it)->param("name");
            if(par != NULL)
                l_fileid=par->strVal();
            else
                throw mdbParseException("FileDefinition with unknown name");
            m_ContextDataStack.top()->filelist.DefineFile(l_fileid);
        }
    }

    return 0;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   LOAD command: This method retrieves data identified by the Type
//	    parameter (and for Load Timer the FromItem parameter) into the
//	    specified variable.
//
// @parm    LOADTYPE | Type | Load operation to be performed
// @parm    string | ToVarName | Variable (including any indices) to be loaded.
// @parm    string | FromItem | Item from which data is to be obtained.
//
void TContext::Load (LOADTYPE Type, string ToVarName ,string FromItem)
{
    string VarValue;

    // Decision on the Type parameter
    switch(Type)
    {
    case LD_ACTIVETHRDS:
        VarValue = get_variable(THREAD_COUNT_VAR,true);
        break;

    case LD_DATE:
        VarValue = mdb_dateonly();
        break;

    case LD_DIRECTORY://not relevant
        break;

    case LD_ERROR:
        break;

    case LD_NODENAME:
        {
#ifndef CYRANO_TP_STANDALONE
            CECInetInfo inet;
            VarValue = inet.GetHostName();
#else
            VarValue="localhost";
#endif
            break;
        }

    case LD_SCRIPT:
        VarValue = ScriptName();
        break;

    case LD_TEST:
        VarValue = get_test_name();
        break;

    case LD_THREAD:
        VarValue = GetThreadID();
        break;

    case LD_TIME:
        VarValue = mdb_timeonly();
        break;

    case LD_TIMER:
        {
            VarValue = LoadTimer (FromItem);
            break;
        }

    default:
        break;
    }// End of switch

    // Set the Variable with the value determined above.
#ifndef TH_POOL_VER
    TtVarRef varref(this);              // Variable reference object
#else
    TtVarRef varref(Script());			    // Variable reference object
#endif
    (void) varref.Parse(ToVarName);		    // Parse variable reference
    string varidx = varref.Evaluate(*this);	    // Get variable index
    set_variable(varref.Varnam(), VarValue, varidx);

} // End of Load


///////////////////////////////////////////////////////////////////////////////
// FileOpen
// ========
///////////////////////////////////////////////////////////////////////////////
// @mfunc Open a file for access
// @parm Filename to open (no pathname)
// @parm unique fileid
// @parm access opions (INPUT,OUTPUT,APPEND)
// @parm other options (e.g. BINARY)
// @rdesc
int TContext::FileOpen(const string& filename, const string& fileid, int access, int options)
{
    CECScriptFile* file=m_ContextDataStack.top()->filelist.GetFile(fileid);
    file->Open(filename,access,options);
    return 0;
}// end of TContext::FileOpen

///////////////////////////////////////////////////////////////////////////////
// FileRead
// ========
///////////////////////////////////////////////////////////////////////////////
// @mfunc Read from file opened for input
// @parm  File ID of file to be read
// @parm  variable
// @rdesc 0 for success, -1 for EOF and exception thrown for errors
int TContext::FileRead(const string& fileid,const TtVarRef* varref)
{
    CECScriptFile* file=m_ContextDataStack.top()->filelist.GetFile(fileid);
    string VarName=varref->Varnam();
    CECVar* var= m_ContextDataStack.top()->varlist.get_variable_ptr(VarName);
    int bytes=var->get_size();
    string result=file->Read(bytes);
    set_variable(VarName,result,varref->Evaluate(*this));
    if(file->IsEOF()!=0)return -1;//EOF
    return 0;
}// end of TContext::FileRead

///////////////////////////////////////////////////////////////////////////////
// FileWrite
// =========
///////////////////////////////////////////////////////////////////////////////
// @mfunc
// @parm string | fileid |
// @parm string | data |
// @rdesc
int TContext::FileWrite(const string& fileid,const string& data)
{
    CECScriptFile* file=m_ContextDataStack.top()->filelist.GetFile(fileid);
    file->Write(data);
    return 0;

}// end of TContext::FileWrite

///////////////////////////////////////////////////////////////////////////////
// FileControl
// ========
///////////////////////////////////////////////////////////////////////////////
// @mfunc
// @parm int | Operation |
// @parm string | fileid |
// @rdesc
int TContext::FileControl(int operation, const string& fileid)
{
    if(operation==TOF_FCNOP_CLOSE)
    {
        CECScriptFile* file=m_ContextDataStack.top()->filelist.GetFile(fileid);
        file->Close();
    }
    else if(operation==TOF_FCNOP_REWIND)
    {
        CECScriptFile* file=m_ContextDataStack.top()->filelist.GetFile(fileid);
        file->Rewind();
    }
    return 0;
}// end of TContext::FileControl



///////////////////////////////////////////////////////////////////////////////
// TCterminate
// ===========
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Terminates the current test case, as either passed or failed
//	    depending on the status of the flag passed in.
//
// @rdesc   integer 0 if the test case is terminated successfully, otherwise
//	    false.
// @parm    int | result | Flag determining the state of termination
//	    (TOF_TCTRES_PASS/FAIL).
// @parm    Description of TestCase passed or failed (taken from start
//	    test-case record at prepare time).
int TContext::TCterminate(int result,string desc)
{
    int status = 0;

    if(GetVerbose(REPORTMSG))
    {
        string resstr;

        if(result == TOF_TCTRES_FAIL)
            resstr = "FAIL";
        else
            resstr = "PASS";

        string mesg = resstr + ' ' + desc;
        log(mesg, REPORTMSG);
    }

    return status;

}// end of TContext::TCterminate

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This function is invoked when calling a subroutine. It evaluates
//	    the parameters in the actual parameter list and sets the
//	    corresponding parameter variables.
//
// @rdesc   <PRE>
//	    Return status:
//	    -1 = Failed; NO error message logged by this routine.
//	    0 (zero) = Success.
//	    </PRE>
//
// @parm    TScript* | script | Pointer to current script object.
// @parm    string | subNam | The name of the subroutine to be called.
// @parm    string | pmrListP | Pointer to TtVarList object for the
//	    actual parameter list.
//
int TContext::SubParameters(TScript* script, string subNam, TtValList* prmListP)
{
    // Return immediately, if the subroutine has no parameters.
    if (prmListP == 0) return 0;

    // Set-up an iterator for the subroutine names/parameters map.
    subrtn_map_t& subMap = script->Subroutines();
    subrtn_map_t::iterator subItr = subMap.begin();

    // Check that the subroutine is present in the map. if not return error.
    subItr = subMap.find(subNam);
    if(subItr == subMap.end()) return -1;

    // Get pointer to parameter descriptor list for formal parameters.
    TtValList* varPrmListP = (*subItr).second;
    const list<CECExprDsc>* varExprListP = varPrmListP->ExprListP();

    // Get pointer to parameter descriptor list for actual parameters.
    const list<CECExprDsc>* exprListP = prmListP->ExprListP();
    list<CECExprDsc>::const_iterator dscItr = exprListP->begin();

    // Iterate through the formal parameter list, evaluating the actual parameter
    // values and setting the corresponding parameter variables.
    string prmVal;  // Evaluated parameter value
    string varIdx;  // Variable index
    for (list<CECExprDsc>::const_iterator varDscItr = varExprListP->begin();
            varDscItr != varExprListP->end();
            ++varDscItr)
    {
        // Get pointer to the descriptor for the actual parameter and
        // report an error if there isn't one.
        if (dscItr == exprListP->end()) return -1;	// Return error if no actual parameter
        CECExprDsc prmValDsc = *dscItr;			// Get actual parameter descriptor
        ++dscItr;					// Point to next actual parameter

        // Get pointer to the descriptor for the formal parameter and
        // return failure, if the formal and actual parameters do NOT have the same data type.
        CECExprDsc varPrmDsc = *varDscItr;		// Get variable parameter descriptor
        if (prmValDsc.DTyp() != varPrmDsc.DTyp()) return -1;

        // Get the actual parameter value.
        // Return failure, if evaluation fails or no value returned.
        list<CECOperand>::const_iterator exprItr = prmValDsc.ExprIt();
        if (!(prmListP->EvaluateValue(*this, exprItr, prmVal))) return -1;
        if (prmVal.empty()) return -1;

        // Put the actual parameter value into the corresponding parameter variable.
        list<CECOperand>::const_iterator varExprItr = varPrmDsc.ExprIt();
        if (!(varPrmListP->EvaluateVarIdx(*this, &varExprItr, &varIdx))) return -1;
        set_variable(varPrmDsc.VarNam(), prmVal, varIdx);   // Set parameter variable

    } // End of for


    return 0;
} // End of TContext::SubParameters

///////////////////////////////////////////////////////////////////////////////
// @mfunc   FORMAT Command: This function calls helper functions to perform
//	    the required formatting.
//
// @rdesc   integer 0 if format successfull, otherwise -1
//
// @parm    string& | targetVar | Name of the variable (excluding any array
//	    subscripts) into which the result of operation is to be placed.
// @parm    string& | targetIdx | Subscript(s) (if any) associated with
//	    target variable as comma separated list, e.g. "1,10".
// @parm    string& | format | The format string.
// @parm    TtValList* | valListP | Pointer to list of format value descriptors.
//
int TContext::Format(const string& targetVar, const string& targetIdx, const string& format,
                     const TtValList* valListP)
{
    // Get pointer to variable object for "targetVar".
    // If not defined throw execption to advise caller.
    CECVar* vp = m_ContextDataStack.top()->varlist.get_variable_ptr(string(targetVar));
    if(vp == NULL) throw mdbParseException("Undefined variable");

    // Get the target variable options
    unsigned voptions = vp->get_options();

    // Call appropriate helper function to do the formatting.
    if (voptions & VTINT)
    {
        return FormatIntTarget(targetVar, targetIdx, format, valListP);
    }
    else
    {
        return FormatCharTarget(targetVar, targetIdx, format, valListP);
    }
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   FORMAT into Integer target. This helper function will translate a
//	    character string value into an integer value using the supplied
//	    format string, which must consist of a single format identifier.
//	    The only valid format identifiers for integer targets are:
//
//	    <PRE>
//	    %D - Convert a character string date value into numeric format
//	    representing the number of days since 17/11/1858.
//	    %T - Convert character string time value into a numeric format
//	    representing the number of 10 milli-seconds since midnight.
//	    </PRE>
//
// @rdesc   integer 0 if translation is successful, otherwise -1.
//
// @parm    string& | targetVar | Name of the variable (excluding any array
//	    subscripts) into which the result of operation is to be placed.
// @parm    string& | targetIdx | Subscript(s) (if any) associated with
//	    target variable as comma separated list, e.g. "1,10".
// @parm    string& | format | The format string.
// @parm    list<string> | varlist | Pointer to list of format value descriptors.
//
int TContext::FormatIntTarget(const string& targetVar, const string& targetIdx,
                              const string& format, const TtValList* valListP)
{
    // Check that there is at least one value to be translated; return an error if not.
    // (if too many values, any extra values will be ignored)
    if (valListP->ValCnt() < 1) return -1;

    // Check that the format identifier is "%D" or "%T"; return an error if not.
    if((format!="%D") && (format!="%T")) return -1;

    // Get pointer to the value descriptor for the value to be formatted
    list<CECExprDsc>::const_iterator dscItr = valListP->ExprListP()->begin();
    CECExprDsc valDsc = *dscItr;

    // Check that the value to be formatted is a character value, return an error if not.
    if (!(valDsc.IsChr())) return -1;

    // Get the value to be formatted - need to dereference variable values.
    list<CECOperand>::const_iterator exprIt = valDsc.ExprIt();
    string valStr;  // Value string
    if (!(valListP->EvaluateValue(*this, exprIt, valStr))) return -1;
    if (valStr.empty()) return -1;

    // Now do the Formatting.
    char resBuf[20]; // Result buffer
    if (format=="%T") // FORMAT character time value into integer
    {
        // if string < 8 characters exit TBD Log some messages or something
        if(valStr.length()<8) return -1;
        //remove any trailing spaces from string value
        (void) remove_trailing (valStr, " ");
        unsigned long TimeType;
        if ((valStr.length() > 8) && (valStr.length() <= 12)) // Millisecond value
        {
            TimeType = PSDFLD_TIME;	// Millisecond flag
        }

        else if (valStr.length() == 8 )	// No millisecond value
        {
            TimeType = PSDFLD_SECOND;	// Seconds flag
        }

        else
        {
            return -1;
        }

        // FORMAT Time.
        unsigned long ResultTime;
        unsigned long iret = PSL_PARSE_DATTIM (
                                 valStr.c_str(),	// String to parse
                                 &ResultTime,	// Address hold long
                                 &ResultTime,	// Address hold long
                                 PSDTYP_PSL,
                                 PSDPRS_TIME,	// Time parsed
                                 &TimeType);	// Time value out

        // Divide by 10 to get result in number of 10 millisecond units.
        ResultTime /= 10;
        if (!(psl_test_status (iret))) return -1;

        sprintf (resBuf, "%Lu", ResultTime);
    } // End of FORMAT time

    else if (format=="%D") // FORMAT character date value into integer
    {
        // if string < 9 characters exit TBD Log some messages or something
        if (valStr.length()<9) return -1;
        // Remove any trailing spaces from value
        (void) remove_trailing (valStr, " ");
        unsigned long DateType;
        // Conversion depends on size of character
        if (valStr.length() == 11)	// Century value
        {
            DateType = PSDFLD_CENTURY;	// Century flag
        }

        else if (valStr.length() == 9)	// No century value
        {
            DateType = PSDFLD_YEAR;	// Year flag
        }

        else
        {
            return -1;
        }

        // FORMAT Date
        unsigned long ResultDate;
        unsigned long iret = PSL_PARSE_DATTIM (
                                 valStr.c_str(),	// String to parse
                                 &ResultDate,	// Pointer hold long
                                 &ResultDate,	// Pointer hold long
                                 PSDTYP_PSL,
                                 PSDPRS_DATE|PSDPRS_Y2KCOMPLIANT,
                                 &DateType);	// Time value out

        if (!(psl_test_status (iret)))
        {			// If we had an error formatting
            return -1;
        }
        sprintf(resBuf, "%Lu", ResultDate);

    } // End of format DATE

    set_variable(targetVar, string(resBuf), targetIdx);
    return 0; // Success
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   FORMAT into Character target. This helper function will translate
//	    one or more character and/or integer values into a character
//	    string using the supplied format string, which may contain one or
//	    more format identifiers. The valid format identifiers for character
//	    targets are:
//
//	    <PRE>
//	    %D - Convert a numeric format date (representing the number of days
//	    since 17/11/1858) into a character string date in the fixed format
//	    "DD-MMM-CCYY".
//	    %T - Convert an integer time (representing the number of 10
//	    milli-seconds since midnight) into a character string time in the
//	    fixed format "HH:MM:SS.MMM".
//	    %U - This will take as input a character string and write it to the output
//	    character buffer converting any lower-case letters found to upper-case.
//	    %L - This will take as input a character string and write it to the output
//	    character buffer converting any upper-case letters found to lower-case.
//	    </PRE>
//
// @rdesc   integer 0 if translation is successfull, otherwise -1
//
// @parm    string& | targetVar | Name of the variable (excluding any array
//	    subscripts) into which the result of operation is to be placed.
// @parm    string& | targetIdx | Subscript(s) (if any) associated with
//	    target variable as comma separated list, e.g. "1,10".
// @parm    string& | format | The format string.
// @parm    list<string> | varlist | Pointer to list of format value descriptors.
//

int TContext::FormatCharTarget(const string& targetVar, const string& targetIdx,
                               const string& format, const TtValList* valListP)
{
    string resStr;						// Format into here
    string::const_iterator fmtItr;				// Format string iterator

    // Get pointer to the value descriptor for the first value to be formatted.
    list<CECExprDsc>::const_iterator dscItr = valListP->ExprListP()->begin();
    CECExprDsc valDsc = *dscItr;
    list<CECOperand>::const_iterator valItr = valDsc.ExprIt();
    string valStr;  // Value string
    if (!(valListP->EvaluateValue(*this, valItr, valStr))) return -1;
    if (valStr.empty()) return -1;

    // Empty the result string.
    resStr.empty();

    // Loop through the format string until end of string or no more arguments
    // searching  for '%', formating the target string on each iteration.
    // Upon finding a '%' check the following character, perform the required
    // operation and substitute the formatted value into the target string.
    bool next_arg = false;  // Flag for if an argument to format has been used and we need to move to the next one
    for (fmtItr = format.begin();
            (fmtItr != format.end()) && (dscItr != valListP->ExprListP()->end());
            ++fmtItr)
    {
        if (*fmtItr == '%')
        {
            // Correct identifier found, increment iterator and check next character
            ++fmtItr;
            // Decision on the next charcter, as to what formatting to do.
            switch (*fmtItr)
            {
            case 'D':	// Format integer DATE into character buffer
                {
                    // Check value is an integer value.
                    if (!(valDsc.IsInt())) return -1;
                    // Format numeric date to string.
                    unsigned long Date = atol(valStr.c_str());
                    char ConvDate[512];
                    unsigned long iret;
                    iret = PSL_FMT_DATTIM (Date,	    // Nothing pointed at
                                           Date,		    // Pointer to time int
                                           ConvDate,		    // Result buffer
                                           sizeof(ConvDate),	    // Length of result buf
                                           PSDTYP_PSL,
                                           (PSDFMT_CENTURY | PSDFMT_DATE)); // Convert date only
                    if (!(psl_test_status (iret))) return -1;

                    resStr += string(ConvDate);
                    next_arg = true;    // Move on to next argument
                    break;
                }

            case 'T':	// Format integer TIME into character buffer
                {
                    // Check value is an integer value.
                    if (!(valDsc.IsInt())) return -1;
                    // Format numeric time into string.
                    unsigned long Time = atol(valStr.c_str());
                    char ConvTime[512];
                    Time *= 10;   // Format time into milli-seconds

                    unsigned long iret;
                    iret = PSL_FMT_DATTIM (
                               Time,		    // Nothing pointed at
                               Time,		    // Pointer to time int
                               ConvTime,		    // Result buffer
                               sizeof(ConvTime),	    // Length of result buffer
                               PSDTYP_PSL,
                               (PSDFMT_MILLISEC | PSDFMT_TIME));   // Convert time only
                    if (!(psl_test_status (iret))) return -1;
                    resStr += string(ConvTime);
                    next_arg = true;
                    break;
                }

            case 'U':	// Uppercase character value
                {
                    // Check value is an integer value.
                    if (!(valDsc.IsChr())) return -1;
                    // Uppercase value
                    string_toupper (valStr);
                    resStr += valStr;
                    next_arg = true;
                    break;
                }

            case 'L':	// Lowercase character value
                {
                    // Check value is an integer value.
                    if (!(valDsc.IsChr())) return -1;
                    // Lowercase value
                    string_tolower (valStr);
                    resStr += valStr;
                    next_arg = true;
                    break;
                }

            default:
                {
                    --fmtItr;
                    resStr += *fmtItr;  // Add character to result string
                    next_arg = false;
                }

            } // end switch

            if (next_arg)
            {
                ++dscItr;	// Move to next value
                if (dscItr != valListP->ExprListP()->end())
                {
                    valDsc = *dscItr;
                    list<CECOperand>::const_iterator valItr = valDsc.ExprIt();
                    if (!(valListP->EvaluateValue(*this, valItr, valStr))) return -1;
                }
            }
        } // end of if...then
        else // Identifer not found
        {
            resStr += *fmtItr;	 // Copy character into result string
        }

    } // for...

    // Set the target variable with the resultant string.
    set_variable(targetVar, resStr, targetIdx);

    return 0;
} // End of TContext::Format

//
// @mfunc StartTimer
//
//    Start the named timer.
//
int		TContext::StartTimer		(const string& name)
{
    //
    //    Search for an existing entry in the timer map for this timer and use it otherwise allocate
    //    ourselves a new entry to use.
    //
    CECTimer* pTimer;

    TimerMap_t::iterator iter = m_ContextDataStack.top()->TimerMap.find (name);
    if (iter == m_ContextDataStack.top()->TimerMap.end())
    {
        pTimer = new (CECTimer);
        //
        // LINUX Port Changes, do not use make_pair
        //
        m_ContextDataStack.top()->TimerMap.insert(TimerMap_t::value_type(string(name) ,pTimer));
    }
    else
    {
        //
        //	  LINUX Port Changes
        //
        //    For Linux we canot use the arrow operator on this object
        pTimer = (*iter).second;
    }
    //
    //    ...and start the clock ticking.
    //
    pTimer->Start();
    return 0;
}

//
// @mfunc StopTimer
//
//    Stop the named timer and create a timer output record for it.
//
int		TContext::StopTimer		(const string& name)
{
    //
    //    Search for an existing entry in the timer map for this timer. If we cannot find one or the timer
    //    has already been stopped then we will discard this stop timer as it is superfluous
    //
    CECTimer* pTimer;
    TimerMap_t::iterator iter = m_ContextDataStack.top()->TimerMap.find (name);
    if (iter == m_ContextDataStack.top()->TimerMap.end())
        throw mdbParseException ("Timer has not been declared or has not been started");

    //
    //	  LINUX Port Changes
    //
    //    For Linux we canot use the arrow operator on this object
    pTimer = (*iter).second;
    if (!pTimer->Running()) return 0;

    //
    //    ...and stop the clock ticking.
    //
    int ElapsedTime = pTimer->Stop();

    //
    //    Construct the identification string which we will log with the timer and is of the form
    //
    //
    string ThreadId=GetThreadID();
    string scriptname=ScriptName();
    string TestName=get_test_name();
#ifndef TH_POOL_VER
    int no_threads=atoi(get_variable(THREAD_COUNT_VAR,true).c_str());
#else
    int no_threads = _threadPoolController->GetUsersCount();
#endif
    //
    //    Create a timer output and store it with the test run.
    //
    if(Repository) Repository->TimerOutput (name,ElapsedTime ,scriptname,ThreadId,TestName,no_threads);
    return 0;
}

//
// @mfunc	LoadTimer
//
//    Helper routine to return the value for the last elapsed time for the specified timer
// @parm String containing the name of the timer to load
// @rdesc Last elapsed time of the specified timer returned as an ASCII string
//
// Throws mdbParseExeception is the timer has either never been started or has never been stopped
//
string	TContext::LoadTimer	(string FromTimer)
{
    string VarValue;

    //
    //    Search for a timer entry of the specified name and if it exists recover the last
    //    expired timer value for it.
    //
    TimerMap_t::iterator iter = m_ContextDataStack.top()->TimerMap.find (FromTimer);
    if (iter == m_ContextDataStack.top()->TimerMap.end())
        throw mdbParseException ("Timer has not been declared or has not been started");

    //
    //    Now check for an elapsed time. If this is zero then the timer has never been stopped and
    //    again we will throw an exception
    //

    //
    //	LINUX Port Changes
    //
    //  For Linux we canot use the arrow operator on this object
    CECTimer* pTimer = (*iter).second;

    int ElapsedTime = pTimer->ElapsedTime();
    if (ElapsedTime == 0)
        throw mdbParseException ("Timer has been started but has never been stopped");

    //
    //    ...otherwise just format the elapsed time as a string
    //
    VarValue = int_to_string(ElapsedTime);
    return VarValue;
}

#ifndef CYRANO_TP_STANDALONE

bool TContext::LockMutex(string name,int scope)
{
    CECMutex* mtx=m_MutexList.GetMutex(name,scope);
    bool is_locked=mtx->trylock();
    return is_locked;
}
bool TContext::UnLockMutex(string name,int scope)
{
    CECMutex* mtx=m_MutexList.GetMutex(name,scope);
    bool is_unlocked=mtx->unlock();
    return is_unlocked;
}

#endif  // #ifndef CYRANO_TP_STANDALONE
