/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
///////////////////////////////////////////////////////////////////////////////
// @doc     CECScriptFile
// @module  CECScriptFile | File Access class
///////////////////////////////////////////////////////////////////////////////
#include "CORBARepository.h"
#include "CECScriptFile.h"
#include "mdbexcept.h"
#include "mdbtypes2.h"
extern string exedir;//global string variable
list<string> CECScriptFile::filename_cache;
#ifndef CYR_NO_STD_NAMESPACE
using namespace std;
#endif
///////////////////////////////////////////////////////////////////////////////
//@mfunc    constructor, define a file for use
//@parm	    FileId of file
CECScriptFile::CECScriptFile(string fileid):
        m_fileid(fileid),
        m_filename(""),
        m_options(0),
        m_access(0),
        m_File()
{}
///////////////////////////////////////////////////////////////////////////////
//@mfunc Destructor

CECScriptFile::~CECScriptFile()
{}
///////////////////////////////////////////////////////////////////////////////
//@mfunc Open the file
//@parm  local file name for file (no pathname as will be retrieved from repository)
//@parm  access type
//@parm  other options (e.g. binary)

int CECScriptFile::Open(string filename,int access,int options)
{
    m_access=access;
    m_options=options;
    int openoptions=0;
    openoptions|=(options&TOF_FILOPT_BINFIL)?ios::binary:0;
    openoptions|=(access&TOF_FILACC_INPUT)?ios::in:0;
    openoptions|=(access&TOF_FILACC_OUTPUT)?ios::out:0;
    openoptions|=(access&TOF_FILACC_APPEND)?(ios::app|ios::out):0;
    //m_filename = Repository->GetFile(filename);//Get the file as this is not a tof it will not be cached
    bool have_file=false;
    if(m_access&TOF_FILACC_INPUT)//if for read only then only get if required
    {
        for(list<string>::iterator i=filename_cache.begin();i!=filename_cache.end();i++)
        {
            if(*i==filename)
                have_file=true;
        }
    }
    if(!have_file)
        GetFile(filename);
    m_File.open(m_filename.c_str(),openoptions);
#ifndef HPUX
    return m_File.is_open();
#else
    return 1;
#endif
}
string CECScriptFile::Read(int bytes)
{
    if(!(m_access&TOF_FILACC_INPUT))
        throw mdbParseException("Attempt to read file opened for output");
    string retstr("");
    char* buf=new char[bytes];
    //Read in required number of bytes
    m_File.read(buf,bytes);
    if(!(m_options&TOF_FILOPT_BINFIL))//not binary file reading
    {
        char* nlpos=(char*)memchr(buf,'\n',bytes);
        if(nlpos==NULL)//no newline found read until newline
        {
            char throw_away_char;
            do
            {
                m_File.read (&throw_away_char, 1);
                //Read the next byte from the file
                if(m_File.eof())
                {
                    retstr= buf;
                    delete[] buf;
                    return retstr;//if end of file return buffer now
                }

                if(!m_File.good())
                {
                    delete[] buf;
                    throw mdbParseException(string("Error Reading from File ")+m_filename);
                }
            }
            while (throw_away_char != '\n');
        }
        else//rewind to newline
        {
            size_t nllength=nlpos-buf;//length to new line is address of new line-address of buf[0]
            long rewind=bytes-nllength;//length to rewind is the length read in - the length to the new line
            m_File.seekg(rewind,ios::cur);
            buf[nllength]='\0';
        }

    }
    retstr=buf;
    delete[] buf;
    return retstr;
}
int CECScriptFile::Write(string data)
{
    if(!(m_options&TOF_FILOPT_BINFIL))
        data+="\n";
    if(m_access&TOF_FILACC_INPUT)
        throw mdbParseException("Attempt to write to file opened for input");
    m_File.write(data.c_str(),data.length());
    if(!m_File.good())
        throw mdbParseException(string("Error Writing to File ")+m_filename);
    return 0;

}
int CECScriptFile::Close()
{
    m_File.close();
    if(!m_options&TOF_FILACC_INPUT)
        PutFile();
    return 0;
}
int CECScriptFile::Rewind()
{
    if(m_options&TOF_FILACC_INPUT)
    {
        m_File.seekg(0L,ios::beg);
    }
    else
    {
        m_File.seekp(0L,ios::beg);
    }
    return 0;
}
void CECScriptFile::GetFile(const string& Filename)
{
    string FilenamePath = exedir + FN_PATHSEP + Filename;
#ifndef CYRANO_TP_STANDALONE
    try
    {
        m_OrbFile = new CECOrbFile(Repository->GetDataFolder(), Filename.c_str());
    }
    catch(...)//assume this is beacuse it is not there and try and create a new one
    {
        m_OrbFile = new CECOrbFile(Repository->GetDataFolder(), Filename.c_str(),"Data File");
    }
    // Validate the file exchange pointer
    if (m_OrbFile == NULL)
    {
        throw mdbParseException("Cannot get file");
    }
    // Copy the file requested to the local disk space.
    m_OrbFile->CheckOut(FilenamePath.c_str(),true);
#endif
    m_filename=FilenamePath;
}
void CECScriptFile::PutFile()
{
#ifndef CYRANO_TP_STANDALONE
    m_OrbFile->CheckIn(m_filename.c_str(),true);
    delete m_OrbFile;
#endif
}

