/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
///////////////////////////////////////////////////////////////////////////////
// @doc     CECVarOptions.h
// @module  CECVarOptions class header | This module...
//
///////////////////////////////////////////////////////////////////////////////
#ifndef _CECVAROPTIONS_H_
#define _CECVAROPTIONS_H_
#include <string>
#include <map>
#include <vector>
#ifndef CYR_NO_STD_NAMESPACE
using namespace std;
#endif
typedef map<string, int, less<string> > OPTMAP;

#include <tofprmdef.h>

// Default variable option mask
#define DEFAULT_VAR_OPTIONS     (unsigned)(VTCHAR | VSLIST | VONEXT | VCSCRIPT)
// Default variable size
#define DEFAULT_VAR_SIZE        1
// Default array size
#define DEFAULT_ARRAY_SIZE        ""
// Maximum length of a dataname
#define MAX_DATANAME_LEN        16
// Maximum length of a variable value
#define MAX_VALUE_LEN           511

// OUTPUT FORMATTING PARAMETERS
// Max line length in .isq file.
// Note: will exceed this if individual value string too long.
#define ISQ_PAGE_WIDTH          77
// Name indent
#define ISQ_NAME_INDENT         15
// Value list and option indent
#define ISQ_VAL_INDENT          32
// ISQ (variable decl) file header
#define ISQ_HEADER              "#ignore differences\n\n"
// ISQ (variable decl) file trailer
#define ISQ_TRAILER             "\n#end ignore"

// Type Declaration Text
#define VDECCHAR    "Character"
#define VDECINT     "Integer"
#define VDECTIM     "Timer"
#define VDECFIL     "File"

// Variable options text
#define VOPTLOCAL   " ,LOCAL"
#define VOPTSCRIPT  " ,SCRIPT"
#define VOPTGLOBAL  " ,GLOBAL"
#define VOPTTHREAD  " ,THREAD"
#define VOPTRAND    " ,RANDOM"

// Dataname prefix for FILE
#define VFPREFIX    "F_"

// Characters not allowed in a filename. Note: Some of
// these are not illegal but could potentially be treated
// as "special" by a shell.
#define FN_ILLEGAL_CHARS    " \\/:*?<>|\"'`$~&"
// External datafile name extension
#define SQV_FILE_EXTENSION  ".sqv"

typedef map<unsigned, string, less<unsigned> > VALLIST;
#endif
