/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     TEThread - Thread Implementation for TestManager
//
///////////////////////////////////////////////////////////////////////////////
#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif
#include "CORBARepository.h"
#ifndef CYRANO_TP_STANDALONE
#include <OSTAORB/OSTAORB.h>
#endif
#include "TEThread.h"
#ifndef CYRANO_TP_STANDALONE
#include "TMVarFactory.h"
#include "Objects/SpawnProc.h"
#include <GlobalVariable.h>
#endif
#include "TContext.h"
#include "TEthreadcontrol.h"
#include "ociphases.h"
extern CORBAServer*    pCServer;
#ifndef CYR_NO_STD_NAMESPACE
using namespace std;
#endif
extern string exedir;
extern string toffiledir;
CTMutex TEThread::m_ThreadMap_mutex;
CTMutex TEThread::m_cache_mutex;
TEThread::TEThread(TEthreadcontrol* threadcontrol,string filename):
        Cmdbthread(threadcontrol,filename),
        m_TestName(""),
        m_NoteText(""),
        have_cleaned_up(false),
        m_TestCaseFail(0),
        m_TestCasePass(0),
        m_ParamsInfo(NULL),
        m_tethreadcontrol(threadcontrol),
        m_last_error("")
{
    Initialise();
}

TEThread::~TEThread()
{
    //CleanUp();should have already been told to clean up!
}


void TEThread::Initialise()
{
    have_cleaned_up=false;
#ifndef CYRANO_TP_STANDALONE
    ThreadInfo thinfo;
    thinfo.UIdent=m_ThreadID.c_str();
    thinfo.id=intid;
    thinfo.ScriptName=(const char*)"";
    thinfo.Status=ACTIVE;
    thinfo.NoteText=(const char*)"";
    CTTime tm;
    tm.now();
    thinfo.StartTime=tm.getSeconds();
    CTLockGuard<CTMutex> guard(m_ThreadMap_mutex);
    m_tethreadcontrol->ThreadMap.insert(ThreadMap_t::value_type(m_ThreadID,thinfo));
#endif
    //increment number of threads running TBD this should have some global mutex locking
    int no_threads=atoi(get_variable(THREAD_COUNT_VAR,true).c_str());
    no_threads++;
    set_variable(THREAD_COUNT_VAR,int_to_string(no_threads));
}
#ifndef CYRANO_TP_STANDALONE
void TEThread::SetThreadInfo(const string& TestName,const ThreadDef & thrdef)
{
    m_TestName=TestName;//name of Test Running this thread
    m_ThreadDef=thrdef;
}
#endif
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Evaluete parameters for Call Script Entry Command
// @parm    string of comma separated variable names to be set
// @rdesc   0 for success
int TEThread::Entry(TtValList varPrmListP)
{
    // Get pointer to parameter descriptor list for formal parameters.
    const list<CECExprDsc>* varExprListP = varPrmListP.ExprListP();


    //////////////////////
    //////////////////////
    if (!m_ParamsInfo)
    {
        set_variable((*(*varExprListP).begin()).VarNam(), "DEFAULT");   // Set parameter variable
        return 0;
    }
    //////////////////////
    //////////////////////


    // Get pointer to parameter descriptor list for actual parameters.
    const list<CECExprDsc>* exprListP = m_ParamsInfo->ExprListP();
    list<CECExprDsc>::const_iterator dscItr = exprListP->begin();

    // Iterate through the formal parameter list, evaluating the actual parameter
    // values and setting the corresponding parameter variables.
    string prmVal;  // Evaluated parameter value
    string varIdx;  // Variable index
    list<string>::const_iterator ParamValIter=m_Params.begin();
    for (list<CECExprDsc>::const_iterator varDscItr = varExprListP->begin();
            varDscItr != varExprListP->end();
            varDscItr++,ParamValIter++)
    {
        // Get pointer to the descriptor for the actual parameter and
        // report an error if there isn't one.
        if (dscItr == exprListP->end())
            throw mdbParseException("Required Paramter Missing");// Return error if no actual parameter
        CECExprDsc prmValDsc = *dscItr;         // Get actual parameter descriptor
        dscItr++;                   // Point to next actual parameter

        // Get pointer to the descriptor for the formal parameter and
        // return failure, if the formal and actual parameters do NOT have the same data type.
        CECExprDsc varPrmDsc = *varDscItr;      // Get variable parameter descriptor
        if (prmValDsc.DTyp() != varPrmDsc.DTyp())
            throw mdbParseException("Paramter type mismatch");

        // Put the actual parameter value into the corresponding parameter variable.
        list<CECOperand>::const_iterator varExprItr = varPrmDsc.ExprIt();
        if (!(varPrmListP.EvaluateVarIdx(*this, &varExprItr, &varIdx)))
            throw mdbParseException("Failed to Evaluate");
        prmVal = *ParamValIter;//cooresponding value in list
        set_variable(varPrmDsc.VarNam(), prmVal, varIdx);   // Set parameter variable

    } // End of for



    return 0;
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Clear up this Script on exit from script with/without keep alive options set
// @parm    true - with keep alive option set
int TEThread::ExitCommand(bool keepalive,const string& retvar)
{
    if(m_ContextDataStack.top()->exit_called_flag==true)
        return 0;//Have already exited don't try and do it again
    logr->AddTrace(DBGSTD,DETAILLEVEL,"TEThread::ExitCommand");
#ifndef CYRANO_TP_STANDALONE
    //Deregister ScriptObject from TestManagerVariableFactories for
    string Pfactory_regname=Repository->GetPrimaryTestRunString();
    string factory_regname=Repository->GetTestRunString();
    CORBA::Object_var obj;
    obj=pCServer->FindObjectWithName(TMVarFactory::CLASS_NAME,Pfactory_regname.c_str());
    TMVarFactory_var TMVF = TMVarFactory::_narrow(obj.inout());
    if(!CORBA::is_nil(TMVF))
    {
        CORBA::String_var cb_scriptname=CORBA::string_dup(ScriptName().c_str());
        TMVF->DeregisterScript(cb_scriptname.in(),keepalive);
        if(Pfactory_regname!=factory_regname)//if the two TestManager Factories are not the same one
        {
            obj=pCServer->FindObjectWithName(TMVarFactory::CLASS_NAME,factory_regname.c_str());
            TMVF = TMVarFactory::_narrow(obj.inout());
            TMVF->DeregisterScript(cb_scriptname.in(),keepalive);
        }
        TContext::ExitCommand(keepalive,retvar);
        m_ContextDataStack.top()->exit_called_flag=true;//have explicitly called exit in Script
    }


    else
    {
        logr->AddTrace(DBGSTD,DETAILLEVEL,"TestManager Variable Factory Unreachable");
    }
    return 0;//Got here without throwing an exception so must be successful
#else
    m_ContextDataStack.top()->exit_called_flag=true;//have explicitly called exit in Script
    return 0;
#endif
}
void TEThread::CleanUp()
{
    if(have_cleaned_up)return;//Should only do this once per thread
    // Should be explicit but there is an implicit call in the destrctor for aborted threads
    //decrement number of threads running TBD this should have some global mutex locking
    int no_threads=atoi(get_variable(THREAD_COUNT_VAR,true).c_str());
    no_threads--;
    set_variable(THREAD_COUNT_VAR,int_to_string(no_threads));
    set_phase_text("CleanUp");
    ExitCommand(false,"");
#ifndef CYRANO_TP_STANDALONE
    CTLockGuard<CTMutex> guard(m_ThreadMap_mutex);
    ThreadMap_t::size_type sz=m_tethreadcontrol->ThreadMap.erase(m_ThreadID);
    if(sz!=1)//did not remove 1 and only 1 item
    {
        string Errmsg="ThreadInfo for ThreadID :"+m_ThreadID +" Could not be erased properly";
        throw mdbParseException(Errmsg);
    }
#endif
    set_phase_text(TP_PHASE_NONE);
    have_cleaned_up=true;
}
void TEThread::SetNoteText(const string& mesg)
{
    m_NoteText=mesg;
#ifndef CYRANO_TP_STANDALONE
    (*m_tethreadcontrol->ThreadMap.find(m_ThreadID)).second.NoteText=m_NoteText.c_str();
#endif
}
#ifndef CYRANO_TP_STANDALONE
ThreadDef TEThread::GetThreadDef()
{
    return m_ThreadDef;
}
#endif
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Get Name of test Currently running (not script), retrieve from Parent test level
// @rdesc   Name of Test Running
string TEThread::get_test_name(void)
{
    return m_TestName;
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Any Set up required specifically for this class of thread at start of a script
// @parm    Name of Script Being started
void    TEThread::StartScript(const string& fullscriptname)
{
    string scriptname=strip_extension(filename_base(string(fullscriptname)));
    m_ScriptNameStack.push(scriptname);
#ifndef CYRANO_TP_STANDALONE
    //find ThreadID in map , if not there then error
    ThreadMap_t::iterator ThMiter=m_tethreadcontrol->ThreadMap.find(m_ThreadID);
    if(ThMiter==m_tethreadcontrol->ThreadMap.end())
    {
        string Errmsg="ThreadInfo not available for ThreadID :"+m_ThreadID;
        throw mdbParseException(Errmsg);
    }
    (*ThMiter).second.ScriptName=scriptname.c_str();//update ThreadMap for CORBA Calls
#endif
    m_ContextDataStack.top()->exit_called_flag=false;
#ifndef CYRANO_TP_STANDALONE
    //Register script for Variable factory
    string Pfactory_regname=Repository->GetPrimaryTestRunString();
    string factory_regname=Repository->GetTestRunString();
    CORBA::Object_var obj;
    obj=pCServer->FindObjectWithName(TMVarFactory::CLASS_NAME,Pfactory_regname.c_str());
    TMVarFactory_var TMVF = TMVarFactory::_narrow(obj.inout());
    CORBA::String_var cb_scriptname=CORBA::string_dup(ScriptName().c_str());
    TMVF->RegisterScript(cb_scriptname.in());
    if(Pfactory_regname!=factory_regname)//if the two TestManager Factories are not the same one
    {
        obj=pCServer->FindObjectWithName(TMVarFactory::CLASS_NAME,factory_regname.c_str());
        TMVF = TMVarFactory::_narrow(obj.inout());
        TMVF->RegisterScript(cb_scriptname.in());
    }
#endif
    string msg="Start Script";
    log(msg,AUDITMSG);
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Any Clear up required specifically for this class of thread at end of a script
void    TEThread::EndScript()
{
    string msg="End Script";
    log(msg,AUDITMSG);
    //at end of script default to ending without keepalive option
    if(!m_ContextDataStack.top()->exit_called_flag)
        ExitCommand(false,"");
    if(m_ScriptNameStack.size()>1)//if still have scripts to return to remove scriptname from stack
    {
        m_ScriptNameStack.pop();
    }
#ifndef CYRANO_TP_STANDALONE
    (*m_tethreadcontrol->ThreadMap.find(m_ThreadID)).second.ScriptName=ScriptName().c_str();//update ThreadMap for CORBA Calls
#endif
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc Call another script within this thread
// @parm  ScriptName
// @parm  parameters to be passed to script
// @rdesc -1 for failure
int TEThread::CallScript(const string& name,TtValList params)
{
    logr->AddTrace(DBGSTD,DEBUGLEVEL,"Entry to TEThread::CallScript");
    // get parameter data before we stack up
    m_ParamsInfo=&params;
    const list<CECExprDsc>* varExprListP = m_ParamsInfo->ExprListP();
    if(!varExprListP->empty())
    {
        bool eval=m_ParamsInfo->EvaluateValues(*this,m_Params);
        if(!eval)
            throw mdbParseException("Unable to Evaluate Parameter values for Call Script");
    }
    //stack up member data
    TContextData* ptdata=new TContextData();//was *m_ContextDataStack.top());do not default to values of current Data
    ptdata->phase=m_ContextDataStack.top()->phase;
    ptdata->phasetxt="Start New Script";
    ptdata->flags=m_ContextDataStack.top()->flags;
    ptdata->oi=-1;
    //Add threadwide variables from previous varlist to this one
    ptdata->varlist.add_vars(m_ContextDataStack.top()->varlist.get_threadwide_vars());
    m_ContextDataStack.push(ptdata);
    // get new file based upon ScriptName
    string ScriptFilename;
    // bodge to stop unnecessary checking out of files from repository
#ifndef CYRANO_TP_STANDALONE
    string tmpstring=exedir + FN_PATHSEP + name + ".tof";
#else
    string tmpstring=toffiledir + name + ".tof";
#endif

    set_file(tmpstring);//push latest FULL file name onto stack

    // Put a mutex lock around the code to retrieve the file.
    ThreadTask* ScriptTask;
    {
        logr->AddTrace(DBGSTD,DEBUGLEVEL,"Lock Mutex to retrieve script");
        CTLockGuard<CTMutex> guard(m_cache_mutex);

        bool is_file_there=m_threadcontrol->doesTaskExist(tmpstring);
        if(!is_file_there)
        {
            string objnam = name + ".tof";
            logr->AddTrace(DBGSTD,DEBUGLEVEL,(string("Get the File: ") + objnam).c_str());
            ScriptFilename = Repository->GetTofFile(objnam);
        }
        else
        {
            ScriptFilename = tmpstring;
        }

#ifdef CYRANO_TP_STANDALONE
        ScriptFilename = tmpstring;
#endif

        //Create new task
        try
        {
            ScriptTask=m_threadcontrol->RetrieveTask(ScriptFilename);
        }
        catch(mdbParseException &e)
        {
            string msg = "Error creating new Script Task: " + e.what();
            log(msg,ERRMSG);
            return -1;
        }
        catch(...)
        {
            log("Error creating new Script Task",ERRMSG);
            return -1;
        }
        logr->AddTrace(DBGSTD,DEBUGLEVEL,"UnLock Mutex to retrieve script");
    }   // Release mutex lock around the code to retrieve the file.

    // run task, Note: this calls the StartScript and EndScript methods in
    // derived TContext classes for specific setup/clear down
    int iret=0;//return value, -1 indicates an error
    try
    {
        ScriptTask->run(*this);
    }
    catch(DetachThreadException& d)//if thread detatching pass on up to top
    {
        throw d;
    }
    catch(mdbParseException &e)//to change to a more specific exception e.g CyrRuntime
    {
        m_last_error="Error in (File: "+ filename_base(get_file()) + " Phase: "+string(get_phase_text())
                     +") with "+e.what();
        log(m_last_error,ERRMSG);
        iret=-1;
    }
    catch(CyrException&e)
    {
        m_last_error="Error in (File: "+ filename_base(get_file()) + " Phase: "+string(get_phase_text())
                     +") with "+e.what();
        log(m_last_error,ERRMSG);
        iret=-1;
    }
#ifndef CYRANO_TP_STANDALONE
    catch(CyrGlobalVariableException &e)
    {
        m_last_error="Error in (File: "+ filename_base(get_file()) + " Phase: "+string(get_phase_text())
                     +") with "+(const char*)e.what;
        log(m_last_error,ERRMSG);
        iret=-1;
    }
    catch(CORBA::Exception&e)
    {
        m_last_error="Error in (File: "+ filename_base(get_file()) + " Phase: "+string(get_phase_text())
                     +") with CORBA::"+CORBAExString(e);
        log(m_last_error,ERRMSG);
        iret=-1;
    }
#endif
    catch(exception &e)
    {
        m_last_error="Error in (File: "+ filename_base(get_file()) + " Phase: "+string(get_phase_text())
                     +") with "+e.what();
        log(m_last_error,ERRMSG);
        iret=-1;
    }
    catch(...)
    {
        //Catch all exceptions for now and
        //treat as pass on to error handling to ensure neater closedown
        m_last_error="Error in (File: "+ filename_base(get_file()) + " Phase: "+string(get_phase_text())
                     +") with unknown exception type";
        log(m_last_error,ERRMSG);
        iret=-1;
    }
    // retrieve parameter data
    // pop member data
    delete m_ContextDataStack.top();
    m_ContextDataStack.pop();
    return iret;
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc Log a message saying that a thread is starting
void TEThread::LogStartMessage()
{
    string msg="Start Thread "+GetThreadID();
    Repository->LogMessage(msg,AUDITMSG);
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc Log a message saying that a thread is ending
void TEThread::LogEndMessage()
{
    string msg="End Thread "+GetThreadID();
    Repository->LogMessage(msg,AUDITMSG);
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Terminates the current test case, as either passed or failed depending
//      on the status of the flag passed in.
//
// @rdesc   integer 0 if the test case is terminated successfully, otherwise
//      false.
// @parm    int | result | Flag determining the state of termination
//      (TOF_TCTRES_PASS/FAIL).
// @parm    description of TestCase passed or failed (taken from start test case
//      record at prepare time).
int TEThread::TCterminate(int result,string desc)
{
    if(result==TOF_TCTRES_FAIL)
        m_TestCaseFail++;
    else
        m_TestCasePass++;
    return TContext::TCterminate(result,desc);
}
