// TestInfo.cpp : Defines the entry point for the console application.
//
#include <iostream>
#include <fstream>
#include <map>
using namespace std;
#include "TestInitiator.h"
#include <TestExecuter.h>
#include "ctime.hpp"

typedef map<string, int, less<string> > CountMap;

void usage();
int findTest(string testName);
int findAllTests();
int scriptMon(string ID, string FileName, int nRefresh);
void writeThreadInfo(const ThreadInfo& TI,string strNode,string strTest, ofstream* outFile);
void updateLevel(TestLevel_var TL, ofstream* outFile);
int getSummary(string id, string FileName);
int scriptCount(string id, string FileName, string testName);
void scriptCountLevel(TestLevel_var TL, ofstream* outFile, string testName);

int main(int argc, char* argv[])
{
	int nRet = 0;

	BOOL bMonitor= false;
	BOOL bSummary = false;
	BOOL bScriptCount = false;
	string testName;
	string outFile;
	int nRefresh = 5;
	string id;

	if (argc == 1)
	{
		usage();
		return 1;
	}

    for(int i=1;i<argc;i++)
    {
        if(strcmp(argv[i],"-F")==0)             // Node to run on
        {
            if(i+1<argc)testName=argv[++i];
			nRet = findTest(testName);
        }
        else if(strcmp(argv[i],"-A")==0)             // Node to run on
        {
			nRet = findAllTests();
        }
        else if(strcmp(argv[i],"-S")==0)             // Node to run on
        {
			bSummary = true;
        }
        else if(strcmp(argv[i],"-M")==0)             // Node to run on
        {
			bMonitor = true;
        }
        else if(strcmp(argv[i],"-E")==0)             // Node to run on
        {
			bScriptCount = true;
        }
        else if(strcmp(argv[i],"-T")==0)             // Node to run on
        {
            if(i+1<argc)testName=argv[++i];
        }
        else if(strcmp(argv[i],"-I")==0)             // Node to run on
        {
            if(i+1<argc)id=argv[++i];
        }
        else if(strcmp(argv[i],"-O")==0)             // Node to run on
        {
            if(i+1<argc)outFile=argv[++i];
        }
        else if(strcmp(argv[i],"-R")==0)             // Node to run on
        {
			string strRefresh;
            if(i+1<argc)strRefresh=argv[++i];

			nRefresh = atoi(strRefresh.c_str());
        }
        else
        {
            usage();
            nRet = 1;
        }
		
	}
	
	if (bMonitor && id!="" && outFile!="")
	{
		cout<<"Starting Script Monitor"<<endl;
		nRet = scriptMon(id, outFile, nRefresh);

	}
	if (bSummary && id!="" && outFile!="")
	{
		nRet = getSummary(id, outFile);

	}
	if (bScriptCount && id!="" && outFile!="" && testName!="")
	{
		nRet = scriptCount(id, outFile, testName);

	}


	return nRet;
}


void usage()
{
	cout<<"CommandLine:										 Purpose"<<endl;
	cout<<"============										 ======="<<endl;
	cout<<"TestInfo -F TestName								 Search for active tests of a given name."<<endl;
	cout<<"TestInfo -A										 Search for all active tests."<<endl;
	cout<<"TestInfo -M -I [id] -O [ouptutfile] -R [refresh]	 Monitor Thread Informaion for given test."<<endl;
	cout<<"TestInfo -S -I [id] -O [ouptutfile]				 Summarize users for each subtest"<<endl;
	cout<<"TestInfo -E -I [id] -O [ouptutfile] -T [testname] Summarize scripts for each subtest"<<endl;
}

int findTest(string TestName)
{
	int nRet = 0;

	try
	{
		CORBA::Object_var obj;
		CORBAServer* corba=new CORBAServer ( 0, NULL );
		CORBAServer::NameList_t TMList=corba->GetListOfNames(TestManager::CLASS_NAME);
		for (std::vector<std::string>::size_type iter=0;iter<TMList.size();iter++)  
		{                                                                           
                obj=corba->FindObjectWithName(TestManager::CLASS_NAME, TMList[iter].Id.c_str()); 
                TestManager_var TM=TestManager::_narrow(obj.inout());
				TestLevel_var TL = TM->GetTestLevel();

				if(TL->TestName() == TestName)
				{
					cout<<"TM="<<TMList[iter].Id.c_str()<<endl;
				}
		}
	}
	catch(...)
	{
		nRet = 1;
	}
	return nRet;
}

int findAllTests()
{
	int nRet = 0;

	try
	{
		CORBA::Object_var obj;
		CORBAServer* corba=new CORBAServer ( 0, NULL );
		CORBAServer::NameList_t TMList=corba->GetListOfNames(TestManager::CLASS_NAME);
		for (std::vector<std::string>::size_type iter=0;iter<TMList.size();iter++)  
		{                                                                           
            obj=corba->FindObjectWithName(TestManager::CLASS_NAME, TMList[iter].Id.c_str()); 
            TestManager_var TM=TestManager::_narrow(obj.inout());
			TestLevel_var TL = TM->GetTestLevel();

			cout<<"TEST="<<TL->TestName()<<",TM="<<TMList[iter].Id.c_str()<<endl;
		}
	}
	catch(...)
	{
		nRet = 1;
	}
	return nRet;
}

int scriptMon(string ID, string FileName, int nRefresh)
{
	int nRet = 0;
	try
	{
		CORBA::Object_var obj;
		CORBAServer* corba=new CORBAServer ( 0, NULL );

		cout<<"Creating output file"<<endl;

	    ofstream* outFile = new ofstream( FileName.c_str(), ios::binary|ios::out);
		if( !outFile->good() )
			return -1;
		cout<<"File Created!"<<endl;

		cout<<"Connecting to TestLevel"<<endl;
        obj=corba->FindObjectWithName(TestManager::CLASS_NAME, ID.c_str()); 
        TestManager_var TM=TestManager::_narrow(obj.inout());
		TestLevel_var TL = TM->GetTestLevel();
		cout<<"Connected"<<endl;

		while(TL)
		{
			// update information about this level
			updateLevel(TL, outFile);

			// update information about child levels
			TestLevelList TLList;
			long nTLS = TL->GetChildTestLevelList(TLList);
			cout<<nTLS<<" Child TestLevels";
			long TLiter;
			TestLevel_var CL;
		    for(TLiter=0;TLiter<TLList.length();TLiter++)
		    {
				CL = TLList[TLiter];
				cout<<"Found test level"<<endl;
				updateLevel(CL, outFile);
			}


			Sleep(nRefresh*1000);
		}


		delete outFile;
	}
	catch(...)
	{
		cout
			<<"Exception!!!"<<endl;
		nRet = 1;
	}
	return nRet;
}

void updateLevel(TestLevel_var TL, ofstream* outFile)
{
	TestExecuterList TEList;
	unsigned long no_executers = TL->GetTestExecuterList(TEList);
	cout<<no_executers<<" Executer(s)."<<endl;

	long TEiter;
	for(TEiter=0;TEiter<TEList.length();TEiter++)
	{
		try
		{
			cout<<"Found an executer of type "<<TEList[TEiter]->GetTarget()<<endl;
			string strNode = TEList[TEiter]->GetNode();
			string strTest = TEList[TEiter]->TestName();

		    ThreadInfoList TIList;
		    CORBA::ULong no_threads=TEList[TEiter]->GetThreadInformation(TIList);
			cout<<"NumThreads = "<<no_threads<<endl;
		    for(long TIiter=0;TIiter<TIList.length();TIiter++)
		    {
				 cout<<"Writing ThreadInfo"<<endl;
		         writeThreadInfo(TIList[TIiter],strNode,strTest,outFile);
				 cout<<"Done!"<<endl;
		    }
		}
		catch(...)
		{
		}
	}
}

void writeThreadInfo(const ThreadInfo& TI,string strNode,string strTest, ofstream* outFile)
{
	string strScript = TI.ScriptName;

	// Strip off script path.
	int nLastSlashPos = strScript.find_last_of('\\');
	if (nLastSlashPos != -1)
	{
	    strScript.erase(0,nLastSlashPos+1);
	}

	string strUIdent = TI.UIdent;
	int nPos = strUIdent.find_last_of('-');
	if (nPos != -1)
	{
	    strUIdent.erase(0,nPos+1);
	}

	string strNoteText = TI.NoteText;
	CTTime StartTime(TI.StartTime);
	string TimeString = StartTime.Format("%H:%M:%S %d/%m/%Y" );

	outFile->write(strUIdent.c_str(), strUIdent.length());
	outFile->write(",",1);
	outFile->write(strScript.c_str(), strScript.length());
	outFile->write(",",1);
	outFile->write(strNode.c_str(), strNode.length());
	outFile->write(",",1);
	outFile->write(strNoteText.c_str(), strNoteText.length());
	outFile->write(",",1);
	outFile->write(TimeString.c_str(), TimeString.length());
	outFile->write("\r\n",2);

	outFile->flush();
}

int getSummary(string id, string FileName)
{
	int nRet =0;
    ofstream outFile( FileName.c_str(), ios::binary|ios::out);

	try
	{
		CORBA::Object_var obj;
		CORBAServer* corba=new CORBAServer ( 0, NULL );

		if( !outFile.good() )
			return -1;

        obj=corba->FindObjectWithName(TestManager::CLASS_NAME, id.c_str()); 
        TestManager_var TM=TestManager::_narrow(obj.inout());
		TestLevel_var TL = TM->GetTestLevel();

		while(TL)
		{
			// update information about child levels
			TestLevelList TLList;
			long nTLS = TL->GetChildTestLevelList(TLList);
			long TLiter;
			TestLevel_var CL;
		    for(TLiter=0;TLiter<TLList.length();TLiter++)
		    {
				CL = TLList[TLiter];
				outFile<<CL->TestName()<<","<<CL->NoChildThreads(true)<<"\r\n";
				outFile.flush();
			}
			Sleep(5000);
		}
	}
	catch(...)
	{
		cout
			<<"Exception!!!"<<endl;
		nRet = 1;
	}
	outFile.close();
	return nRet;
}

int scriptCount(string id, string FileName, string testName)
{
	int nRet = 0;
	try
	{
		CORBA::Object_var obj;
		CORBAServer* corba=new CORBAServer ( 0, NULL );

		cout<<"Creating output file"<<endl;

	    ofstream* outFile = new ofstream( FileName.c_str(), ios::binary|ios::out);
		if( !outFile->good() )
			return -1;
		cout<<"File Created!"<<endl;

		cout<<"Connecting to TestLevel"<<endl;
        obj=corba->FindObjectWithName(TestManager::CLASS_NAME, id.c_str()); 
        TestManager_var TM=TestManager::_narrow(obj.inout());
		TestLevel_var TL = TM->GetTestLevel();
		cout<<"Connected"<<endl;

		while(TL)
		{
			// update information about child levels
			TestLevelList TLList;
			long nTLS = TL->GetChildTestLevelList(TLList);
			cout<<nTLS<<" Child TestLevels";
			long TLiter;
			TestLevel_var CL;
		    for(TLiter=0;TLiter<TLList.length();TLiter++)
		    {
				CL = TLList[TLiter];
				cout<<"Found test level"<<endl;
				scriptCountLevel(CL, outFile, testName);
			}


			Sleep(5000);
		}


		delete outFile;
	}
	catch(...)
	{
		cout
			<<"Exception!!!"<<endl;
		nRet = 1;
	}
	return nRet;
}

void scriptCountLevel(TestLevel_var TL, ofstream* outFile, string testName)
{
	TestExecuterList TEList;
	unsigned long no_executers = TL->GetTestExecuterList(TEList);
	cout<<no_executers<<" Executer(s)."<<endl;

	long TEiter;
	for(TEiter=0;TEiter<TEList.length();TEiter++)
	{
		try
		{
			cout<<"Found an executer of type "<<TEList[TEiter]->GetTarget()<<endl;
			string strTest = TEList[TEiter]->TestName();
			if (strTest == testName)
			{
			    CountMap theMap;
				CountMap::iterator theIterator;

			    ThreadInfoList TIList;
			    CORBA::ULong no_threads=TEList[TEiter]->GetThreadInformation(TIList);
				cout<<"NumThreads = "<<no_threads<<endl;
			    for(long TIiter=0;TIiter<TIList.length();TIiter++)
			    {
					string strScript = TIList[TIiter].ScriptName;
					theIterator = theMap.find(strScript);
					if(theIterator != theMap.end())
					{
						(*theIterator).second = (*theIterator).second +1;
					}
					else
					{
						theMap.insert(CountMap::value_type(strScript,1));
					}
			    }

			    for ( theIterator = theMap.begin(); theIterator != theMap.end(); theIterator++ )
				{
					string strScript = theIterator->first;
					int nScripts = theIterator->second;
					char buf[200];
					int nLen = sprintf(buf, ",%d\r\n",nScripts);
					outFile->write(strScript.c_str(), strScript.length());
					outFile->write(buf,nLen);
					outFile->flush();
				}


			}
		}
		catch(...)
		{
		}
	}
}

