/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     ErrorCodes
// @module  Cyrano Error Codes | This module contains a complet list of error
//          codes that may be generated by Cyrano applications.  These values
//          may appear as possible return values from any of the Cyrano API
//          calls and are also used in any Cyrano defined exceptions. If the
//          error code you require is not here then add it.  To prevent clashes
//          do not add them anywhere else and ensure that the one you add does
//          not clash with an existing value.
//
//          The codes are currently in numeric order with some vague grouping
//          applied.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _CyrError_h
#define _CyrError_h

#define SEPARATOR '|'

///////////////////////////////////////////////////////////////////////////////
// Macros used for thread and timer synchronisation
const int ONE_SECOND    = 1000;
const int TEN_SECONDS   = 10 * ONE_SECOND;
const int ONE_MINUTE    = 60 * ONE_SECOND;
const int TEN_MINUTES   = 10 * ONE_MINUTE;

///////////////////////////////////////////////////////////////////////////////
// Misc.
const int CYR_UNKNOWN_ERROR             =     0;
const int CYR_SUCCESS                   =     1;

// JNI error codes
const int CYR_JAVA_CANNOT_START_JVM     = -2000;
const int CYR_JAVA_CANNOT_FIND_API      = -2001;
const int CYR_JAVA_CANNOT_FIND_METHOD   = -2002;
const int CYR_JAVA_CANNOT_CREATE_OBJECT = -2003;
const int CYR_JAVA_NULL_OBJECT          = -2004;
const int CYR_JAVA_CANNOT_FIND_JVM      = -2005;
const int CYR_JAVA_CANNOT_ATTACH_JVM    = -2006;
const int CYR_JAVA_CANNOT_DETACH_JVM    = -2007;

// OpenSTARep error codes
const int CYR_SERVER_CONNECT_FAILED     = -2100;
const int CYR_SERVER_NOT_FOUND          = -2101;

const int CYR_FILE_LOCK_FAILED          = -2110;
const int CYR_FILE_NOT_FOUND            = -2111;
const int CYR_FILE_ADD_FAILED           = -2112;

const int CYR_PROJECT_NOT_FOUND         = -2120;
const int CYR_PROJECT_NEW_FAILED        = -2121;
const int CYR_PROJECT_WRONG_TYPE        = -2122;

const int CYR_FOLDER_NOT_FOUND          = -2130;
const int CYR_FOLDER_NEW_FAILED         = -2131;

// CORBA error codes
const int CYR_CORBA_INIT_FAILED         = -2200;
const int CYR_CORBA_EXIT_FAILED         = -2201;
const int CYR_CORBA_EXCEPTION           = -2202;
const int CYR_CORBA_OBJECT_NOT_FOUND    = -2203;
const int CYR_CORBA_OBJECT_NOT_READY    = -2204;
const int CYR_CORBA_NARROW_FAILED       = -2205;
const int CYR_CORBA_OBJECT_INVALID      = -2206;

// Global data exception status errors
const int CYR_CORBA_OUT_OF_RANGE        = -2250;
const int CYR_CORBA_INVALID_ENTRY       = -2251;

// Script errors
const int CYR_SCRIPT_INVALID            = -2300;

#endif
