#ifndef PSL__DIAGSTKDEF_LOADED
#define PSL__DIAGSTKDEF_LOADED 1

/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*               **** C O M M O N ****                   */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: DIAGSTKDEF                  */
/*          ============                         */
/*                                       */
/*  Description:                                 */
/*  ============                                 */
/*                                       */
/*  This module defines the Diagnostic Stack data structure          */
/*  used by the Diagnostic Stack routines.                   */
/*                                       */
/*                                       */
/*===========================================================================*/

/*** MODULE DIAGSTKDEF ***/

struct diagstk
{

    /*  Structure Type */

    /*  This field identifies the structure as a screen image. */

    PSL_U_LONGWORD_T        diag_strtypl; /* Structure Type       */


    /*  The previous field contains this constant value.  Note that */
    /*  this is held as a hexadecimal number to improve performance, */
    /*  but the precise value varies from platform to platform as */
    /*  necessary to represent the ASCII string "DIAG". */

#ifdef LITTLE_ENDIAN
#define            DIAG_STRNAM 0X47414944   /* = "DIAG" */
#else  /* LITTLE_ENDIAN */
#define            DIAG_STRNAM 0X44494147   /* = "DIAG" */
#endif /* LITTLE_ENDIAN */

    /*  Number of Entries in Diagnostic Stack */

#define            DIAG_STKSIZ 100  /* Stack Size as Parameter */

    /*  Diagnostic Stack itself. */

    char                    *diag_stackpc100[100];

    /*  Diagnostic Stack Flags. */

    PSL_U_LONGWORD_T        diag_flagsl100[100];
#define            DIAGFL_STAT 0X1  /* Message is Static    */

    /*  Current Subscript. */

    /*  This is the subscript (starting at 1) of the most recent */
    /*  diagnostic added to the stack. */

    PSL_LONGWORD_T          diag_cursubl;

    /*  Current Subscript Bias. */

    /*  This is the bias that must be added to each subscript in */
    /*  the array to calculate the associated diagnostics context. */

    PSL_LONGWORD_T          diag_biasl;

    /*  Start of Latest Error. */

    /*  This is the subscript of the first entry on the stack that */
    /*  corresponds to the most recent error logged via PSL_PUTDIAG */
    /*  (or similar). */

    PSL_LONGWORD_T          diag_lsterrl;

    /*  Error Trap list head. */

    /*  This is used by the Error Trapping mechanisms and is tied */
    /*  into the Diagnostics Stack to ensure it gets reset when a */
    /*  routine exits. */

    struct errtrap          *diag_errtrpps;

    /*  Signalled Error Number */

    /*  This contains the message number of the most recent error */
    /*  signalled via PSL_SIGNAL_ERROR. */

    PSL_U_LONGWORD_T        diag_signuml;

    /*  Signalled Error Class */

    /*  This contains the message class of the most recent error */
    /*  signalled via PSL_SIGNAL_ERROR. */

    PSL_U_LONGWORD_T        diag_sigclsl;

    /*  Output Types used by PSL_OUTPUT_DIAG. */

    /*  This defines the type of output that PSL_OUTPUT_DIAG should */
    /*  perform. */

    PSL_U_LONGWORD_T        diag_outtypl;
#define            DIAG_OUTTYP_LIST 0X1 /* Listing File     */
#define            DIAG_OUTTYP_TCB 0X2  /* TCB          */
#define            DIAG_OUTTYP_USER 0X4 /* User defined     */
#define            DIAG_OUTTYP_WRAP_LIST 0X8    /* Wrap Listing Messages*/
#define            DIAG_OUTTYP_WRAP_TCB 0X10    /* Wrap TCB Messages    */
#define            DIAG_OUTTYP_WRAP_USER 0X20   /* Wrap User Messages   */

    /*  Pointer to Listing Context Block Structure. */

    /*  This is only used if the output type DIAG_OUTTYP_LIST is */
    /*  set in the OUTTYP field. */

    struct list_ctx         *diag_list_ctx_ptrps;

    /*  Pointer to Listing Action Routine used by PSL_OUTPUT_DIAG. */

    /*  This is only used if the output type DIAG_OUTTYP_LIST is */
    /*  set in the OUTTYP field. */

    PSL_U_LONGWORD_T        (*diag_list_actrtnpf)
    (struct list_ctx *context_ptr,
     const char *errbuf,
     unsigned long numlin,
     unsigned long width)
    ;

    /*  Output Width used by PSL_OUTPUT_DIAG. */

    /*  This defines the width that each line should be padded to or */
    /*  wrapped at in PSL_OUTPUT_DIAG when outputting to a listing. */
    /*  It is only used if the output type DIAG_OUTTYP_LIST is */
    /*  set in the OUTTYP field. */

    PSL_LONGWORD_T          diag_list_outwidl;

    /*  Pointer to TCB Structure. */

    /*  This is only used if the output type DIAG_OUTTYP_TCB is set */
    /*  in the OUTTYP field. */

    struct tcb              *diag_tcb_ptrps;

    /*  Pointer to TCB Action Routine used by PSL_OUTPUT_DIAG. */

    /*  This is only used if the output type DIAG_OUTTYP_TCB is set */
    /*  in the OUTTYP field. */

    PSL_U_LONGWORD_T        (*diag_tcb_actrtnpf)
    (struct tcb *tcb_ptr,
     char *errbuf,
     long numlin,
     long width)
    ;

    /*  Output Width used by PSL_OUTPUT_DIAG. */

    /*  This defines the width that each line should be padded to or */
    /*  wrapped at in PSL_OUTPUT_DIAG when outputting to the terminal. */
    /*  It is only used if the output type DIAG_OUTTYP_TCB is */
    /*  set in the OUTTYP field. */

    PSL_LONGWORD_T          diag_tcb_outwidl;

    /*  Pointer to User Output Type Structure. */

    /*  This is only used if the output type DIAG_OUTTYP_USER is */
    /*  set in the OUTTYP field. */

    void                    *diag_user_ptrp;

    /*  Pointer to User Action Routine used by PSL_OUTPUT_DIAG. */

    /*  This is only used if the output type DIAG_OUTTYP_USER is */
    /*  set in the OUTTYP field. */

    PSL_U_LONGWORD_T        (*diag_user_actrtnpf)
    (void *output_ptr,
     const char *errbuf,
     unsigned long numlin,
     unsigned long outwid)
    ;

    /*  Output Width used by PSL_OUTPUT_DIAG. */

    /*  This defines the width that each line should be padded to or */
    /*  wrapped at in PSL_OUTPUT_DIAG when outputting to a user routine. */
    /*  It is only used if the output type DIAG_OUTTYP_USER is */
    /*  set in the OUTTYP field. */

    PSL_LONGWORD_T          diag_user_outwidl;

    /*  Padding for quadword alignment. */

    char                    diag_paddingb4[4];

} ;


/*  End of C Include File */

#endif  /* PSL__DIAGSTKDEF_LOADED */
