/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECGridCtrl.cpp: implementation of the CECGridCtrl class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CECGridCtrl.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
BEGIN_MESSAGE_MAP(CECGridCtrl, CGridCtrl)
//{{AFX_MSG_MAP(CECGridCtrl)
ON_WM_LBUTTONUP()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CECGridCtrl::CECGridCtrl()
{
    m_BtnDataBase.SetGrid( this );
}

CECGridCtrl::~CECGridCtrl()
{
}


void CECGridCtrl::CreateSpinCell(int row, int col, BOOL bNumeric)
{
    int iCtlWidth = GetSystemMetrics( SM_CXVSCROLL);

    SetCell( row, col, new CECSpinEditCell);//RUNTIME_CLASS(CGridBtnCell) );
    CECSpinEditCell* pGridBtnCell = (CECSpinEditCell*)GetCell( row, col);
	pGridBtnCell->SetNumberOnly(bNumeric?true:false);

    // Add the UP ARROW of the spin control
    pGridBtnCell->SetBtnDataBase( &m_BtnDataBase);
    pGridBtnCell->SetupBtns(
        0,        // zero-based index of image to draw
        DFC_SCROLL, // type of frame control to draw e.g. DFC_BUTTON
        DFCS_SCROLLUP, // like DrawFrameControl()'s nState  e.g. DFCS_BUTTONCHECK
        CGridBtnCellBase::CTL_ALIGN_RIGHT,
        // horizontal alignment of control image
        iCtlWidth,                         // fixed width of control or 0 for size-to-fit
        FALSE,  // T=btn is member of a radio group
        NULL );   // Text to insert centered in button; if NULL no text

    // Add the DOWN ARROW of the spin control
    pGridBtnCell->SetBtnDataBase( &m_BtnDataBase);
    pGridBtnCell->SetupBtns(
        1,        // zero-based index of image to draw
        DFC_SCROLL, // type of frame control to draw e.g. DFC_BUTTON
        DFCS_SCROLLDOWN, // like DrawFrameControl()'s nState  e.g. DFCS_BUTTONCHECK
        CGridBtnCellBase::CTL_ALIGN_RIGHT,
        // horizontal alignment of control image
        iCtlWidth,                         // fixed width of control or 0 for size-to-fit
        FALSE,  // T=btn is member of a radio group
        NULL );   // Text to insert centered in button; if NULL no text

    // Zero the spin control.
    pGridBtnCell->SetText("0");
}


// Handle messages from the custom cell controls
void CECGridCtrl::OnLButtonUp(UINT nFlags, CPoint point)
{
    CGridCtrl::OnLButtonUp(nFlags, point);

    CCellID cellInfo = GetCellFromPt(point);

    CGridCellBase* pGridCell = (CGridCellBase*)GetCell( cellInfo.row, cellInfo.col);
    if( pGridCell)
    {
        if( pGridCell->IsKindOf( RUNTIME_CLASS(CGridBtnCellCombo) ) )
        {
            // Handle any messages for the combo box
        }

        else if( pGridCell->IsKindOf( RUNTIME_CLASS(CGridBtnCell) ) )
        {
            CGridBtnCell* pGridCellBtn = (CGridBtnCell*)pGridCell;

            UINT type = pGridCellBtn->GetDrawCtlType(0);

            switch (type)
            {
            case DFC_SCROLL:
                {
                    // Handle any messages for the spin control
                    HandleSpinMsg(pGridCellBtn);
                    break;
                }
            case DFC_BUTTON:
                {
                    // Handle any messages for the button
                    HandleButtonMsg(pGridCellBtn,cellInfo.row, cellInfo.col);
                    break;
                }
            }
        }
    }

    RedrawCell( cellInfo.row, cellInfo.col);
}

// Update the text in the connecting edit field as the spin control has just been used!
void CECGridCtrl::HandleSpinMsg(CGridBtnCell *pCell)
{
    int iOffset;
    char* pszStop;
    unsigned long lNbr;

    if( pCell->GetLastCtlClicked() < 0)
        return;

    if( pCell->GetLastCtlClicked() == 0) // Ctrl 0 == UP ARROW of spin ctrl
        iOffset = 1;
    else                                 // Ctrl 1 == DOWN ARROW
        iOffset = -1;

    // increment / decrement the number
    CString strMsg = pCell->GetText();
    lNbr = strtol( strMsg, &pszStop, 10 );
    lNbr += iOffset;
    strMsg.Format( "%i", lNbr);

    pCell->SetText( strMsg);
}

void CECGridCtrl::CreateListCell(int nRow, int nCol, const CStringArray& list)
{
    int iCtlWidth = GetSystemMetrics( SM_CXVSCROLL);

    SetCellType( nRow, nCol, RUNTIME_CLASS(CGridBtnCellCombo) );
    CGridBtnCellCombo* pGridBtnCellCombo = (CGridBtnCellCombo*)GetCell( nRow, nCol);

    pGridBtnCellCombo->SetComboStyle( CBS_DROPDOWN );
    pGridBtnCellCombo->SetComboString( list );
    pGridBtnCellCombo->SetBtnDataBase( &m_BtnDataBase);

    RedrawCell(nRow, nCol);
}

void CECGridCtrl::CreateCheckCell(int nRow, int nCol)
{
    SetCellType( nRow, nCol, RUNTIME_CLASS(CGridBtnCell) );
    CGridBtnCell* pGridBtnCell = (CGridBtnCell*)GetCell( nRow, nCol);

    // tell new cell where to get its property data
    pGridBtnCell->SetBtnDataBase( &m_BtnDataBase);
    pGridBtnCell->SetupBtns(
        0,        // zero-based index of image to draw
        DFC_BUTTON, // type of frame control to draw e.g. DFC_BUTTON
        DFCS_BUTTONCHECK, // like DrawFrameControl()'s nState  e.g. DFCS_BUTTONCHECK
        CGridBtnCellBase::CTL_ALIGN_CENTER,
        // horizontal alignment of control image
        0,                         // fixed width of control or 0 for size-to-fit
        FALSE,  // T=btn is member of a radio group
        NULL );   // Text to insert centered in button; if NULL no text

    RedrawCell(nRow, nCol);
}


void CECGridCtrl::SetCheckBoxCellState(int nRow, int nCol, BOOL bCheck)
{
    // Get the current state of the check box
    CGridBtnCell* pGridBtnCell = (CGridBtnCell*)GetCell( nRow, nCol);
    UINT uiState = pGridBtnCell->GetDrawCtlState(0);

    // If the current state is already correct just return
    if( GetCheckBoxCellState(nRow, nCol) && bCheck)
        return;
    if( !GetCheckBoxCellState(nRow, nCol) && !bCheck)
        return;

    // Toggle the check state
    if( uiState & DFCS_CHECKED)
    {
        uiState &= (~DFCS_CHECKED); // uncheck
    }

    else
    {
        uiState |= DFCS_CHECKED;    // check
    }

    // Update the state to the cell and redraw.

    pGridBtnCell->SetDrawCtlState(0, uiState);
    RedrawCell(nRow, nCol);
}

BOOL CECGridCtrl::GetCheckBoxCellState(int nRow, int nCol)
{
    // Get the current state of the check box
    CGridBtnCell* pGridBtnCell = (CGridBtnCell*)GetCell( nRow, nCol);
    UINT uiState = pGridBtnCell->GetDrawCtlState(0);

    // If the current state is already correct just return

    BOOL bRet = FALSE;
    if( uiState & DFCS_CHECKED)
        bRet = TRUE;
    else
		bRet = FALSE;

    return bRet;
}

// Can add a large amaount of text to the grid in one go! Use '\t' as the colummn
// separator and '\n' at the row separator. Specify the start cell where the text
// will be be added. This function will insert rows/columns where necessary to
// accomodate all of the text.
void CECGridCtrl::PasteTextFromBuffer(int startRow, int startCol, CString text)
{
    int nRows = GetRowCount();
    int nCols = GetColumnCount();

    CString line = "";
    int nextRow = startRow;
    int nextCol = startCol;
    while (GetNextField('\n', text, line) != -1)
    {
        CString field = "";
        while(GetNextField('\t', line, field) != -1)
        {
            while(nRows = GetRowCount() <= nextRow)
            {
                InsertRow("");
            }
            while(nCols = GetColumnCount() <= nextCol)
            {
                InsertColumn("");
            }

            SetItemText(nextRow, nextCol, field);
            nextCol++;
        }
        nextCol = startCol;
        nextRow++;
    }


}


// String parsing function
int CECGridCtrl::GetNextField(char sep, CString &inStr, CString &field)
{
    if (inStr=="")
    {
        return -1;
    }

    int len = inStr.GetLength();
    int nPos = inStr.Find(sep,0);

    if (nPos == -1)
    {
        field = inStr;
        inStr="";
        return 0;
    }
    else
    {
        field = inStr.Left(nPos);
        inStr=inStr.Right(len-nPos-1);
        return 0;
    }
}

// Colour the text of all cells in the column
void CECGridCtrl::SetColumnTextClr( int col, COLORREF rgb)
{
    for (int i = 0; i<GetRowCount(); i++)
    {
        CGridBtnCell* pCell = (CGridBtnCell*)GetCell( i, col);
        pCell->SetTextClr(rgb);
    }
}

void CECGridCtrl::CreateButtonCell(int nRow, int nCol, CString btnText, ALIGN align)
{
    SetCellType( nRow, nCol, RUNTIME_CLASS(CGridBtnCell) );
    CGridBtnCell* pGridBtnCell = (CGridBtnCell*)GetCell( nRow, nCol);

    CGridBtnCellBase::CTL_ALIGN ctlAlign;
    int iCtlWidth = 0;
    switch (align)
    {
    case ALIGN_CENTER:
        {
            ctlAlign = CGridBtnCellBase::CTL_ALIGN_CENTER;
            iCtlWidth = 0;
            break;
        }
    case ALIGN_LEFT:
        {
            ctlAlign = CGridBtnCellBase::CTL_ALIGN_LEFT;
            iCtlWidth = GetSystemMetrics( SM_CXVSCROLL);
            break;
        }
    case ALIGN_RIGHT:
        {
            ctlAlign = CGridBtnCellBase::CTL_ALIGN_RIGHT;
            iCtlWidth = GetSystemMetrics( SM_CXVSCROLL);
            break;
        }
    }

    // tell new cell where to get its property data
    pGridBtnCell->SetBtnDataBase( &m_BtnDataBase);
    pGridBtnCell->SetupBtns(
        0,        // zero-based index of image to draw
        DFC_BUTTON, // type of frame control to draw e.g. DFC_BUTTON
        DFCS_BUTTONPUSH, // like DrawFrameControl()'s nState  e.g. DFCS_BUTTONCHECK
        ctlAlign,
        // horizontal alignment of control image
        iCtlWidth,                         // fixed width of control or 0 for size-to-fit
        FALSE,  // T=btn is member of a radio group
        btnText );   // Text to insert centered in button; if NULL no text

    RedrawCell(nRow, nCol);
}

void CECGridCtrl::HandleButtonMsg(CGridBtnCell *pCell, int nRow, int nCol)
{
    if( pCell->GetLastCtlClicked() < 0)
        return;
    GetParent()->SendMessage(WMU_CELL_BUTTON_PRESSED, nRow, nCol);
}

void CECGridCtrl::SetCellEnabled(int nRow, int nCol, BOOL bEnabled)
{
    CGridBtnCell* pGridBtnCell = (CGridBtnCell*)GetCell( nRow, nCol);

    // Set the Base Cell state to read only
    DWORD state = pGridBtnCell->GetState();
    if (!bEnabled)
    {
        state |= (GVIS_READONLY);
    }
    else
    {
        state &= (~GVIS_READONLY);
    }
    pGridBtnCell->SetState(state);
    /*
        // Set any button controls inactive
        int i =-1;
        while (i<pGridBtnCell->GetDrawCtlNbrMax())
        {
            i++;
            UINT state = pGridBtnCell->GetDrawCtlState(i);
            if (!bEnabled)
            {
                state |= (DFCS_INACTIVE);
            }
            else
            {
                state &= (~DFCS_INACTIVE);
            }
            pGridBtnCell->SetDrawCtlState(i, state);
        }
    */
    RedrawCell(nRow, nCol);
}

BOOL CECGridCtrl::GridCtrlInit(HINSTANCE hInst)
{
    //Need to do this provide this function to register the window class
    //otherwise the gridctrl will not appear  in windows 2000 (copied from GridCtrl.cpp in CECGridLib)
    WNDCLASS wndcls;
    if (!(::GetClassInfo(hInst, GRIDCTRL_CLASSNAME, &wndcls)))
    {
        // otherwise we need to register a new class
        wndcls.style            = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
        wndcls.lpfnWndProc      = ::DefWindowProc;
        wndcls.cbClsExtra       = wndcls.cbWndExtra = 0;
        wndcls.hInstance        = hInst;
        wndcls.hIcon            = NULL;
#ifndef _WIN32_WCE_NO_CURSOR
        wndcls.hCursor          = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
#else
        wndcls.hCursor          = 0;
#endif
        wndcls.hbrBackground    = (HBRUSH) (COLOR_3DFACE + 1);
        wndcls.lpszMenuName     = NULL;
        wndcls.lpszClassName    = GRIDCTRL_CLASSNAME;

        if (!AfxRegisterClass(&wndcls))
        {
            AfxThrowResourceException();
            return FALSE;
        }
    }

    return TRUE;
}

