/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CommanderView.cpp : implementation of the CCommanderView class
//

#include "stdafx.h"
#include "Commander.h"

#include "CommanderDoc.h"
#include "CntrItem.h"
#include "CommanderView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCommanderView
extern CLIPFORMAT g_ClipFormat;

IMPLEMENT_DYNCREATE(CCommanderView, CView)

BEGIN_MESSAGE_MAP(CCommanderView, CView)
	//{{AFX_MSG_MAP(CCommanderView)
	ON_WM_DESTROY()
	ON_WM_SETFOCUS()
	ON_WM_SIZE()
	ON_COMMAND(ID_CANCEL_EDIT_CNTR, OnCancelEditCntr)
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCommanderView construction/destruction

CCommanderView::CCommanderView()
{
	m_pSelection = NULL;
	// TODO: add construction code here

}

CCommanderView::~CCommanderView()
{
}

BOOL CCommanderView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CCommanderView drawing

void CCommanderView::OnDraw(CDC* pDC)
{
	CCommanderDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
	// TODO: also draw all OLE items in the document

	// Draw the selection at an arbitrary position.  This code should be
	//  removed once your real drawing code is implemented.  This position
	//  corresponds exactly to the rectangle returned by CCommanderCntrItem,
	//  to give the effect of in-place editing.

	// TODO: remove this code when final draw code is complete.
	CRect rcClient;
	GetClientRect(&rcClient);

	if (m_pSelection == NULL)
	{
		POSITION pos = pDoc->GetStartPosition();
		m_pSelection = (CCommanderCntrItem*)pDoc->GetNextClientItem(pos);
	}
	if (m_pSelection != NULL && 
		m_pSelection->IsRunning() && 
		m_pSelection->IsInPlaceActive())
	{
		m_pSelection->Draw(pDC, rcClient);
	}
}

void CCommanderView::OnInitialUpdate()
{
	CView::OnInitialUpdate();

	
	// TODO: remove this code when final selection model code is written
	m_pSelection = NULL;    // initialize selection

}

/////////////////////////////////////////////////////////////////////////////
// CCommanderView printing

BOOL CCommanderView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CCommanderView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CCommanderView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

void CCommanderView::OnDestroy()
{
	// Deactivate the item on destruction; this is important
	// when a splitter view is being used.
   CView::OnDestroy();
   COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
   if (pActiveItem != NULL && pActiveItem->GetActiveView() == this)
   {
		pActiveItem->Deactivate();
		ASSERT(GetDocument()->GetInPlaceActiveItem(this) == NULL);
   }
}


/////////////////////////////////////////////////////////////////////////////
// OLE Client support and commands

BOOL CCommanderView::IsSelected(const CObject* pDocItem) const
{
	// The implementation below is adequate if your selection consists of
	//  only CCommanderCntrItem objects.  To handle different selection
	//  mechanisms, the implementation here should be replaced.

	// TODO: implement this function that tests for a selected OLE client item

	return pDocItem == m_pSelection;
}

// The following command handler provides the standard keyboard
//  user interface to cancel an in-place editing session.  Here,
//  the container (not the server) causes the deactivation.
void CCommanderView::OnCancelEditCntr()
{
	// Close any in-place active item on this view.
	COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
	if (pActiveItem != NULL)
	{
		pActiveItem->Close();
	}
	ASSERT(GetDocument()->GetInPlaceActiveItem(this) == NULL);
}

// Special handling of OnSetFocus and OnSize are required for a container
//  when an object is being edited in-place.
void CCommanderView::OnSetFocus(CWnd* pOldWnd)
{
	COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
	if (pActiveItem != NULL &&
		pActiveItem->GetItemState() == COleClientItem::activeUIState)
	{
		// need to set focus to this item if it is in the same view
		CWnd* pWnd = pActiveItem->GetInPlaceWindow();
		if (pWnd != NULL)
		{
			pWnd->SetFocus();   // don't call the base class
			return;
		}
	}

	CView::OnSetFocus(pOldWnd);
}

void CCommanderView::OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);
	COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
	if (pActiveItem != NULL)
		pActiveItem->SetItemRects(CRect(0,0,cx,cy),CRect(0,0,cx,cy));
}

/////////////////////////////////////////////////////////////////////////////
// CCommanderView diagnostics

#ifdef _DEBUG
void CCommanderView::AssertValid() const
{
	CView::AssertValid();
}

void CCommanderView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CCommanderDoc* CCommanderView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CCommanderDoc)));
	return (CCommanderDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CCommanderView message handlers
BOOL CCommanderView::ShowObject(const CString& strPath,REFCLSID clsid)
{
	//open new item
	m_strOpenItemPath = strPath;
	
	BeginWaitCursor();

	//Set the path name on the clipboard because we cannot(as yet) obtain it in the plugin 
	int nDataSize = strPath.GetLength() + 1;
	
	COleDataSource DataSource;
	HGLOBAL hData = GlobalAlloc(GMEM_FIXED|GMEM_ZEROINIT,nDataSize);
	char *szData = (char*)GlobalLock(hData);

	strcpy(szData,strPath);
	
	GlobalUnlock(hData);

	OpenClipboard();
	SetClipboardData(g_ClipFormat,hData);
	CloseClipboard();

	//Create the new server item
	CCommanderCntrItem* pItem = NULL;
	TRY
	{
		// Create new item connected to this document.
		CCommanderDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);
		pItem = new CCommanderCntrItem(pDoc);
		ASSERT_VALID(pItem);

		// Initialize the item
		if(!pItem->CreateFromFile(strPath))
			AfxThrowMemoryException();  // any exception will do

		ASSERT_VALID(pItem);

		pItem->DoVerb(OLEIVERB_SHOW, this);
		
		ASSERT_VALID(pItem);

		CRect rcClient;
		GetClientRect(&rcClient);
		pItem->SetItemRects(rcClient,rcClient);

		// As an arbitrary user interface design, this sets the selection
		//  to the last item inserted.
		m_pSelection = pItem;   // set selection to last inserted item
		pDoc->UpdateAllViews(NULL);

		GetDocument()->SetPathName(strPath,FALSE);
	}
	CATCH(CException, e)
	{
		delete pItem;
	}
	END_CATCH

	//Clear just our data from the clipboard
	OpenClipboard();
	SetClipboardData(g_ClipFormat,NULL);	
	CloseClipboard();

	EndWaitCursor();

	return true;
}

BOOL CCommanderView::OnEraseBkgnd(CDC* pDC) 
{
	if (!m_pSelection || !m_pSelection->IsRunning() || !m_pSelection->IsInPlaceActive())
	{
		CRect rcClient;
		GetClientRect(&rcClient);
		pDC->FillSolidRect(rcClient,GetSysColor(COLOR_APPWORKSPACE));

	}
	return true;
}
