/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "Commander.h"

#include "CommanderView.h"
#include "MainFrm.h"
#include "MonitorSelect.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define NEW_MENU_ID_START 0x1000
#define NEW_MENU_ID_END 0x1100

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_COMMAND(ID_TESTS_MONITOR, OnTestsMonitor)
	//}}AFX_MSG_MAP
	ON_COMMAND_RANGE(NEW_MENU_ID_START,NEW_MENU_ID_END,OnCustomMenuCommand)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame():
	m_strScriptMenu("&Scripts"),
	m_strProfileMenu("&Profiles"),
	m_strTestMenu("&Tests"),
	m_strNewMenu("&New")
{
	m_nMenuItemsAdded = 0;
	m_pCurrentDoc=NULL;
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	if (!m_wndWorkspace.Create(_T("Workspace"), this, 123))
	{
	    TRACE0("Failed to create workspace bar\n");
	    return -1;
	    // fail to create
	}
	m_wndWorkspace.SetBarStyle(m_wndWorkspace.GetBarStyle() |
	    CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);
	m_wndWorkspace.EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndWorkspace, AFX_IDW_DOCKBAR_LEFT);

	CCreateContext *pContext = (CCreateContext*)lpCreateStruct->lpCreateParams;
	CDocTemplate *pDocTemplate = pContext->m_pNewDocTemplate;
	m_MenuIP.Attach(pDocTemplate->m_hMenuInPlace);
	m_pCurrentDoc = pContext->m_pCurrentDoc;

	LoadTestPlugins();
	LoadScriptPlugins();
	LoadProfilePlugins();

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers
void CMainFrame::LoadTestPlugins()
{
	//Load information about test plugins
	int nNumTests =	LoadPluginType(enTest);
	
	//Modify standard menu to include tests
	CMenu *pMainMenu = GetMenu();
	
	CString str;
	int nNumItems = pMainMenu->GetMenuItemCount();
	for(int nNdx=0;nNdx<nNumItems;nNdx++)
	{
		pMainMenu->GetMenuString(nNdx,str,MF_BYPOSITION);
		if(str != m_strTestMenu)
			continue;
		
		if(nNumTests)
		{
			CMenu *pTestMenu = pMainMenu->GetSubMenu(nNdx);
	
			POSITION pos = m_listPlugins.GetHeadPosition();
			while(pos)
			{
				PluginInfo& PI = m_listPlugins.GetNext(pos);
				if(PI.nType != enTest)
					continue;
				
				int nNumItems = pTestMenu->GetMenuItemCount();
				for(int nItem=0;nItem<nNumItems;nItem++)
				{
					pTestMenu->GetMenuString(nItem,str,MF_BYPOSITION);
					if(str != m_strNewMenu)
						continue;

					//Add plugin to new menu
					PI.uiNewCommandId = NEW_MENU_ID_START + m_nMenuItemsAdded;
					
					CMenu *pNewMenu = pTestMenu->GetSubMenu(nItem);
					pNewMenu->InsertMenu(0,MF_BYPOSITION,PI.uiNewCommandId,PI.strName);	
					pNewMenu->RemoveMenu(ID_INSERTPOINT,MF_BYCOMMAND);
					m_nMenuItemsAdded++;
					break;
				}
			}
		}
		else
			pMainMenu->RemoveMenu(nNdx,MF_BYPOSITION);
		
		break;
	}

	//Modify In place menu for Tests
	nNumItems = m_MenuIP.GetMenuItemCount();
	for(nNdx=0;nNdx<nNumItems;nNdx++)
	{
		m_MenuIP.GetMenuString(nNdx,str,MF_BYPOSITION);
		if(str != m_strTestMenu)
			continue;
		
		if(nNumTests)
		{
			CMenu *pTestMenu = m_MenuIP.GetSubMenu(nNdx);
	
			POSITION pos = m_listPlugins.GetHeadPosition();
			while(pos)
			{
				PluginInfo& PI = m_listPlugins.GetNext(pos);
				if(PI.nType != enTest)
					continue;
				
				int nNumItems = pTestMenu->GetMenuItemCount();
				for(int nItem=0;nItem<nNumItems;nItem++)
				{
					pTestMenu->GetMenuString(nItem,str,MF_BYPOSITION);
					if(str != m_strNewMenu)
						continue;

					//Add plugin to new menu
					CMenu *pNewMenu = pTestMenu->GetSubMenu(nItem);
					pNewMenu->InsertMenu(0,MF_BYPOSITION,PI.uiNewCommandId,PI.strName);	
					pNewMenu->RemoveMenu(ID_INSERTPOINT,MF_BYCOMMAND);
					m_nMenuItemsAdded++;
					break;
				}
			}
		}
		else
			m_MenuIP.RemoveMenu(nNdx,MF_BYPOSITION);
		
		break;
	}
}

void CMainFrame::LoadScriptPlugins()
{
	//Load information about test plugins
	int nNumScripts =	LoadPluginType(enScript);
	
	//Modify standard menu to include Scripts
	CMenu *pMainMenu = GetMenu();
	
	CString str;
	int nNumItems = pMainMenu->GetMenuItemCount();
	for(int nNdx=0;nNdx<nNumItems;nNdx++)
	{
		pMainMenu->GetMenuString(nNdx,str,MF_BYPOSITION);
		if(str != m_strScriptMenu)
			continue;
		
		if(nNumScripts)
		{
			CMenu *pScriptMenu = pMainMenu->GetSubMenu(nNdx);
	
			POSITION pos = m_listPlugins.GetHeadPosition();
			while(pos)
			{
				PluginInfo& PI = m_listPlugins.GetNext(pos);
				if(PI.nType != enScript)
					continue;
				
				int nNumItems = pScriptMenu->GetMenuItemCount();
				for(int nItem=0;nItem<nNumItems;nItem++)
				{
					pScriptMenu->GetMenuString(nItem,str,MF_BYPOSITION);
					if(str != m_strNewMenu)
						continue;

					//Add plugin to new menu
					PI.uiNewCommandId = NEW_MENU_ID_START + m_nMenuItemsAdded;
					
					CMenu *pNewMenu = pScriptMenu->GetSubMenu(nItem);
					pNewMenu->InsertMenu(0,MF_BYPOSITION,PI.uiNewCommandId,PI.strName);	
					pNewMenu->RemoveMenu(ID_INSERTPOINT,MF_BYCOMMAND);
					m_nMenuItemsAdded++;
					break;
				}
			}
		}
		else
			pMainMenu->RemoveMenu(nNdx,MF_BYPOSITION);
		
		break;
	}

	//Modify In place menu for Scripts
	nNumItems = m_MenuIP.GetMenuItemCount();
	for(nNdx=0;nNdx<nNumItems;nNdx++)
	{
		m_MenuIP.GetMenuString(nNdx,str,MF_BYPOSITION);
		if(str != m_strScriptMenu)
			continue;
		
		if(nNumScripts)
		{
			CMenu *pScriptMenu = m_MenuIP.GetSubMenu(nNdx);
	
			POSITION pos = m_listPlugins.GetHeadPosition();
			while(pos)
			{
				PluginInfo& PI = m_listPlugins.GetNext(pos);
				if(PI.nType != enScript)
					continue;
				
				int nNumItems = pScriptMenu->GetMenuItemCount();
				for(int nItem=0;nItem<nNumItems;nItem++)
				{
					pScriptMenu->GetMenuString(nItem,str,MF_BYPOSITION);
					if(str != m_strNewMenu)
						continue;

					//Add plugin to new menu
					CMenu *pNewMenu = pScriptMenu->GetSubMenu(nItem);
					pNewMenu->InsertMenu(0,MF_BYPOSITION,PI.uiNewCommandId,PI.strName);	
					pNewMenu->RemoveMenu(ID_INSERTPOINT,MF_BYCOMMAND);
					m_nMenuItemsAdded++;
					break;
				}
			}
		}
		else
			m_MenuIP.RemoveMenu(nNdx,MF_BYPOSITION);
		
		break;
	}
}

void CMainFrame::LoadProfilePlugins()
{
	//Load information about Profile plugins
	int nNumProfiles =	LoadPluginType(enProfile);
	
	//Modify standard menu to include Profiles
	CMenu *pMainMenu = GetMenu();
	
	CString str;
	int nNumItems = pMainMenu->GetMenuItemCount();
	for(int nNdx=0;nNdx<nNumItems;nNdx++)
	{
		pMainMenu->GetMenuString(nNdx,str,MF_BYPOSITION);
		if(str != m_strProfileMenu)
			continue;
		
		if(nNumProfiles)
		{
			CMenu *pProfileMenu = pMainMenu->GetSubMenu(nNdx);
	
			POSITION pos = m_listPlugins.GetHeadPosition();
			while(pos)
			{
				PluginInfo& PI = m_listPlugins.GetNext(pos);
				if(PI.nType != enProfile)
					continue;
				
				int nNumItems = pProfileMenu->GetMenuItemCount();
				for(int nItem=0;nItem<nNumItems;nItem++)
				{
					pProfileMenu->GetMenuString(nItem,str,MF_BYPOSITION);
					if(str != m_strNewMenu)
						continue;

					//Add plugin to new menu
					PI.uiNewCommandId = NEW_MENU_ID_START + m_nMenuItemsAdded;
					
					CMenu *pNewMenu = pProfileMenu->GetSubMenu(nItem);
					pNewMenu->InsertMenu(0,MF_BYPOSITION,PI.uiNewCommandId,PI.strName);	
					pNewMenu->RemoveMenu(ID_INSERTPOINT,MF_BYCOMMAND);
					m_nMenuItemsAdded++;
					break;
				}
			}
		}
		else
			pMainMenu->RemoveMenu(nNdx,MF_BYPOSITION);
		
		break;
	}

	//Modify In place menu for Profiles
	nNumItems = m_MenuIP.GetMenuItemCount();
	for(nNdx=0;nNdx<nNumItems;nNdx++)
	{
		m_MenuIP.GetMenuString(nNdx,str,MF_BYPOSITION);
		if(str != m_strProfileMenu)
			continue;
		
		if(nNumProfiles)
		{
			CMenu *pProfileMenu = m_MenuIP.GetSubMenu(nNdx);
	
			POSITION pos = m_listPlugins.GetHeadPosition();
			while(pos)
			{
				PluginInfo& PI = m_listPlugins.GetNext(pos);
				if(PI.nType != enProfile)
					continue;
				
				int nNumItems = pProfileMenu->GetMenuItemCount();
				for(int nItem=0;nItem<nNumItems;nItem++)
				{
					pProfileMenu->GetMenuString(nItem,str,MF_BYPOSITION);
					if(str != m_strNewMenu)
						continue;

					//Add plugin to new menu
					CMenu *pNewMenu = pProfileMenu->GetSubMenu(nItem);
					pNewMenu->InsertMenu(0,MF_BYPOSITION,PI.uiNewCommandId,PI.strName);	
					pNewMenu->RemoveMenu(ID_INSERTPOINT,MF_BYCOMMAND);
					m_nMenuItemsAdded++;
					break;
				}
			}
		}
		else
			m_MenuIP.RemoveMenu(nNdx,MF_BYPOSITION);
		
		break;
	}
}

int CMainFrame::LoadPluginType(PluginType nType)
{
	int nCount = 0;
	CString strRegSubKey("SOFTWARE\\OpenSTA\\PlugIns\\");

	switch(nType)
	{
	case enScript:
		strRegSubKey += strScriptType;
	break;
	case enProfile:
		strRegSubKey += strProfileType;
	break;
	case enTest:
	strRegSubKey += strTestType;
	break;
	default:
		ASSERT(FALSE);
		return nCount;
	break;
	}

	HKEY hKey;
	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE,strRegSubKey,0,KEY_READ,&hKey)!=ERROR_SUCCESS)
		return nCount;

	DWORD dwIndex=0;
	DWORD dwNameSize=64;
	char szName[64];
	while(RegEnumKeyEx(hKey,dwIndex,szName,&dwNameSize,0,NULL,NULL,NULL)==ERROR_SUCCESS)
	{
		dwNameSize = 64;
		HKEY hKeyPlugin;
		if(RegOpenKeyEx(hKey,szName,0,KEY_READ,&hKeyPlugin) == ERROR_SUCCESS)
		{
			PluginInfo PI;
			PI.strName = szName;
			PI.nType = nType;
			PI.uiNewCommandId = 0;

			DWORD dwType;
			DWORD dwDataSize = 256;
			BYTE byData[256];
			//Get Extension for this type
			if(RegQueryValueEx(hKeyPlugin,"Extension",0,&dwType,byData,&dwDataSize)==ERROR_SUCCESS)
				PI.strExtension = (char*)byData;
			dwDataSize = 256;

			//Get Clsid
			if(RegQueryValueEx(hKeyPlugin,"CLSID",0,&dwType,byData,&dwDataSize)==ERROR_SUCCESS)
			{
				BSTR szGuid = CString((char*)byData).AllocSysString();
				CLSIDFromString(szGuid,&PI.clsid);
				SysFreeString(szGuid);
			}
			dwDataSize = 256;

			RegCloseKey(hKeyPlugin);
			m_listPlugins.AddTail(PI);
			nCount++;
		}
		dwIndex++;
	}
	return nCount;
}

void CMainFrame::OpenFile(const CString& strPath)
{
	int nDotPos = strPath.ReverseFind('.');
	if(nDotPos == -1)
		return;

	CString strExtension(strPath.Mid(nDotPos+1));

	POSITION pos = m_listPlugins.GetHeadPosition();
	while(pos)
	{
		PluginInfo &PI = m_listPlugins.GetNext(pos);
		CString strPIExts(PI.strExtension);

		char *pszExt = strtok(strPIExts.GetBuffer(0),";");
		while(pszExt)
		{
			if(strExtension.CompareNoCase(pszExt)==0)
			{
				if(PI.strName == "HTTP")		//BODGE to invoke modeller
					InvokeModeller(strPath);
				else
				{
					POSITION pos2 = m_pCurrentDoc->GetFirstViewPosition();
					if(pos2)
						((CCommanderView*)m_pCurrentDoc->GetNextView(pos2))->ShowObject(strPath,PI.clsid);
				}
				return;
			}

			pszExt = strtok(NULL,";");
		}
	}
	
	//Show unknown type
	pos = m_pCurrentDoc->GetFirstViewPosition();
	if(pos)
		((CCommanderView*)m_pCurrentDoc->GetNextView(pos))->ShowObject(strPath);
}

BOOL CMainFrame::InvokeModeller(const CString& strScript)
{
	CString strFilePath;
    if (!strScript.IsEmpty())
        strFilePath = "\"" + strScript + "\"";
	
	CString strOpenStaPath(getenv("OPENSTA_HOME"));

    CString strExePath(strOpenStaPath + "Engines\\WEB\\Modeller\\TModeller_Web.exe");
	CString strWorkDir(strOpenStaPath + "Common");

    //Start up the Http modeller
	if((int)ShellExecute(m_hWnd,NULL,strExePath,strFilePath,strWorkDir,SW_SHOWNORMAL) < 32)
	{
		AfxMessageBox("Failed to start the HTTP Modeler");
		return false;
	}

	return true;
}

void CMainFrame::OnCustomMenuCommand(UINT nID)
{
	//Create new file
	POSITION pos = m_listPlugins.GetHeadPosition();
	while(pos)
	{
		PluginInfo& PI = m_listPlugins.GetNext(pos);

		if(PI.uiNewCommandId == nID)
		{
			switch(PI.nType)
			{
			case enScript:
				m_wndWorkspace.m_ctlObjectTree.AddNewScript(PI.strExtension.SpanExcluding(";"));			return;
			case enProfile:
				m_wndWorkspace.m_ctlObjectTree.AddNewProfile(PI.strExtension.SpanExcluding(";"));
			return;
			case enTest:
				m_wndWorkspace.m_ctlObjectTree.AddNewTest();
			return;
			}
		}
	}
}


void CMainFrame::OnTestsMonitor() 
{
	CMonitorSelect dlg;
	if (dlg.DoModal() == IDOK)
	{
		OpenFile(dlg.GetTestFile());
	}
}


