/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ObjectTree.cpp : implementation file
//

#include "stdafx.h"
#include "commander.h"
#include "ObjectTree.h"
#include "RegFunctions.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNAMIC(CObjectTree, CTreeCtrl)

/////////////////////////////////////////////////////////////////////////////
// CObjectTree
#define REFRESH_TIMER 1
extern CLIPFORMAT g_ClipFormat;

CObjectTree::CObjectTree():
	m_hTestFolder(NULL),
	m_hScriptFolder(NULL),
	m_hProfileFolder(NULL)
{
}

CObjectTree::~CObjectTree()
{

}

BEGIN_MESSAGE_MAP(CObjectTree, CTreeCtrl)
	//{{AFX_MSG_MAP(CObjectTree)
	ON_WM_CREATE()
	ON_NOTIFY_REFLECT(TVN_BEGINLABELEDIT, OnBeginlabeledit)
	ON_NOTIFY_REFLECT(TVN_ENDLABELEDIT, OnEndlabeledit)
	ON_NOTIFY_REFLECT(TVN_BEGINDRAG, OnBegindrag)
	ON_NOTIFY_REFLECT(TVN_KEYDOWN, OnKeydown)
	ON_NOTIFY_REFLECT(NM_RCLICK, OnRclick)
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_NEW_FOLDER, OnNewFolder)
	ON_COMMAND(ID_DELETE, OnDelete)
	ON_COMMAND(ID_RENAME, OnRename)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CObjectTree message handlers

int CObjectTree::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CTreeCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	m_DropTarget.Register(this);

	m_TreeImages.Create(IDB_TREEIMAGES_BMP,16,1,RGB(0,128,128));
	SetImageList(&m_TreeImages,TVSIL_NORMAL);
	SetImageList(&m_TreeImages,TVSIL_STATE);

	m_TreeImages.SetOverlayImage(enEmptyFileOverlay,1);

	SetTimer(REFRESH_TIMER,5000,NULL);

	RefreshTree();

	return 0;
}

CString CObjectTree::GetItemPath(HTREEITEM hItem)
{
	CString strPath;
	if(m_mapItemToPath.Lookup(hItem,strPath))
		return strPath;

	return "";
}

BOOL CObjectTree::LoadProfilesFromDisk()
{
	CString strPath = GetRepositoryPath();
	
	if(!m_hProfileFolder)
		m_hProfileFolder = InsertItem(strProfileType,enProfilesFold,enSelProfilesFold);
		
	CString strProfilePath(strPath + strProfileType);
	m_mapItemToPath.SetAt(m_hProfileFolder,strProfilePath);

	return LoadProfilesInPath(strProfilePath+"\\",m_hProfileFolder);
}

BOOL CObjectTree::LoadProfilesInPath(const CString& strPath,HTREEITEM hParent)
{
	BOOL bMoreProfiles;
	CFileFind FF;
	int nImage,nSelImage;
	//Look for profiles in the repository
	if(FF.FindFile(strPath + "*.*"))
	{
		do
		{
			bMoreProfiles = FF.FindNextFile();
			if(FF.IsDots() || FF.IsHidden())
				continue;
			
			HTREEITEM hItem;
			if(FF.IsDirectory())
			{
				hItem = InsertItem(FF.GetFileName(),enFolder,enSelFolder,hParent);
				LoadProfilesInPath(strPath + FF.GetFileName() + "\\",hItem);
				m_mapItemToPath.SetAt(hItem,FF.GetFilePath());
			}
			else
			if(GetFileItemImage(FF.GetFilePath(),nImage,nSelImage))
			{
				hItem = InsertItem(FF.GetFileTitle(),nImage,nSelImage,hParent);
				m_mapItemToPath.SetAt(hItem,FF.GetFilePath());
				if(!FF.GetLength())
					SetItemState(hItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);
			}
		}while(bMoreProfiles);
	}
	return true;
}

BOOL CObjectTree::LoadScriptsFromDisk()
{
	CString strPath = GetRepositoryPath();
	
	if(!m_hScriptFolder)
		m_hScriptFolder = InsertItem(strScriptType,enScriptsFold,enSelScriptsFold);
		
	CString strScriptPath(strPath + strScriptType);
	m_mapItemToPath.SetAt(m_hScriptFolder,strScriptPath);

	return LoadScriptsInPath(strScriptPath+"\\",m_hScriptFolder);
}

BOOL CObjectTree::LoadScriptsInPath(const CString& strPath,HTREEITEM hParent)
{
	BOOL bMoreScripts;
	CFileFind FF;
	int nImage,nSelImage;
	//Look for Scripts in the repository
	if(FF.FindFile(strPath + "*.*"))
	{
		do
		{
			bMoreScripts = FF.FindNextFile();
			if(FF.IsDots() || FF.IsHidden())
				continue;

			HTREEITEM hItem;
			if(FF.IsDirectory())
			{
				hItem = InsertItem(FF.GetFileName(),enFolder,enSelFolder,hParent);
				LoadScriptsInPath(strPath + FF.GetFileName() + "\\",hItem);
				m_mapItemToPath.SetAt(hItem,FF.GetFilePath());
			}
			else
			if(GetFileItemImage(FF.GetFilePath(),nImage,nSelImage))
			{
				hItem = InsertItem(FF.GetFileTitle(),nImage,nSelImage,hParent);
				m_mapItemToPath.SetAt(hItem,FF.GetFilePath());
				if(!FF.GetLength())
					SetItemState(hItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);
			}
		}while(bMoreScripts);
	}
	return true;
}

BOOL CObjectTree::GetFileItemImage(const CString& strFile,int& nImage,int& nSelImage)
{
	nImage = nSelImage = RegisterFileExtension(strFile);
	
	return true;
}

BOOL CObjectTree::LoadTestsFromDisk()
{
	CString strPath = GetRepositoryPath();
	
	if(!m_hTestFolder)
		m_hTestFolder = InsertItem(strTestType,enTestsFold,enSelTestsFold);
		
	CString strTestsPath(strPath + strTestType);
	m_mapItemToPath.SetAt(m_hTestFolder,strTestsPath);

	return LoadTestsInPath(strTestsPath+"\\",m_hTestFolder);
}

BOOL CObjectTree::LoadTestsInPath(const CString& strPath,HTREEITEM hParent)
{
	BOOL bMoreTests;
	CFileFind FF;
	int nImage,nSelImage;
	//Look for Tests in the repository
	if(FF.FindFile(strPath + "*.*"))
	{
		do
		{
			bMoreTests = FF.FindNextFile();
			if(FF.IsDots() || FF.IsHidden())
				continue;

			HTREEITEM hItem;
			if(FF.IsDirectory())
			{
				//Is this a test folder (check if it contains the test file)
				CString strTestFile = FF.GetFilePath();
				strTestFile += "\\" + FF.GetFileTitle() + strTestExtension;

				//Found a test folder					
				HTREEITEM hItem;
				CFileStatus fs;
				if(CFile::GetStatus(strTestFile,fs))
				{
					GetFileItemImage(strTestFile,nImage,nSelImage);
					hItem = InsertItem(FF.GetFileTitle(),nImage,nSelImage,hParent);

					if(!fs.m_size)
						SetItemState(hItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);

					m_mapItemToPath.SetAt(hItem,strTestFile);
				}
				else		
				{
					//just a normal folder
					hItem = InsertItem(FF.GetFileName(),enFolder,enSelFolder,hParent);
					m_mapItemToPath.SetAt(hItem,FF.GetFilePath());
					LoadTestsInPath(strPath + FF.GetFileName() + "\\",hItem);
				} 
			}
			else
			if(GetFileItemImage(FF.GetFilePath(),nImage,nSelImage))
			{
				hItem = InsertItem(FF.GetFileTitle(),nImage,nSelImage,hParent);
				m_mapItemToPath.SetAt(hItem,FF.GetFilePath());
				if(FF.GetLength())
					SetItemState(hItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);
			}
		}while(bMoreTests);
	}
	return true;
}

void CObjectTree::SaveBranchState(HTREEITEM hBranch,CMap <CString,LPCTSTR,UINT,UINT>& mapPathToState)
{
	if(ItemHasChildren(hBranch))
	{
		CString strPath;
		m_mapItemToPath.Lookup(hBranch,strPath);
		UINT uiState = GetItemState(hBranch,TVIS_EXPANDED|TVIS_SELECTED);
		mapPathToState.SetAt(strPath,uiState);

		HTREEITEM hNextItem;
		HTREEITEM hChildItem = GetChildItem(hBranch);
		while (hChildItem != NULL)
		{
			hNextItem = GetNextItem(hChildItem, TVGN_NEXT);

			m_mapItemToPath.Lookup(hChildItem,strPath);
			uiState = GetItemState(hChildItem,TVIS_EXPANDED|TVIS_SELECTED);
			mapPathToState.SetAt(strPath,uiState);
	
			SaveBranchState(hChildItem,mapPathToState);
			
			hChildItem = hNextItem;
		}
	}
}

void CObjectTree::RestoreBranchState(HTREEITEM hBranch,CMap <CString,LPCTSTR,UINT,UINT>& mapPathToState)
{
	if(ItemHasChildren(hBranch))
	{
		UINT uiState;
		CString strPath;

		m_mapItemToPath.Lookup(hBranch,strPath);
		if(mapPathToState.Lookup(strPath,uiState))
		{
			if(uiState & TVIS_EXPANDED)
				Expand(hBranch,TVE_EXPAND);
			if(uiState & TVIS_SELECTED)
				SelectItem(hBranch);
		}

		HTREEITEM hNextItem;
		HTREEITEM hChildItem = GetChildItem(hBranch);
		while (hChildItem != NULL)
		{
			hNextItem = GetNextItem(hChildItem, TVGN_NEXT);

			m_mapItemToPath.Lookup(hChildItem,strPath);
			if(mapPathToState.Lookup(strPath,uiState))
			{
				if(uiState & TVIS_EXPANDED)
					Expand(hChildItem,TVE_EXPAND);
				if(uiState & TVIS_SELECTED)
					SelectItem(hChildItem);
			}
				
			RestoreBranchState(hChildItem,mapPathToState);
			
			hChildItem = hNextItem ;
		}
	}
}

BOOL CObjectTree::RefreshTree()
{
	SetRedraw(FALSE);
	CMap <CString,LPCTSTR,UINT,UINT> mapPathToState;

	//Store tree state
	HTREEITEM hNextItem;
	HTREEITEM hRoot = GetRootItem();
	
	while(hRoot != NULL)
	{
		hNextItem = GetNextItem(hRoot, TVGN_NEXT);
		SaveBranchState(hRoot,mapPathToState);
		hRoot = hNextItem;
	}

	
	BOOL bResult = DeleteAllItems();
	m_mapItemToPath.RemoveAll();

	m_hProfileFolder = NULL;
	m_hTestFolder = NULL;
	m_hScriptFolder = NULL;	
	bResult &= LoadProfilesFromDisk();
	bResult &= LoadScriptsFromDisk();
	bResult &= LoadTestsFromDisk();

	//restore tree state
	hRoot = GetRootItem();
	while(hRoot != NULL)
	{
		hNextItem = GetNextItem(hRoot, TVGN_NEXT);
		RestoreBranchState(hRoot,mapPathToState);
		hRoot = hNextItem;
	}

	SetRedraw();
	return bResult;
}

HTREEITEM CObjectTree::AddNewScript(const CString& strType)
{
	CString strPath = GetRepositoryPath() + strScriptType + "\\";
	HTREEITEM hParent = m_hScriptFolder;

	CFileStatus fs;
	
/*	//TODO: Folders should are only one level deep (name conflicts)
	//If the selected item is a directory within 
	//the scripts folder add the new script to this folder
	HTREEITEM hItem = GetSelectedItem();
	CString strSelPath = GetItemPath(hItem);
	if(CFile::GetStatus(strSelPath,fs) && (fs.m_attribute & CFile::Attribute::directory))
	{
		if(strSelPath.Left(strPath.GetLength()).CompareNoCase(strPath)==0)
		{
			strPath = strSelPath;
			hParent = hItem;
		}	
	}
*/
	CString strFile = "NEWSCRIPT." + strType;		//Path to script
	
	int nIndex=1;
	while(CFile::GetStatus(strPath + strFile,fs))
	{
		//File exists, so increase index
		strFile.Format("NEWSCRIPT_%d.%s",nIndex++,strType);
	}

	CString strNewScript(strPath + strFile);
	CFile f;
	f.Open(strNewScript,CFile::modeCreate);
	f.Close();

	int nImage,nSelImage;
	if(GetFileItemImage(strNewScript,nImage,nSelImage))
	{
		CString strName = strFile.SpanExcluding(".");
		HTREEITEM hItem = InsertItem(strName,nImage,nSelImage,hParent);
		m_mapItemToPath.SetAt(hItem,strNewScript);
		SetItemState(hItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);
		EnsureVisible(hItem);
		EditLabel(hItem);
		return hItem;
	}
	return NULL;
}

HTREEITEM CObjectTree::AddNewTest()
{
	CString strPath = GetRepositoryPath() + strTestType + "\\";
	HTREEITEM hParent = m_hTestFolder;

	CFileStatus fs;
	//If the selected item is a directory within 
	//the tests folder add the new Test to this folder
	HTREEITEM hItem = GetSelectedItem();
	CString strSelPath = GetItemPath(hItem);
	if(CFile::GetStatus(strSelPath,fs) && (fs.m_attribute & CFile::Attribute::directory))
	{
		if(strSelPath.Left(strPath.GetLength()).CompareNoCase(strPath)==0)
		{
			strPath = strSelPath;
			hParent = hItem;
		}	
	}
	
	CString strFile = "NEWTEST";		//Path to test folder
	
	int nIndex=1;
	while(CFile::GetStatus(strPath + strFile + "\\" + strFile + strTestExtension,fs))
	{
		//File exists, so increase index
		strFile.Format("NEWTEST_%d",nIndex++);
	}

	CString strDir(strPath + strFile);
	CreateDirectory(strDir,NULL);
	
	CString strNewTest(strDir + "\\" + strFile + strTestExtension);
	CFile f;
	f.Open(strNewTest,CFile::modeCreate);
	f.Close();

	int nImage,nSelImage;
	if(GetFileItemImage(strNewTest,nImage,nSelImage))
	{
		CString strName = strFile.SpanExcluding(".");
		HTREEITEM hItem = InsertItem(strName,nImage,nSelImage,hParent);
		m_mapItemToPath.SetAt(hItem,strNewTest);
		SetItemState(hItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);
		EnsureVisible(hItem);
		EditLabel(hItem);
		return hItem;
	}
	return NULL;
}

HTREEITEM CObjectTree::AddNewProfile(const CString& strType)
{
	CString strPath = GetRepositoryPath() + strProfileType + "\\";
	HTREEITEM hParent = m_hProfileFolder;
	
	CFileStatus fs;
/*	//TODO: Folders should are only one level deep (name conflicts)
	//If the selected item is a directory within 
	//the profiles folder add the new profile to this folder
	HTREEITEM hItem = GetSelectedItem();
	CString strSelPath = GetItemPath(hItem);
	if(CFile::GetStatus(strSelPath,fs) && (fs.m_attribute & CFile::Attribute::directory))
	{
		if(strSelPath.Left(strPath.GetLength()).CompareNoCase(strPath)==0)
		{
			strPath = strSelPath;
			hParent = hItem;
		}	
	}
*/	
	CString strFile = "NEWPROFILE." + strType;		//Path to profile
	
	int nIndex=1;
	while(CFile::GetStatus(strPath + strFile,fs))
	{
		//File exists, so increase index
		strFile.Format("NEWPROFILE_%d.%s",nIndex++,strType);
	}

	CString strNewProfile(strPath + strFile);
	CFile f;
	f.Open(strNewProfile,CFile::modeCreate);
	f.Close();

	int nImage,nSelImage;
	if(GetFileItemImage(strNewProfile,nImage,nSelImage))
	{
		CString strName = strFile.SpanExcluding(".");
		HTREEITEM hItem = InsertItem(strName,nImage,nSelImage,hParent);
		m_mapItemToPath.SetAt(hItem,strNewProfile);
		SetItemState(hItem,INDEXTOOVERLAYMASK(1),TVIS_OVERLAYMASK);
		EnsureVisible(hItem);
		EditLabel(hItem);
		return hItem;
	}
	return NULL;
}

void CObjectTree::OnBeginlabeledit(NMHDR* pNMHDR, LRESULT* pResult) 
{
	TV_DISPINFO* pTVDispInfo = (TV_DISPINFO*)pNMHDR;
	
	int nImage,nSelImage;
	GetItemImage(pTVDispInfo->item.hItem,nImage,nSelImage);

	if((pTVDispInfo->item.hItem == m_hProfileFolder) || 
		(pTVDispInfo->item.hItem == m_hScriptFolder) ||
		(pTVDispInfo->item.hItem == m_hTestFolder))
	{
		*pResult = 1;
		return;
	}

	CString strPath = GetItemPath(pTVDispInfo->item.hItem);
	if(IsTest(strPath)||IsProfile(strPath)||IsScript(strPath))
	{
		CEdit *pEdit = GetEditControl();		//Use data name if one of these types
		HWND hwnd = pEdit->GetSafeHwnd();
		m_ctlLabelEdit.SubclassWindow(hwnd);
	}

	*pResult = 0;
}

void CObjectTree::OnEndlabeledit(NMHDR* pNMHDR, LRESULT* pResult) 
{
	TV_DISPINFO* pTVDispInfo = (TV_DISPINFO*)pNMHDR;
	
	*pResult = 0;

	if(!pTVDispInfo->item.pszText || (GetItemText(pTVDispInfo->item.hItem) == pTVDispInfo->item.pszText))
	{
		*pResult = 1;
		return;
	}
	
	CString strPath = GetItemPath(pTVDispInfo->item.hItem);
	
	int nSlashPos = strPath.ReverseFind('\\');
	
	CString strNewPath(strPath.Left(nSlashPos+1));
	strNewPath += pTVDispInfo->item.pszText;
	
	CFileStatus fs;
	if(!CFile::GetStatus(strPath,fs))
		return;
	
	if(!(fs.m_attribute & CFile::Attribute::directory))
	{
		int nDotPos = strPath.ReverseFind('.');
		if(nDotPos != -1)
			strNewPath += strPath.Mid(nDotPos);
	}

	if(CFile::GetStatus(strNewPath,fs))
	{
		AfxMessageBox(CString("The file \"") + pTVDispInfo->item.pszText + "\" already exists");
		return;
	}

	try
	{
		if(IsTest(strPath))
		{
			//If this is a test rename the folder as well
			CString strOldFolder(strPath.Left(nSlashPos));
			int nSlash = strOldFolder.ReverseFind('\\');
			CString strNewFolder(strOldFolder.Left(nSlash+1));
			strNewFolder += pTVDispInfo->item.pszText;
			
			nSlash = strNewPath.ReverseFind('\\');
			CString strNewTest = strNewFolder + strNewPath.Mid(nSlash);

			if(CFile::GetStatus(strNewFolder,fs))
			{
				AfxMessageBox(CString("The test \"") + pTVDispInfo->item.pszText + "\" already exists");
				return;
			}

			//Rename the file
			CFile::Rename(strPath,strNewPath);

			//Rename the folder
			CFile::Rename(strOldFolder,strNewFolder);

			strNewPath = strNewTest;
		}
		else
			//Rename the file
			CFile::Rename(strPath,strNewPath);

		m_mapItemToPath.SetAt(pTVDispInfo->item.hItem,strNewPath);
	}
	catch(CFileException *e)
	{
		CString strError;
		e->GetErrorMessage(strError.GetBuffer(256), 256);
		strError.ReleaseBuffer();
		strError.Replace("an unnamed file", "\"" + strPath + "\"");

		AfxMessageBox(strError);
		e->Delete();
	}

	*pResult = 1;
}

int CObjectTree::RegisterFileExtension(const CString& strFile)
{
	int nPos = strFile.ReverseFind('.');
	if(nPos == -1)
		return -1;

	CString strExtension(strFile.Mid(nPos));

	int nIndex=0;
	if(m_mapExtToImage.Lookup(strExtension,nIndex))
		return nIndex;

	SHFILEINFO shfi;
	SHGetFileInfo( strFile, 0, &shfi, sizeof( SHFILEINFO ), 
               SHGFI_ICON | SHGFI_SHELLICONSIZE | SHGFI_SMALLICON);

	if(!shfi.hIcon || !(nIndex = (WORD)m_TreeImages.Add(shfi.hIcon)))
		return -1;

	DestroyIcon(shfi.hIcon);

	m_mapExtToImage.SetAt(strExtension,nIndex);	
	return nIndex;
}

void CObjectTree::OnBegindrag(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

	HTREEITEM hItem = pNMTreeView->itemNew.hItem;

	CString strPath = GetItemPath(hItem);
	int nDataSize = strPath.GetLength() + 1;
	
	COleDataSource DataSource;
	HGLOBAL hData = GlobalAlloc(GMEM_FIXED|GMEM_ZEROINIT,nDataSize);
	char *szData = (char*)GlobalLock(hData);
	
	strcpy(szData,strPath);
	
	GlobalUnlock(hData);
	DataSource.CacheGlobalData(g_ClipFormat,hData);

	DROPEFFECT DE = DataSource.DoDragDrop();

	*pResult = 0;
}


void CObjectTree::OnKeydown(NMHDR* pNMHDR, LRESULT* pResult) 
{
	TV_KEYDOWN* pTVKeyDown = (TV_KEYDOWN*)pNMHDR;

	switch(pTVKeyDown->wVKey)
	{
	case VK_F5:
		RefreshTree();
	break;
	case VK_DELETE:
		m_hMenuSel = GetSelectedItem();
		OnDelete();
	break;
	}

	*pResult = 0;
}

void CObjectTree::OnRclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	SendMessage(WM_CONTEXTMENU, (WPARAM) m_hWnd, GetMessagePos());
	
	*pResult = 1;
}

void CObjectTree::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

	UINT uFlags;
	ScreenToClient(&point);
	HTREEITEM hItem = HitTest(point,&uFlags);
	if(hItem)
	{
		m_hMenuSel = hItem;

	    CMenu menu;
		menu.LoadMenu(IDR_TREE_MENU);

		CString strPath = GetItemPath(hItem);
	
		CFileStatus fs;
		if(!CFile::GetStatus(strPath,fs))
			return;
		
		//For the time being can only create new folders in test folder
		BOOL bTestFolder = FALSE;
		HTREEITEM hParent=hItem;
		do
		{
			if(hParent == m_hTestFolder)
			{
				bTestFolder = true;
				break;
			}
		}
		while(hParent = GetParentItem(hParent));
		
		if(!(fs.m_attribute & CFile::Attribute::directory) || !bTestFolder)
		{
			menu.RemoveMenu(ID_NEW_FOLDER,MF_BYCOMMAND);
		}

		if((hItem==m_hProfileFolder) ||
			(hItem==m_hScriptFolder) ||
			(hItem==m_hTestFolder))
		{
			menu.RemoveMenu(ID_DELETE,MF_BYCOMMAND);			
			menu.RemoveMenu(ID_RENAME,MF_BYCOMMAND);			
		}
			
		ClientToScreen(&point);
		menu.GetSubMenu(0)->TrackPopupMenu(TPM_LEFTALIGN|TPM_LEFTBUTTON|TPM_RIGHTBUTTON,point.x+5,point.y+5,this);
	}
}


void CObjectTree::OnNewFolder() 
{
	CString strPath = GetItemPath(m_hMenuSel) + "\\";
	CString strFolder("NewFolder");
	
	int nIndex=1;
	CFileStatus fs;
	while(CFile::GetStatus(strPath + strFolder,fs))
		//Folder exists, so increase index
		strFolder.Format("NewFolder(%d)",nIndex++);

	CString strNewFolder(strPath + strFolder);
	if(!CreateDirectory(strNewFolder,NULL))
	{
		CString strMsg;
		strMsg.Format("Failed to create a new folder. (Error %d)",GetLastError());
		AfxMessageBox(strMsg,MB_OK);
		return;
	}

	HTREEITEM hItem = InsertItem(strFolder,enFolder,enSelFolder,m_hMenuSel);
	m_mapItemToPath.SetAt(hItem,strNewFolder);
	EnsureVisible(hItem);
	EditLabel(hItem);
}

void CObjectTree::OnDelete() 
{
	CString strPath = GetItemPath(m_hMenuSel);

	if(IsTest(strPath))
	{
		int nSlashPos = strPath.ReverseFind('\\');
		
		CString strFolder(strPath.Left(nSlashPos));
		CString strTest(strPath.Mid(nSlashPos+1));

		int nDotPos= strTest.ReverseFind('.');
		strTest = strTest.Left(nDotPos);

		if(AfxMessageBox("Are you sure you want to delete test \"" + strTest + "\" and all its results?",MB_YESNO)==IDNO)
			return;

		//Delete Files from disk
		DeleteFolder(strFolder);
		RefreshTree();
		return;
	}

	CFileStatus fs;
	if(!CFile::GetStatus(strPath,fs))
		return;

	if(AfxMessageBox("Are you sure you want to delete \"" + strPath + "\"?",MB_YESNO)==IDNO)
		return;
	
	if(fs.m_attribute & CFile::Attribute::directory)
	{
		//Delete Files from disk
		DeleteFolder(strPath);
		RefreshTree();
	}
	else
	{
		DeleteFile(strPath);
		RefreshTree();
	}
}

BOOL CObjectTree::DeleteFolder(const CString& strFolder)
{
	CFileFind FF;
	if(!FF.FindFile(strFolder + "\\*.*"))
		return FALSE;

	BOOL bContinue=true;
	while(bContinue)
	{
		bContinue = FF.FindNextFile();
		if(FF.IsDots())
			continue;

		if(FF.IsDirectory())
		{
			if(!DeleteFolder(FF.GetFilePath()))
				return false;
		}
		else
		{
			if(!DeleteFile(FF.GetFilePath()))
			{
				CString str;
				str.Format("Could not delete \"%s\"\n\n%s\nContinue with remaining files?",FF.GetFilePath(),GetErrorMessage(GetLastError()));
				if(AfxMessageBox(str,MB_YESNO)==IDNO)
					return false;
			}
		}
	}
	FF.Close();
	
	if(!RemoveDirectory(strFolder))
	{
		CString str;
		str.Format("Could not delete \"%s\"\n\n%s\nContinue with remaining files?",strFolder,GetErrorMessage(GetLastError()));
		if(AfxMessageBox(str,MB_YESNO)==IDNO)
			return false;
	}

	return true;
}

CString CObjectTree::GetErrorMessage(DWORD dwError)
{
	LPVOID lpMsgBuf;
	FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM | 
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,
		0,
		NULL 
	);
	
	CString str((LPCTSTR)lpMsgBuf);
	str.Replace("\n"," ");
	str.Remove('\r');

	// Free the buffer.
	LocalFree( lpMsgBuf );

	return str;
}

void CObjectTree::OnRename() 
{
	EditLabel(m_hMenuSel);
}

BOOL CObjectTree::PreTranslateMessage(MSG* pMsg) 
{
	// If edit control is visible in tree view control, when you send a
	// WM_KEYDOWN message to the edit control it will dismiss the edit
	// control. When the ENTER key was sent to the edit control, the
	// parent window of the tree view control is responsible for updating
	// the item's label in TVN_ENDLABELEDIT notification code.
	if (pMsg->message == WM_KEYDOWN &&	pMsg->wParam == VK_RETURN 
			|| pMsg->wParam == VK_ESCAPE)
	{
		CEdit* pEdit = GetEditControl();
		if (pEdit)
		{
			pEdit->SendMessage(WM_KEYDOWN, pMsg->wParam, pMsg->lParam);
			return TRUE;
		}
	}

	return CTreeCtrl::PreTranslateMessage(pMsg);
}

DROPEFFECT CObjectTree::OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point) 
{
	//Is the right sort of data available
	if(pDataObject->IsDataAvailable(g_ClipFormat))
	{
		//Find the target item
		UINT uFlags;
		HTREEITEM hItem = HitTest(point,&uFlags);
		if(hItem)
		{
			//Get path of target item
			CString strTargetPath = GetItemPath(hItem);
			
			CFileStatus fs;
			if(CFile::GetStatus(strTargetPath,fs) &&	//Check target is a directory...
				(fs.m_attribute & CFile::Attribute::directory) &&
				!IsTest(strTargetPath))	//...and that it is not a test directory
			{
				//Can only drop to a folder in test directory
				int nReposPathLen = GetRepositoryPath().GetLength();
				if(strTargetPath.Mid(nReposPathLen,strTestType.GetLength())==strTestType)
				{
					HGLOBAL hData = pDataObject->GetGlobalData(g_ClipFormat);
					CString strItem = (char*)GlobalLock(hData);
					GlobalUnlock(hData);
					if(IsTest(strItem))
					{
						SelectDropTarget(hItem);
						return DROPEFFECT_MOVE;
					}
				}
			}
		}
	}
	
	SelectDropTarget(NULL);
	return DROPEFFECT_NONE;
}

BOOL CObjectTree::OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point) 
{
	HTREEITEM hItem = GetDropHilightItem();
	SelectDropTarget(NULL);

	HGLOBAL hData = pDataObject->GetGlobalData(g_ClipFormat);
	CString strSource = (char*)GlobalLock(hData);
	GlobalUnlock(hData);

	CString strTarget = GetItemPath(hItem);
	if(IsTest(strSource))
	{
		int nSlashPos = strSource.ReverseFind('\\');
		strSource = strSource.Left(nSlashPos);
	}

	int nSrcLen = strSource.GetLength();	//need a /0/0 terminated string
	char *szSrc = new char[nSrcLen+2];
	strcpy(szSrc,strSource);
	szSrc[nSrcLen+1] = NULL;

	int nTgtLen = strTarget.GetLength();	//need a /0/0 terminated string
	char *szTgt = new char[nTgtLen+2];
	strcpy(szTgt,strTarget);
	szTgt[nTgtLen+1] = NULL;

	SHFILEOPSTRUCT FileOp;
	memset(&FileOp,0,sizeof(SHFILEOPSTRUCT));
	FileOp.wFunc = FO_MOVE;
	FileOp.pFrom = szSrc;
	FileOp.pTo = szTgt;
	BOOL bResult = (SHFileOperation(&FileOp)==0);

	delete []szSrc;
	delete []szTgt;
	
	RefreshTree();
	
	return bResult;
}

void CObjectTree::OnTimer(UINT nIDEvent) 
{
	switch(nIDEvent)
	{
	case REFRESH_TIMER:
		if(GetEditControl())
			return;
		//Refresh the tree if something has changed on the disk
		ULARGE_INTEGER uliUser,uliTotal,uliFree;
		GetDiskFreeSpaceEx(GetRepositoryPath(),&uliUser,&uliTotal,&uliFree);
		if(m_uliFreeDiskSpace.QuadPart != uliFree.QuadPart)
		{
			m_uliFreeDiskSpace = uliFree;
			RefreshTree();
		}
	break;
	}

	CTreeCtrl::OnTimer(nIDEvent);
}

