/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MonitorSummary.cpp : implementation file
//

#include "stdafx.h"
#include "testplugin.h"
#include "MonitorSummary.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define IDT_POLL_DATA 1
static int ID_REFRESH_RATE = 1000;
/////////////////////////////////////////////////////////////////////////////
// CMonitorSummary dialog


CMonitorSummary::CMonitorSummary(CWnd* pParent /*=NULL*/)
	: CDialog(CMonitorSummary::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMonitorSummary)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	m_pTestFile = NULL;
	m_nTotalIndex = 0;
	m_pCmd = NULL;
}


void CMonitorSummary::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMonitorSummary)
	DDX_Control(pDX, IDC_SUMMARY_LIST, m_ctlSummary);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMonitorSummary, CDialog)
	//{{AFX_MSG_MAP(CMonitorSummary)
	ON_WM_TIMER()
	ON_WM_SIZE()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMonitorSummary message handlers

BOOL CMonitorSummary::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	CRect r;
	GetClientRect(r);
	m_ctlSummary.InsertColumn(0,"Task Group",LVCFMT_LEFT,r.Width()/2);
	m_ctlSummary.InsertColumn(1,"VU Count",LVCFMT_LEFT,r.Width()/2);

	ASSERT(m_pTestFile != NULL);

	for(int g = 0; g<m_pTestFile->GetNumTaskGroups(); g++)
	{
		m_ctlSummary.InsertItem(g, m_pTestFile->GetTaskGroupName(g));
		m_ctlSummary.SetItemText(g,1,"0");
	}

	m_nTotalIndex = m_ctlSummary.GetItemCount();
	m_ctlSummary.InsertItem(m_nTotalIndex, "Total");
	m_ctlSummary.SetItemText(m_nTotalIndex,1,"0");

	UpdateSummary();
	SetTimer(IDT_POLL_DATA, ID_REFRESH_RATE, NULL);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

BOOL CMonitorSummary::Init(CString strManagerReg, CTestFile* pTestFile)
{
	m_pTestFile = pTestFile;
	try
	{
		// Set the directory to the engines dir.
		CString enginesPath = getenv("OPENSTA_HOME");
		enginesPath+="Engines";

		SetCurrentDirectory(enginesPath);
	
		// Execute TestInfo
		CString strInfoPath = enginesPath+"\\TI_SUMMARY.TXT";
		CString cmd = "TestInfo.exe -S -I "+ strManagerReg + " -O TI_SUMMARY.TXT";
		m_pCmd = new CCmdExec(cmd);
		m_pCmd->Execute();
		Sleep(1000);
		// Open file ready for access
		CFileException e;
		if (!m_outFile.Open(strInfoPath, CFile::modeRead|CFile::typeText|CFile::shareDenyNone,&e))
		{
			AfxMessageBox("File could not be opened " + e.m_cause );
		}
		m_outFile.SeekToEnd();
		return true;
	}
	catch(...)
	{
	}





	return false;
}

void CMonitorSummary::OnTimer(UINT nIDEvent) 
{
	UpdateSummary();
	CDialog::OnTimer(nIDEvent);
}

void CMonitorSummary::UpdateSummary()
{
	CString line;

	while(m_outFile.ReadString(line))
	{
		CString strGroup, strUsers;
		GetNextField(',',line,strGroup);
		GetNextField(',',line,strUsers);
		
		for(int i =0; i<m_ctlSummary.GetItemCount(); i++)
		{
			CString strItem = m_ctlSummary.GetItemText(i,0);
			strItem.MakeUpper();
			if(strItem == strGroup)
			{
				m_ctlSummary.SetItemText(i,1,strUsers);
			}
		}
	}

	int nTotal = 0;
	for(int i =0; i<m_nTotalIndex; i++)
	{
		CString strItem = m_ctlSummary.GetItemText(i,1);
		nTotal+=atoi(strItem);
	}
	CString strTotal;
	strTotal.Format("%d", nTotal);
	m_ctlSummary.SetItemText(m_nTotalIndex, 1, strTotal);
}

int CMonitorSummary::GetNextField(char sep, CString &inStr, CString &field)
{
    if (inStr == "")
    {
        return -1;
    }

    int len = inStr.GetLength();
    int nPos = inStr.Find(sep, 0);

    if (nPos == -1)
    {
        field = inStr;
        inStr = "";
        return 0;
    }
    else
    {
        field = inStr.Left(nPos);
        inStr = inStr.Right(len - nPos - 1);
        return 0;
    }
}

void CMonitorSummary::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);
		
	if(m_ctlSummary.m_hWnd != NULL)
	{
		m_ctlSummary.MoveWindow(0,0,cx,cy);
	}
}

void CMonitorSummary::OnDestroy() 
{
	CDialog::OnDestroy();
	if (m_pCmd != NULL)
	{
		m_pCmd->KillProcess();
		delete m_pCmd;
	}
}
