/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ResultsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "TestPlugin.h"
#include "ResultsDlg.h"
#include "TestFile.h"
#include "PluginTypes.h"
#include "TestResultsPlugin.h"
#include "XAuditViewer.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CResultsDlg dialog


CResultsDlg::CResultsDlg(CWnd* pParent /*=NULL*/)
	: CDragDropDialog(CResultsDlg::IDD, pParent),
	m_strAuditLogName("Test Audit Log")
{
	//{{AFX_DATA_INIT(CResultsDlg)
	//}}AFX_DATA_INIT
	m_pTestFile = NULL;
	m_nLastGraphAdded=0;
	m_pCurrentGraph=NULL;
	m_bFrameHidden = FALSE;
}


void CResultsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDragDropDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CResultsDlg)
	DDX_Control(pDX, IDC_SHOWHIDE_BTN, m_ctlShowHideBtn);
	DDX_Control(pDX, IDC_GRAPHS_CAPTION, m_ctlGraphsCaption);
	DDX_Control(pDX, IDC_RUNS_CAPTION, m_ctlRunsCaption);
	DDX_Control(pDX, IDC_FILTERS, m_ctlFilters);
	DDX_Control(pDX, IDC_LIST_FRAME, m_ctlListFrame);
	DDX_Control(pDX, IDC_RUNS, m_ctlRuns);
	DDX_Control(pDX, IDC_GRAPH_TYPES, m_ctlGraphs);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CResultsDlg, CDragDropDialog)
	//{{AFX_MSG_MAP(CResultsDlg)
	ON_WM_SIZE()
	ON_BN_CLICKED(IDC_FILTERS, OnFilters)
	ON_LBN_SELCHANGE(IDC_RUNS, OnSelchangeRuns)
	ON_LBN_SELCHANGE(IDC_GRAPH_TYPES, OnSelchangeGraphTypes)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_SHOWHIDE_BTN, OnShowhideBtn)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CResultsDlg message handlers

BOOL CResultsDlg::OnInitDialog() 
{
	CDragDropDialog::OnInitDialog();

	m_bmpShowHide.LoadBitmap(m_bFrameHidden ? IDB_DOWN_ARROW : IDB_UP_ARROW);
	m_ctlShowHideBtn.SetBitmap((HBITMAP)m_bmpShowHide);

	FillRunList();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CResultsDlg::FillRunList()
{
	CFileFind FF;
	if(!FF.FindFile(m_strTestFolder + "*.*"))
		return;

	BOOL bContinue=true;
	while(bContinue)
	{
		bContinue = FF.FindNextFile();

		if(FF.IsDirectory() && !FF.IsDots() && !FF.IsHidden())
		{
			m_ctlRuns.AddString(FF.GetFileName());
		}
	}
}

void CResultsDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDragDropDialog::OnSize(nType, cx, cy);
	
	if(!m_ctlRuns.GetSafeHwnd() || !m_ctlGraphs.GetSafeHwnd())
		return;

	int nWidth = cx;
	const int nSeparator = 5;

	//Get size of show/hide button
	CRect rcShowHide;
	m_ctlShowHideBtn.GetWindowRect(&rcShowHide);

	if(!m_bFrameHidden)
	{
		//Get size of List box
		CRect rcRuns;
		m_ctlRuns.GetWindowRect(&rcRuns);
		ScreenToClient(&rcRuns);

		//Fit Frame around lists
		m_ctlListFrame.MoveWindow(0,0,cx,rcRuns.bottom+4);

		//Position filter button to right of window
		CRect rcFilter;
		m_ctlFilters.GetWindowRect(&rcFilter);
		m_ctlFilters.SetWindowPos(NULL,cx-(rcFilter.Width()+nSeparator),rcRuns.top,0,0,SWP_NOZORDER|SWP_NOSIZE);
				
		int nAvailableWidth = cx - (rcFilter.Width()+2*nSeparator);
		
		//Position captions
		m_ctlRunsCaption.SetWindowPos(NULL,nSeparator,nSeparator,0,0,SWP_NOZORDER|SWP_NOSIZE);
		m_ctlGraphsCaption.SetWindowPos(NULL,(nAvailableWidth/2)+nSeparator,nSeparator,0,0,SWP_NOZORDER|SWP_NOSIZE);

		//Get size of caption
		CRect rcGraphCaption;
		m_ctlGraphsCaption.GetWindowRect(&rcGraphCaption);
		ScreenToClient(&rcGraphCaption);

		//Position List boxes
		m_ctlRuns.SetWindowPos(NULL,nSeparator,rcGraphCaption.bottom+nSeparator,(nAvailableWidth/2)-nSeparator,rcRuns.Height(),SWP_NOZORDER);
		m_ctlGraphs.SetWindowPos(NULL,(nAvailableWidth/2)+nSeparator,rcGraphCaption.bottom+nSeparator,(nAvailableWidth/2)-nSeparator,rcRuns.Height(),SWP_NOZORDER);
	}
	else
	{
		//Fit Frame around show/hide button
		m_ctlListFrame.MoveWindow(0,0,cx,rcShowHide.Height()+4);
	}

	CRect rcFrame;
	m_ctlListFrame.GetWindowRect(&rcFrame);
	ScreenToClient(&rcFrame);

	//Position show/hide button at bottom right of frame
	m_ctlShowHideBtn.SetWindowPos(NULL,cx-(nSeparator/2)-rcShowHide.Width(),rcFrame.bottom-rcShowHide.Height()-(nSeparator/2),0,0,SWP_NOZORDER|SWP_NOSIZE);
	
	//Position chart below frame
	if(m_pCurrentGraph)
		m_pCurrentGraph->SetWindowPos(NULL,0,rcFrame.bottom+(nSeparator/2),cx,cy-rcFrame.bottom-(nSeparator/2),SWP_NOZORDER);
}

void CResultsDlg::OnFilters() 
{
	// TODO: Add your control notification handler code here
	
}

void CResultsDlg::OnSelchangeRuns() 
{
	if(m_pCurrentGraph)
	{
		delete m_pCurrentGraph;
		m_pCurrentGraph = NULL;
	}

	CString strItem;
	int nCurSel = m_ctlRuns.GetCurSel();
	m_ctlRuns.GetText(nCurSel,strItem);
	strItem = m_strTestFolder + strItem;

	//Load the selected run
	CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
	CPluginManager *pPluginManager = pApp->GetPluginManager();

	m_arGraphs.RemoveAll();
	m_nLastGraphAdded = 0;

	GraphType GT;
	GT.strPlugin.Empty();
	GT.strGraphTitle = m_strAuditLogName;
	GT.nGraphIndex = m_nLastGraphAdded++;
	m_arGraphs.Add(GT);
	
	CStringArray arResultsPlugIns;
	pPluginManager->ListPluginsByType(TEST_RESULTS_PLUGIN, arResultsPlugIns);
	
	for (int nPlugIn = 0; nPlugIn<arResultsPlugIns.GetSize();nPlugIn++)
	{
		CTestResultsPlugin plug(arResultsPlugIns[nPlugIn], pPluginManager);
		plug.LoadRun(strItem);

		//Get the graphs which are now available
		CStringArray arstrTypes;
		plug.GetAvailableGraphs(arstrTypes);

		GT.strPlugin = arResultsPlugIns[nPlugIn];
		for(int nNdx=0;nNdx<arstrTypes.GetSize();nNdx++)
		{
			GT.strGraphTitle = arstrTypes[nNdx];
			GT.nGraphIndex = m_nLastGraphAdded++;
			m_arGraphs.Add(GT);
		}
	}

	m_ctlGraphs.SetRedraw(FALSE);
	m_ctlGraphs.ResetContent();

	for(int nNdx=0;nNdx<m_arGraphs.GetSize();nNdx++)
	{
		int nItem = m_ctlGraphs.AddString(m_arGraphs[nNdx].strGraphTitle);
		m_ctlGraphs.SetItemData(nItem,m_arGraphs[nNdx].nGraphIndex);
	}
	
	m_ctlGraphs.SetRedraw();
}

void CResultsDlg::OnSelchangeGraphTypes() 
{
	if(m_pCurrentGraph)
	{
		delete m_pCurrentGraph;
		m_pCurrentGraph = NULL;
	}

	int nCurSel = m_ctlGraphs.GetCurSel();
	if(nCurSel == LB_ERR)
		return;

	int nIndex = m_ctlGraphs.GetItemData(nCurSel);
	
	for(int nGraph=0;nGraph<m_arGraphs.GetSize();nGraph++)
	{
		GraphType &GT = m_arGraphs[nGraph];
		if(GT.nGraphIndex != nIndex)
			continue;

		CRect rcClient,rcList;
		GetClientRect(&rcClient);
		m_ctlRuns.GetWindowRect(&rcList);
		ScreenToClient(&rcList);
		rcClient.top = rcList.bottom + 10;

		if(GT.strPlugin.IsEmpty())
		{
			if(GT.strGraphTitle == m_strAuditLogName)
			{
				CString strItem;
				int nCurSel = m_ctlRuns.GetCurSel();
				m_ctlRuns.GetText(nCurSel,strItem);

				CXAuditViewer *pAudit = new CXAuditViewer;
				pAudit->Create(NULL,WS_CHILD|WS_VISIBLE,rcClient,this,0);
				pAudit->SetRunFolder(m_strTestFolder + strItem);
				m_pCurrentGraph	= pAudit;
			}
			
			return;
		}
		else
		{
			CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
			CPluginManager *pPluginManager = pApp->GetPluginManager();
			
			CTestResultsPlugin plug(GT.strPlugin, pPluginManager);

			m_pCurrentGraph = plug.DisplayGraph(GT.strGraphTitle,this,rcClient);
			if(m_pCurrentGraph)
				m_pCurrentGraph->ModifyStyle(0,WS_BORDER);

			Invalidate();
			return;
		}
	}
}

CXChartCtrl* CResultsDlg::AddGraph()
{
	CXChartCtrl* pChart = new CXChartCtrl;
	pChart->Create(NULL,WS_CHILD|WS_VISIBLE,CRect(0,50,500,500),this,0);
	m_listCharts.AddTail(pChart);
	return pChart;
}

void CResultsDlg::SetTestFile(CTestFile* pFile)
{
	m_pTestFile = pFile;

	CString strTestFile(m_pTestFile->GetTestFilePath());
	int nPos = strTestFile.ReverseFind('\\');
	
	if(nPos!=-1)
		m_strTestFolder = strTestFile.Left(nPos+1);
}


void CResultsDlg::OnDestroy() 
{
	CDragDropDialog::OnDestroy();
	
	delete m_pCurrentGraph;
	m_pCurrentGraph = NULL;
}

void CResultsDlg::OnShowhideBtn() 
{
	m_bFrameHidden = !m_bFrameHidden;

	m_ctlGraphs.ShowWindow(m_bFrameHidden ? SW_HIDE : SW_SHOW);
	m_ctlRuns.ShowWindow(m_bFrameHidden ? SW_HIDE : SW_SHOW);
	m_ctlFilters.ShowWindow(m_bFrameHidden ? SW_HIDE : SW_SHOW);
	m_ctlGraphsCaption.ShowWindow(m_bFrameHidden ? SW_HIDE : SW_SHOW);
	m_ctlRunsCaption.ShowWindow(m_bFrameHidden ? SW_HIDE : SW_SHOW);

	CRect rcClient;
	GetClientRect(&rcClient);
	OnSize(SW_NORMAL,rcClient.Width(),rcClient.Height());

	m_bmpShowHide.DeleteObject();
	m_bmpShowHide.LoadBitmap(m_bFrameHidden ? IDB_DOWN_ARROW : IDB_UP_ARROW);
	m_ctlShowHideBtn.SetBitmap((HBITMAP)m_bmpShowHide);

	Invalidate();
}

