/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TestFile.cpp: implementation of the CTestFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TestPlugin.h"
#include "TestFile.h"
#include "RegFunctions.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTestFile::CTestFile()
{
}

CTestFile::~CTestFile()
{
	ClearArrays();
}

void CTestFile::Serialize(CArchive& ar)
{
	const int nCurrentVersion = 0;
	if(ar.IsStoring())
	{
		ar.Write(&nCurrentVersion,sizeof(int));		//write a version number to file so it is ease to change the structure later

		ar.WriteString(m_strFilePath);
		ar.WriteString("\n");

		int nNumGroups = m_arGroups.GetSize();
		ar.Write(&nNumGroups,sizeof(int));
				
		for(int nGroup=0;nGroup<nNumGroups;nGroup++)
		{
			TaskGroup *pGroup = m_arGroups[nGroup];
			
			ar.WriteString(pGroup->strHost);
			ar.WriteString("\n");

			ar.WriteString(pGroup->strName);
			ar.WriteString("\n");

			ar.Write(&(pGroup->dwDataSize),sizeof(DWORD));
			ar.Write(pGroup->pData,pGroup->dwDataSize);

			int nNumTasks = pGroup->m_arTasks.GetSize();
			ar.Write(&nNumTasks,sizeof(int));
			
			for(int nTask=0;nTask<nNumTasks;nTask++)
			{
				Task *pTask = pGroup->m_arTasks[nTask];

				ar.Write(&(pTask->nType),sizeof(int));
				ar.WriteString(pTask->strName);
				ar.WriteString("\n");
				ar.WriteString(pTask->strPath);
				ar.WriteString("\n");
				ar.WriteString(pTask->strTarget);
				ar.WriteString("\n");
				ar.Write(&(pTask->dwDataSize),sizeof(DWORD));
				ar.Write(pTask->pData,pTask->dwDataSize);
			}
		}
	}
	else
	{
		ClearArrays();

		int nVersion;
		if(!ar.Read(&nVersion,sizeof(int)))
			return;

		if(nVersion == 0)
		{
			ar.ReadString(m_strFilePath);

			int nNumGroups;
			ar.Read(&nNumGroups,sizeof(int));

			m_arGroups.SetSize(nNumGroups);
			
			for(int nGroup=0;nGroup<nNumGroups;nGroup++)
			{
				TaskGroup *pGroup = new TaskGroup;
				m_arGroups[nGroup] = pGroup;
				
				ar.ReadString(pGroup->strHost);

				ar.ReadString(pGroup->strName);

				ar.Read(&(pGroup->dwDataSize),sizeof(DWORD));
				if(pGroup->dwDataSize)
				{
					pGroup->pData = new BYTE[pGroup->dwDataSize];
					ar.Read(pGroup->pData,pGroup->dwDataSize);
				}
				else
					pGroup->pData = NULL;
				
				int nNumTasks;
				ar.Read(&nNumTasks,sizeof(int));	
		
				pGroup->m_arTasks.SetSize(nNumTasks);

				for(int nTask=0;nTask<nNumTasks;nTask++)
				{
					Task *pTask = new Task;
					pGroup->m_arTasks[nTask] = pTask;
						
					ar.Read(&(pTask->nType),sizeof(int));
					ar.ReadString(pTask->strName);
					ar.ReadString(pTask->strPath);
					ar.ReadString(pTask->strTarget);
					ar.Read(&(pTask->dwDataSize),sizeof(DWORD));
					if(pTask->dwDataSize)
					{
						pTask->pData = new BYTE[pTask->dwDataSize];
						ar.Read(pTask->pData,pTask->dwDataSize);
					}
					else
						pTask->pData = NULL;
				}
			}
		}
	}
}

void CTestFile::ClearArrays()
{
	//Clear arrays
	for(int nGroup=0;nGroup<m_arGroups.GetSize();nGroup++)
	{
		for(int nTask=0;nTask<m_arGroups[nGroup]->m_arTasks.GetSize();nTask++)
		{
			delete []m_arGroups[nGroup]->m_arTasks[nTask]->pData;
			delete m_arGroups[nGroup]->m_arTasks[nTask];
		}
		m_arGroups[nGroup]->m_arTasks.RemoveAll();

		delete []m_arGroups[nGroup]->pData;
		delete m_arGroups[nGroup];
	}
	m_arGroups.RemoveAll();
}

int CTestFile::AddNewTaskGroup()
{
	//Get Test name
	int nSlashPos = m_strFilePath.ReverseFind('\\');
	CString strTestName( m_strFilePath.Mid(nSlashPos+1));
	int nDotPos = strTestName.ReverseFind('.');
	strTestName = strTestName.Left(nDotPos);

	TaskGroup *pGroup = new TaskGroup;
	pGroup->strHost = "localhost";
	pGroup->strName.Format("%s_%d",strTestName,m_arGroups.GetSize()+1);
	pGroup->pData = NULL;
	pGroup->dwDataSize = 0;
	return m_arGroups.Add(pGroup);
}

BOOL CTestFile::SetTaskGroupHost(int nGroup,const CString& strHost)
{
	m_arGroups[nGroup]->strHost = strHost;
	return true;
}

int CTestFile::AddNewTask(int nGroup,TaskType nType,const CString& strName,const CString& strPath,const CString& strTarget)
{
	Task *pTask = new Task;
	pTask->nType = nType;
	pTask->strName = strName;
	pTask->strPath = GetRelativePath(strPath);
	pTask->strTarget = strTarget;
	pTask->dwDataSize = 0;
	pTask->pData = NULL;

	return m_arGroups[nGroup]->m_arTasks.Add(pTask);
}

BOOL CTestFile::SetTaskData(int nGroup,int nTask,LPBYTE pData,DWORD dwDataSize)
{
	Task *pTask = m_arGroups[nGroup]->m_arTasks[nTask];
	pTask->dwDataSize = dwDataSize;
	pTask->pData = pData;
	
	return true;
}

CString CTestFile::GetTaskPath(int nGroup,int nTask)
{
	return GetRepositoryPath() + m_arGroups[nGroup]->m_arTasks[nTask]->strPath;
}

CTestFile::TaskType CTestFile::GetTaskType(int nGroup,int nTask)
{
	return m_arGroups[nGroup]->m_arTasks[nTask]->nType;
}

CString CTestFile::GetTaskName(int nGroup,int nTask)
{
	return m_arGroups[nGroup]->m_arTasks[nTask]->strName;
}

BOOL CTestFile::GetTaskData(int nGroup,int nTask,LPBYTE* pData,DWORD& dwBufSize)
{
	*pData = m_arGroups[nGroup]->m_arTasks[nTask]->pData;
	dwBufSize = m_arGroups[nGroup]->m_arTasks[nTask]->dwDataSize;

	return true;
}

BOOL CTestFile::SetTask(int nGroup,int nTask,TaskType nType,const CString& strName,const CString& strPath,const CString& strTarget)
{
	Task *pTask = m_arGroups[nGroup]->m_arTasks[nTask];
	
	pTask->nType = nType;
	pTask->strName = strName;
	pTask->strPath = GetRelativePath(strPath);
	pTask->strTarget = strTarget;
	pTask->dwDataSize = 0;
	pTask->pData = NULL;

	return TRUE;
}

int CTestFile::GetNumTasks(int nGroup)
{
	return m_arGroups[nGroup]->m_arTasks.GetSize();
}

CString CTestFile::GetTaskGroupHost(int nGroup)
{
	return m_arGroups[nGroup]->strHost;
}

int CTestFile::GetNumTaskGroups()
{
	return m_arGroups.GetSize();
}

CString CTestFile::GetTaskTargetType(int nGroup,int nTask)
{
	return m_arGroups[nGroup]->m_arTasks[nTask]->strTarget;
}

void CTestFile::DeleteItem(int nGroup,int nTask)
{
	delete []m_arGroups[nGroup]->m_arTasks[nTask]->pData;
	m_arGroups[nGroup]->m_arTasks.RemoveAt(nTask);
	
	if(!m_arGroups[nGroup]->m_arTasks.GetSize())
		m_arGroups.RemoveAt(nGroup);
}

void CTestFile::Clear()
{
	m_strFilePath.Empty();
	ClearArrays();
}

BOOL CTestFile::GetTaskGroupData(int nGroup,LPBYTE* pData,DWORD &dwBufSize)
{
	*pData = m_arGroups[nGroup]->pData;
	dwBufSize = m_arGroups[nGroup]->dwDataSize;

	return true;
}

BOOL CTestFile::SetTaskGroupData(int nGroup,LPBYTE pData,DWORD dwDataSize)
{
	m_arGroups[nGroup]->dwDataSize = dwDataSize;
	m_arGroups[nGroup]->pData = pData;

	return true;
}

CString CTestFile::GetTestFilePath()
{
	return GetRepositoryPath() + m_strFilePath;
}

void CTestFile::SetTestFilePath(const CString& strPath)
{
	m_strFilePath = GetRelativePath(strPath);
}

CString CTestFile::GetRelativePath(const CString& strPath)
{
	CString strRelPath(strPath);
	CString strReposPath = GetRepositoryPath();
	int nReposPathLen = strReposPath.GetLength();
	if(strPath.Left(strReposPath.GetLength()).CompareNoCase(strReposPath)==0)
		strRelPath.Delete(0,nReposPathLen);
	
	return strRelPath;
}

BOOL CTestFile::SetTaskGroupName(int nGroup, const CString& strName)
{
	m_arGroups[nGroup]->strName = strName;
	return true;
}

CString CTestFile::GetTaskGroupName(int nGroup)
{
	return m_arGroups[nGroup]->strName;
}