/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TestPluginDoc.cpp : implementation of the CTestPluginDoc class
//

#include "stdafx.h"
#include "TestPlugin.h"
#include "OstaDefs.h"

#include "TestPluginDoc.h"
#include "SrvrItem.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTestPluginDoc

IMPLEMENT_DYNCREATE(CTestPluginDoc, COleServerDoc)

BEGIN_MESSAGE_MAP(CTestPluginDoc, COleServerDoc)
	//{{AFX_MSG_MAP(CTestPluginDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BEGIN_DISPATCH_MAP(CTestPluginDoc, COleServerDoc)
	//{{AFX_DISPATCH_MAP(CTestPluginDoc)
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

// Note: we add support for IID_ITestPlugin to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the 
//  dispinterface in the .ODL file.

// {0298A257-D4CA-11D4-94C7-00C04F89E392}
static const IID IID_ITestPlugin =
{ 0x298a257, 0xd4ca, 0x11d4, { 0x94, 0xc7, 0x0, 0xc0, 0x4f, 0x89, 0xe3, 0x92 } };

BEGIN_INTERFACE_MAP(CTestPluginDoc, COleServerDoc)
	INTERFACE_PART(CTestPluginDoc, IID_ITestPlugin, Dispatch)
END_INTERFACE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTestPluginDoc construction/destruction

CTestPluginDoc::CTestPluginDoc()
{
//	EnableCompoundFile();

	EnableAutomation();

	AfxOleLockApp();

	OpenClipboard(NULL);
	HANDLE hData = GetClipboardData(RegisterClipboardFormat(strClipboardFormat));
	m_strTestPath = (char *)GlobalLock(hData);
	GlobalUnlock(hData);
	CloseClipboard();

	CTestPluginApp* pApp = (CTestPluginApp*)AfxGetApp();
	if (pApp->GetPluginManager() == NULL)
		pApp->InitPlugins();
}

CTestPluginDoc::~CTestPluginDoc()
{
	AfxOleUnlockApp();
}

BOOL CTestPluginDoc::OnNewDocument()
{
	if (!COleServerDoc::OnNewDocument())
		return FALSE;
	
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CTestPluginDoc server implementation

COleServerItem* CTestPluginDoc::OnGetEmbeddedItem()
{
	// OnGetEmbeddedItem is called by the framework to get the COleServerItem
	//  that is associated with the document.  It is only called when necessary.

	CTestPluginSrvrItem* pItem = new CTestPluginSrvrItem(this);
	ASSERT_VALID(pItem);
	return pItem;
}

/////////////////////////////////////////////////////////////////////////////
// CTestPluginDoc Active Document server implementation

CDocObjectServer *CTestPluginDoc::GetDocObjectServer(LPOLEDOCUMENTSITE pDocSite)
{
	return new CDocObjectServer(this, pDocSite);
}



/////////////////////////////////////////////////////////////////////////////
// CTestPluginDoc serialization

void CTestPluginDoc::Serialize(CArchive& ar)
{
	if(ar.IsStoring())
	{
	}
	else
	{
	}

	if(m_strTestPath.IsEmpty())
		m_TestFile.Serialize(ar);
}

/////////////////////////////////////////////////////////////////////////////
// CTestPluginDoc diagnostics

#ifdef _DEBUG
void CTestPluginDoc::AssertValid() const
{
	COleServerDoc::AssertValid();
}

void CTestPluginDoc::Dump(CDumpContext& dc) const
{
	COleServerDoc::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTestPluginDoc commands



BOOL CTestPluginDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (!COleServerDoc::OnOpenDocument(lpszPathName))
		return FALSE;
	
	if(!m_strTestPath.IsEmpty())
	{
		m_TestFile.SetTestFilePath(m_strTestPath);

		CFile f;
		CFileException e;
		if(f.Open(m_strTestPath,CFile::modeRead,&e))
		{
			CArchive arTst(&f,CArchive::load);
			m_TestFile.Serialize(arTst);
			arTst.Close();
			SetModifiedFlag(FALSE);
		}
	}
	
	return TRUE;
}

BOOL CTestPluginDoc::OnSaveDocument(LPCTSTR lpszPathName) 
{
	if(IsModified() && !m_strTestPath.IsEmpty())
	{
		CFile f;
		if(f.Open(m_strTestPath,CFile::modeWrite|CFile::modeCreate))
		{
			CArchive arTst(&f,CArchive::store);
			m_TestFile.Serialize(arTst);
			arTst.Close();
			SetModifiedFlag(FALSE);
			return true;
		}
	}

	return COleServerDoc::OnSaveDocument(lpszPathName);
}


void CTestPluginDoc::OnDeactivate() 
{
	if(IsModified())
		OnSaveDocument(m_strTestPath);
	
	COleServerDoc::OnDeactivate();
}
