/* Copyright (C) 2001 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ResultsDlg.cpp : implementation file
//

#include "stdafx.h"
#include "XAuditViewer.h"
#include "ResultsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CResultsDlg dialog


CResultsDlg::CResultsDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CResultsDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CResultsDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CResultsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CResultsDlg)
	DDX_Control(pDX, IDC_TYPE, m_ctlAuditType);
	DDX_Control(pDX, IDC_RESULTSLIST, m_ctlResultsList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CResultsDlg, CDialog)
	//{{AFX_MSG_MAP(CResultsDlg)
	ON_WM_SIZE()
	ON_CBN_SELCHANGE(IDC_TYPE, OnSelchangeType)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CResultsDlg message handlers

BOOL CResultsDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	int nItem = m_ctlAuditType.AddString("Audit");
	m_ctlAuditType.SetItemData(nItem,enAudit);
	nItem = m_ctlAuditType.AddString("Report");
	m_ctlAuditType.SetItemData(nItem,enReport);
	nItem = m_ctlAuditType.AddString("History");
	m_ctlAuditType.SetItemData(nItem,enHistory);

	m_ctlAuditType.SetCurSel(enAudit);
	ShowType(enAudit);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CResultsDlg::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);

	if(!m_ctlResultsList.GetSafeHwnd())
		return;

	CRect rcList;
	m_ctlResultsList.GetWindowRect(&rcList);
	ScreenToClient(&rcList);

	m_ctlResultsList.SetWindowPos(NULL,0,0,cx-(rcList.left*2),cy-rcList.top-rcList.left,SWP_NOZORDER|SWP_NOMOVE);
}

BOOL CResultsDlg::SetRunFolder(const CString& strPath)
{
	m_strRunFolder = strPath;

	//Load the data
	int nCurSel = m_ctlAuditType.GetCurSel();
	if(nCurSel == CB_ERR)
		return false;

	return ShowType( (AuditType)m_ctlAuditType.GetItemData(nCurSel));
}


void CResultsDlg::OnSelchangeType() 
{
	int nCurSel = m_ctlAuditType.GetCurSel();
	if(nCurSel == CB_ERR)
		return;
	
	ShowType( (AuditType)m_ctlAuditType.GetItemData(nCurSel));
}

BOOL CResultsDlg::ShowType(AuditType nType)
{
	m_ctlResultsList.DeleteAllItems();

	switch(nType)
	{
	case enAudit:
		return ShowAudit();
	case enReport:
		return ShowReport();
	case enHistory:
		return ShowHistory();
	default:
		ASSERT(FALSE);
	break;
	}
	return false;
}

BOOL CResultsDlg::ShowAudit()
{
	typedef enum { enTimeCol,enThreadCol,enScriptCol,enMsgCol} ListColumns;
    
	m_ctlResultsList.SetRedraw(FALSE);
	m_ctlResultsList.SetFixedRowCount(1);
	m_ctlResultsList.InsertColumn("Time Stamp",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("User ID",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Script",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Message",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	
    int nRow = 1;
    try
    {
        CString strAuditPath(m_strRunFolder  + "\\TestLog.txt");
        CStdioFile AuditFile(strAuditPath,CFile::modeRead);
        
        CString strLine;
        while(AuditFile.ReadString(strLine))
        {
			int nStartPos = 0;
			int nEndPos = strLine.Find(',',0);
			if(nEndPos == -1)
				continue;
			
			CTime time(atol( strLine.Mid(nStartPos,nEndPos-nStartPos) ));
			CString strTime( time.Format("%d/%m/%y %H:%M:%S"));
			nStartPos = nEndPos+1;

            //    ...parse '|' separated message
			CString strThread,strScript,strMsg;//,strExe,strPID,strObjNdx;
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
//				strExe = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}

			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
//				strPID = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
				strThread = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
				strScript = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
//				strObjNdx = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find(',',nStartPos))!=-1)
			{
				strMsg = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}

			m_ctlResultsList.InsertRow("");
            m_ctlResultsList.SetItemText(nRow,enTimeCol,strTime);
			m_ctlResultsList.SetItemText(nRow,enThreadCol,strThread);
			m_ctlResultsList.SetItemText(nRow,enScriptCol,strScript);
			m_ctlResultsList.SetItemText(nRow,enMsgCol,strMsg);

            //    ...and move onto the next entry
            nRow++;
        }
		m_ctlResultsList.AutoSizeColumns();
	}
    catch(...)
    {
	}

	m_ctlResultsList.SetRedraw(TRUE);
    return true;
}

BOOL CResultsDlg::ShowReport()
{
	typedef enum { enTimeCol,enThreadCol,enScriptCol,enMsgCol} ListColumns;
    
	m_ctlResultsList.SetRedraw(FALSE);
	m_ctlResultsList.SetFixedRowCount(1);
	m_ctlResultsList.InsertColumn("Time Stamp",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("User ID",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Script",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Message",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	
    int nRow = 1;
    try
    {
        CString strAuditPath(m_strRunFolder  + "\\TestRep.txt");
        CStdioFile AuditFile(strAuditPath,CFile::modeRead);
        
        CString strLine;
        while(AuditFile.ReadString(strLine))
        {
			int nStartPos = 0;
			int nEndPos = strLine.Find(',',0);
			if(nEndPos == -1)
				continue;
			
			CTime time(atol( strLine.Mid(nStartPos,nEndPos-nStartPos) ));
			CString strTime( time.Format("%d/%m/%y %H:%M:%S"));
			nStartPos = nEndPos+1;

            //    ...parse '|' separated message
			CString strThread,strScript,strMsg;//,strExe,strPID,strObjNdx;
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
//				strExe = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}

			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
//				strPID = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
				strThread = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
				strScript = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
//				strObjNdx = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find(',',nStartPos))!=-1)
			{
				strMsg = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}

			m_ctlResultsList.InsertRow("");
            m_ctlResultsList.SetItemText(nRow,enTimeCol,strTime);
			m_ctlResultsList.SetItemText(nRow,enThreadCol,strThread);
			m_ctlResultsList.SetItemText(nRow,enScriptCol,strScript);
			m_ctlResultsList.SetItemText(nRow,enMsgCol,strMsg);

            //    ...and move onto the next entry
            nRow++;
        }

		m_ctlResultsList.AutoSizeColumns();
    }
    catch(...)
    {
	}

	m_ctlResultsList.SetRedraw(TRUE);
    return true;
}

BOOL CResultsDlg::ShowHistory()
{
	m_ctlResultsList.SetRedraw(FALSE);
	m_ctlResultsList.SetFixedRowCount(1);
	m_ctlResultsList.InsertColumn("Time Stamp",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("User ID",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Script",DT_LEFT|DT_VCENTER|DT_SINGLELINE);
	m_ctlResultsList.InsertColumn("Message",DT_LEFT|DT_VCENTER|DT_SINGLELINE);

	int nSlashPos = m_strRunFolder.ReverseFind('\\');
	CString strTestFolder(m_strRunFolder.Left(nSlashPos));

	BOOL bResult = true;
	CFileFind FF;
	if(FF.FindFile(strTestFolder + "\\*.*"))
	{
		BOOL bContinue = true;
		while(bContinue)
		{
			bContinue = FF.FindNextFile();
			if(FF.IsDirectory() && !FF.IsDots() && !FF.IsHidden())
				bResult &= ShowHistoryInPath(FF.GetFilePath());
		}
	}
	m_ctlResultsList.SetRedraw(TRUE);

	return bResult;
}

BOOL CResultsDlg::ShowHistoryInPath(const CString& strRunFolder)
{
	typedef enum { enTimeCol,enThreadCol,enScriptCol,enMsgCol} ListColumns;
	
    int nRow = m_ctlResultsList.GetRowCount();
    try
    {
        CString strAuditPath(strRunFolder  + "\\TestHis.txt");
        CStdioFile AuditFile(strAuditPath,CFile::modeRead);
        
        CString strLine;
        while(AuditFile.ReadString(strLine))
        {
			int nStartPos = 0;
			int nEndPos = strLine.Find(',',0);
			if(nEndPos == -1)
				continue;
			
			CTime time(atol( strLine.Mid(nStartPos,nEndPos-nStartPos) ));
			CString strTime( time.Format("%d/%m/%y %H:%M:%S"));
			nStartPos = nEndPos+1;

            //    ...parse '|' separated message
			CString strThread,strScript,strMsg;//,strExe,strPID,strObjNdx;
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
//				strExe = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}

			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
//				strPID = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
				strThread = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
				strScript = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find('|',nStartPos))!=-1)
			{
//				strObjNdx = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}
			
			if((nEndPos = strLine.Find(',',nStartPos))!=-1)
			{
				strMsg = strLine.Mid(nStartPos,nEndPos-nStartPos);
				nStartPos = nEndPos+1;
			}

			m_ctlResultsList.InsertRow("");
            m_ctlResultsList.SetItemText(nRow,enTimeCol,strTime);
			m_ctlResultsList.SetItemText(nRow,enThreadCol,strThread);
			m_ctlResultsList.SetItemText(nRow,enScriptCol,strScript);
			m_ctlResultsList.SetItemText(nRow,enMsgCol,strMsg);

            //    ...and move onto the next entry
            nRow++;
        }

		m_ctlResultsList.AutoSizeColumns();
    }
    catch(...)
    {
	}

    return true;
}
