/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     TMMain
// @module  Test Manager Main | This is the main control moduler.
//
///////////////////////////////////////////////////////////////////////////////

#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif

#ifndef CYRANO_TP_STANDALONE
#include "CORBA/CORBAServer.h"
#else
#include <CECSACORBA.h>
#include "ctime.hpp"
#endif

#ifdef CYR_TRACE
#include "TPLog.h"
#endif

#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif

#ifdef CYRANO_TEST_MANAGER
#include "TestManager_i.h"
#else
#include "TestExecuter_i.h"
#endif

#include <string>
#include <new.h>
#include "mdbexcept.h"
#include "mdbgetopt.h"
#include "cxxutils.h"
#include <CyrHelper.inl>

#ifdef CYRANO_TEST_MANAGER
#include "TMthreadcontrol.h"
#else
#include "TEthreadcontrol.h"
#endif

#include "mdbtypes2.h"

extern "C"
{
#include <psl_macros.h>
#include <psl_rtndef.h>
}

#ifdef WIN32
using std::string;
#endif

#ifdef CYRANO_TEST_MANAGER
#define     VERSION      "3.0.0.0"      // Test Manager version
#define     ODLFILE      "TestManager.odl"  // Test Manager's Odl File
const string    TPTEXT  = "TestManager";    // Used for Tracing filename
#elif defined(CYRANO_TEST_EXECUTER)
#define     VERSION      "3.0.0.0"
#define     ODLFILE      "TestExecuter.odl"
const string    TPTEXT  = "TestExecuter_cmd";
#elif defined(CYRANO_MSSQL_EXECUTER)
#define     VERSION      "3.0.0.0"
#define     ODLFILE      "TEMSsql.odl"
const string    TPTEXT  = "TestExecuter_mss";
#elif defined(CYRANO_SYBASE_EXECUTER)
#define     VERSION      "3.0.0.0"
#define     ODLFILE      "TESyb.odl"
const string    TPTEXT  ="TestExecuter_syb";
#elif defined(CYRANO_ORACLE_EXECUTER)
#define     VERSION      "3.0.0.0"
#define     ODLFILE      "TEOra.odl"
const string    TPTEXT  = "TestExecuter_ora";
#elif defined(CYRANO_HTTP_EXECUTER)
#define     VERSION      "3.0.0.0"
#define     ODLFILE      "TEHttp.odl"
const string    TPTEXT  = "TestExecuter_web";
#else
#error "Undefined TestManager/TestProcess"
#endif

// Environment variable for determining maximum number of threads per Test Executer
#define     TE_MAX_THREADS  "TE_MAX_THREADS"

// Local functions
int     Run (CORBAServer* pServer,string exename);
void    sqlr_cleanup (void);

static CTMutex cleanup_mutex;

//The general idea here is to have all TestManagerImpl Access done in this file
//hopefully removing the need to have it as a fully global variable
//This should encourage minimal access from separate threads
#ifdef CYRANO_TEST_MANAGER
#define TESTPROCESSIMPL     TestManagerImpl
#define TESTPROCESS     TestManager
#define TPROCESS        TManager
#define TESTPROCESS_PTR     TestManager_ptr
#define TPROCESS_PTR        TManager_ptr
#define TESTPROCESS_NIL     TestManager::_nil()
#define THREADCONTROL       TMthreadcontrol
#else
#define TESTPROCESSIMPL     TestExecuterImpl
#define TESTPROCESS     TestExecuter
#define TPROCESS        TExecuter
#define TESTPROCESS_PTR     TestExecuter_ptr
#define TPROCESS_PTR        TExecuter_ptr
#define TESTPROCESS_NIL     TestExecuter::_nil()
#define THREADCONTROL       TEthreadcontrol
#endif
static TESTPROCESSIMPL*     TPROCESS=NULL;

#ifndef CYRANO_TP_STANDALONE
//But just in case we do need to access it elsewhere here is a _ptr ptr
TESTPROCESS_PTR         TPROCESS_PTR=TESTPROCESS_NIL;
#endif
// Thread control object pointer
THREADCONTROL*      tc= NULL;
Cthreadcontrol*     Ctc=NULL;
Cmdbthread* stopthread=NULL;


string exedir="";
string filevardir="";
string toffiledir="";

// Global declaration for stopping execution
TScriptRule* rule = NULL;
TaskScript* ts = NULL;


extern "C"
{
    __declspec( dllexport ) int TELibMain(HWND hwind, char *TofFile, char *OdlDir)
    {
#ifdef CYR_TRACE
        logr = new TPLog();
#endif

        if (hwind == NULL)
        {
#ifdef CYR_TRACE
            // Detected error while checking args
            logr->AddTrace(STARTUP,ERRORLEVEL,"Handle for window is NULL !");
#endif
            sqlr_cleanup();
            return 1;
        }

        gHwnd = hwind;

        // Inform modeller the replay is beginning
        logr->AddTrace(STARTUP, TRACELEVEL, "!Begin replay" );

        // Set handler for out of memory conditions in new operator.
        SET_NEW_HANDLER(mdb_new_handler);

        // Maps malloc failure to new fail handler on WIN32 only.
        SET_NEW_MODE
        //initialise CORBA for logging purposes
        CORBAServer* pServer;
        try
        {
            pServer=new CORBAServer(0, NULL);
        }
        catch(...)
        {
            logr->AddTrace(STARTUP,ERRORLEVEL,"Caught Exception on startup, is Nameservice Running?" );
            sqlr_cleanup();
            return 1;
        }


        // Get pid for this TestProcess
        MDBPID tppid = getpid();


        // Read in atvf(tof) file to create the Script Rules, and commence execution
        try
        {
            exedir = OdlDir;
            filevardir = TofFile;
            // remove TOF file name
            int lastSlash = filevardir.find_last_of("\\");
            if (lastSlash != string::npos)
            {
                filevardir.erase(lastSlash);
                toffiledir = filevardir + "\\";

                // remove TOF file folder name
                /*          lastSlash = filevardir.find_last_of("\\");
                            if (lastSlash != string::npos)
                            {
                                filevardir.erase(lastSlash + 1);
                                filevardir.append("Data");
                            }
                */
            }
            else
            {
                filevardir = "";
            }


            Run(NULL,TofFile);

            //
            //    All done - perform any cleanup required
            //
            delete pServer;

        }
        catch(string es)
        {
            // Detected error
#ifdef CYR_TRACE
            logr->AddTrace(STARTUP,ERRORLEVEL,es.c_str());
#endif
            sqlr_cleanup();
            return 1;
        }
        catch(mdbIOException& e)
        {
#ifdef CYR_TRACE
            string msg=e.what()+": "+e.ioname();
            logr->AddTrace(STARTUP,ERRORLEVEL,msg.c_str());
#endif
            sqlr_cleanup();
            return 1;
        }
        catch(mdbParseException& e)
        {
#ifdef CYR_TRACE
            logr->AddTrace(STARTUP,ERRORLEVEL,e.what().c_str());
#endif
            sqlr_cleanup();
            return 1;
        }
        catch(CyrRepositoryException& e)
        {
#ifdef CYR_TRACE
            logr->AddTrace(STARTUP,ERRORLEVEL,e.what().c_str());
#endif
            sqlr_cleanup();
            return 1;
        }
        catch(exception& e)
        {
#ifdef CYR_TRACE
            logr->AddTrace(STARTUP,ERRORLEVEL,e.what());
#endif
            sqlr_cleanup();
            return 1;
        }
        catch( ... )
        {
            // Some other unexpected exception caught
#ifdef CYR_TRACE
            logr->AddTrace(STARTUP,ERRORLEVEL,"Caught exception");
#endif
            sqlr_cleanup();
            return 1;
        }

        return 0;

    }// end of MAIN



    __declspec( dllexport ) void TELibStop(int i)
    {
#ifdef CYR_TRACE
        if (i != 0)
        {
            if(logr)
                logr->AddTrace(SHUTDOWN,ERRORLEVEL,"Stop execution.");
        }
#endif


        if (Ctc)
        {
            Cmdbthread* tp = NULL;
            tp = Ctc->get_threadp(0);

            if ( tp && i )
            {
                tp->abort();
                return;
            }

            if (Ctc)
            {
                delete Ctc;
                Ctc = NULL;
            }
        }

        CyrSleep(1000);

        if (ts)
        {
            delete ts;
            ts = NULL;
        }

        if (rule)
        {
            delete rule;
            rule = NULL;
        }

        if(logr)
        {
            delete logr;
            logr = NULL;
        }
    }

} // end extern "C"


// sqlr_cleanup
// ============
//
//
void sqlr_cleanup(void)
{
    //no-one else here for a bit!
    CTLockGuard<CTMutex> guard(cleanup_mutex);//use guard to deal with exceptions
    // Catch any exceptions during cleanup. This could happen if
    // a thread was hard killed, leaving some object in an undefined
    // state.
#ifdef CYR_TRACE
    if(logr)
        logr->AddTrace(SHUTDOWN,DEBUGLEVEL,"Clean up on Exiting ");
#endif

    try
    {

        //delete threadcontrol before TManager as this
        //will release objects held in each thread
#ifdef CYR_TRACE
        if(logr)logr->AddTrace(SHUTDOWN,DEBUGLEVEL,"Clear down ThreadControl");
#endif


#ifdef CYR_TRACE
        if(logr)logr->AddTrace(SHUTDOWN,DEBUGLEVEL,"Clear down CORBA Object");
#endif

        if (TPROCESS)
        {
            TPROCESS->SetToCloseDown();
        }

#ifdef CYR_TRACE
        if(logr)logr->AddTrace(SHUTDOWN,DEBUGLEVEL,"Clear down Static Info");
#endif

        //      if(cl)
        //          delete cl;
    }

    catch(CyrRepositoryException& e)
    {
#ifdef CYR_TRACE
        strstream logstr;
        logstr<< e.what() <<"," <<e.where()<<" Status: "<<e.status()<< ends;
        if(logr)logr->AddTrace(SHUTDOWN,ERRORLEVEL,logstr.str());
        logstr.freeze(false);
#endif
        exit(0);
    }
    catch(exception &e)
    {
#ifdef CYR_TRACE
        string msg=string("Caught exception in clean-up :")+string(e.what());
        if(logr)logr->AddTrace(SHUTDOWN,ERRORLEVEL,msg.c_str());
#endif
        exit(0);
    }
    catch( ... )
    {
#ifdef CYR_TRACE
        if(logr)logr->AddTrace(SHUTDOWN,ERRORLEVEL,"Caught exception in clean-up ");
        exit(0);
#endif
    }

#ifdef CYR_TRACE
    if(logr)
        logr->AddTrace(SHUTDOWN,DEBUGLEVEL,"Clear out Logging");
    if(logr)
    {
        delete logr;
        logr=NULL;
    }
#endif

} //end of sqlr cleanup

//
//    Run
//
//    This is the main processing section of sqlr.
//
int Run (CORBAServer* pServer, string TofFile)
{

#ifdef CYR_TRACE
    logr->AddTrace(DBGSTD,DEBUGLEVEL,"Before TScript");
#endif
    // Create rule for this apps file type
    string odlfile;
    // create odlfile

    // Build odl file name
    odlfile = exedir + ODLFILE;

#ifdef CYR_TRACE
    string msg="Using odl file: " + odlfile;
    logr->AddTrace(STARTUP,DEBUGLEVEL,msg.c_str());
#endif

    rule = new TScriptRule(odlfile);

#ifdef CYR_TRACE
    logr->AddTrace(DBGSTD,DEBUGLEVEL,"After TScript");
#endif
    //
    //    Initialize the CORBA Initialise - pends Test Manager
    //
    //Create thread control object
    //Get MAXIMUM number of threads required for process if a TestExecuter
#ifndef CYRANO_TEST_MANAGER
    char* c_maxthr=getenv(TE_MAX_THREADS);
    int maxthr;
    if(c_maxthr!=NULL)
    {
        maxthr=atoi(c_maxthr);
    }
    else
    {
        maxthr=0;
    }
    if(maxthr>0)//if managed to get a valid number from environment variable
        //then use for maximum threads
        //otherwise use default defined in threadcontrol.cxx
    {
        tc = new THREADCONTROL(rule,maxthr);
    }
    else
    {
        tc = new THREADCONTROL(rule);
    }
#else
    //Create For TestManager ThreadControl so that New TM threads are created
    tc = new THREADCONTROL(rule);
#endif
    Ctc=tc;//set up global pointer to base class for use in TContext
    // Get the test name passed in on the command line - if any at
    // all. This shall determine in External mode whether this object
    // shall act as a Test Manager(TestName) or Test Executer(no TestName)

    int ProcessID = getpid();

    string HostName="localhost";
    // Set the name to be registered, For test managers this is<nodename.project.test.testname.pid>
    char    strProcess[64];

    sprintf(strProcess, "%s.%8.8lX", HostName.c_str(), ProcessID);

    //    Create the instance of the test manager CORBA object
    TPROCESS=new TESTPROCESSIMPL(pServer, string(strProcess), HostName, ProcessID, TofFile, rule, maxthr);

    TPROCESS->Initialise("","","","");

    string FullTestName = TofFile;
    if(FullTestName=="")
        throw string("No Tof file given for Execution");

    int status = tc->AttachThread(FullTestName.c_str());

    while (tc->get_nactive()<1)//wait for first thread to start
    {
        CyrSleep(1000);
    }

    return 0;
}
