/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"

#include "TEMainStructure.hpp"

CMainExecuterStructure g_TExecuter;

extern "C"
{

    __declspec(dllexport) int TELibMain(HWND hwind, char *TofFile, char *OdlDir)
    {
        if(g_TExecuter.Init(hwind, TofFile, OdlDir) == FALSE)
            return 1;

        // Inform modeller the replay is beginning
        logr->AddTrace(STARTUP, TRACELEVEL, "!Begin replay" );

        BOOL bSucceeded = TRUE;
        string msg;

	    // Read in atvf(tof) file to create the Script Rules, and commence execution
        try
        {
            bSucceeded = g_TExecuter.Run();
        }
        catch(string es)
        {
            bSucceeded = FALSE;
            msg = es;
        }
        catch(mdbIOException& e)
        {
            msg = e.what() + ": " + e.ioname();
            bSucceeded = FALSE;
        }
        catch(mdbParseException& e)
        {
            bSucceeded = FALSE;
            msg = e.what();
        }
        catch(CyrRepositoryException& e)
        {
            bSucceeded = FALSE;
            msg = e.what();
        }
        catch(exception& e)
        {
            bSucceeded = FALSE;
            msg = e.what();
        }
        catch( ... )
        {
            bSucceeded = FALSE;
            msg = "Unknwon exception caught";
        }

        if(bSucceeded == FALSE)
        {
            logr->AddTrace(STARTUP, ERRORLEVEL, msg.c_str());
            g_TExecuter.Complete();
            return 1;
        }
        else
            return 0;
    }



    __declspec( dllexport ) void TELibStop(int i)
    {
        if (i != 0)
        {
			g_TExecuter.Stop();
        }
        else
        {
            // Not used in the TP model.
        }
    }

} // extern "C"


