/* HTML Parser in Phoenix format */

{
}

digit       [0-9]
integer     {digit}+
letter      [a-zA-Z]
letter2     [a-zA-Z_$#\-\.:/%\+,\(\)\*]
id          ({letter2}|{digit})+
tag_open    \<{letter}({letter}|{digit})*
tag_close   \<\/{letter}({letter}|{digit})*
string      ([\"][^\"]*[\"])|([\'][^\']*[\'])
space       [ \t\r\n]+

%%

%I                      // Turn off case sensitivity

"<!"doctype             {
                            bInTag_ = TRUE;
                            pTokenAttributes->eTag_ = nameToTag["DOCTYPE"];
                            pTokenAttributes->label_.pcBegin_ = pcFirstChar_;
                            pTokenAttributes->label_.ulSize_  = pcCurChar_ - pcFirstChar_;
                            return TK_DOCTYPE;
                        }

%S                      // Turn on case sensitivity

"<!--"                  {CollectComment(*pTokenAttributes); return TK_COMMENT;}

">"                     {
                            if(bInTag_)
                            {
                                bInTag_ = FALSE;    // Exiting a tag (opening or closing).

                                // If we are in a script tag, then advance to </script
                                if(bInScript_)
                                    CollectScript(*pTokenAttributes);

                                return TK_TAGEXIT;
                            }
                            else
                            {
                                // Text.
                                CollectText(*pTokenAttributes);

                                return TK_TEXT;
                            }
                        }

"="                     {
                            if(bInTag_)
                                return TK_EQUALS;
                            else
                            {
                                // Text.
                                CollectText(*pTokenAttributes);

                                return TK_TEXT;
                            }
                        }

{tag_open}              {
                            bInTag_ = TRUE;

                            // Check if we are in a script tag.
                            if(InScript())
                                bInScript_ = TRUE;
                            RecognizeTag(*pTokenAttributes);

                            return TK_START_TAG;
                        }

{tag_close}             {
                            bInTag_ = TRUE;
                            bInScript_ = FALSE;
                            RecognizeTag(*pTokenAttributes);

                            return TK_END_TAG;
                        }

{id}                    {
                            if(bInTag_)
                            {
                                pTokenAttributes->label_.pcBegin_ = pcFirstChar_;
                                pTokenAttributes->label_.ulSize_  = pcCurChar_ - pcFirstChar_;

                                return TK_ID;
                            }
                            else
                            {
                                // Text.
                                CollectText(*pTokenAttributes);

                                return TK_TEXT;
                            }
                        }

{string}                {
                            if(bInTag_)
                            {
                                pTokenAttributes->label_.pcBegin_ = pcFirstChar_ + 1;
                                pTokenAttributes->label_.ulSize_  = pcCurChar_ - pcFirstChar_ - 2;

                                return TK_STRING;
                            }
                            else
                            {
                                // Text.
                                CollectText(*pTokenAttributes);

                                return TK_TEXT;
                            }
                        }

{space}                 {
                            // If in a tag, ignore.
                            // If NOT in a tag, ignore also, because we want to reject
                            // trailing text spaces.
                            // Text.
                        }

.                       {
                            // Text.
                            if(!bInTag_)
                            {
                                // Text.
                                CollectText(*pTokenAttributes);

                                return TK_TEXT;
                            }
                        }

%%
