// TimerList.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "TimerList.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTimerList
extern HINSTANCE m_hInstance;
CTimerList::ListColumn CTimerList::m_nStaticSortColumn = enTest;
BOOL CTimerList::m_bStaticReverseSort = false;

CTimerList::CTimerList(const TimerList& listTimers)
{
	m_arTimers.SetSize(listTimers.GetCount());
	int nIndex=0;
	POSITION pos = listTimers.GetHeadPosition();
	while(pos)
		m_arTimers[nIndex++] = listTimers.GetNext(pos);

	m_bReverseSort = false;
	m_nSortColumn = enTest;
}

CTimerList::~CTimerList()
{
}


BEGIN_MESSAGE_MAP(CTimerList, CListCtrl)
	//{{AFX_MSG_MAP(CTimerList)
	ON_WM_CONTEXTMENU()
	ON_NOTIFY_REFLECT(LVN_GETDISPINFO, OnGetdispinfo)
	ON_WM_CREATE()
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnclick)
	ON_COMMAND(ID_CSV_EXPORT, OnCsvExport)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTimerList message handlers

void CTimerList::OnContextMenu(CWnd* pWnd, CPoint point) 
{
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

    AfxSetResourceHandle(m_hInstance);
    CMenu menu;
	VERIFY(menu.LoadMenu(IDR_TIMERLIST_MENU));
	
    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);

    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,this);
	
	AfxSetResourceHandle(AfxGetInstanceHandle());
}

void CTimerList::OnGetdispinfo(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;

    LV_ITEM &lvItem = pDispInfo->item;
    if(lvItem.mask & LVIF_TEXT)
    {
        int nItem = lvItem.iItem;
        Timer *pTimer = m_arTimers[nItem];
        switch(lvItem.iSubItem)
        {
        case 0:
            lvItem.pszText = pTimer->strTestName.GetBuffer(0);
            break;
        case 1:
            lvItem.pszText = pTimer->strThreadName.GetBuffer(0);
            break;
        case 2:
            lvItem.pszText = pTimer->strScriptName.GetBuffer(0);
            break;
        case 3:
            lvItem.pszText = pTimer->strTimerName.GetBuffer(0);
            break;
        case 4:
            m_strTemp.Format("%u",pTimer->dwActiveUsers);
            lvItem.pszText = m_strTemp.GetBuffer(0);
            break;
        case 5:
            m_strTemp.Format("%f",pTimer->dValue);
            lvItem.pszText = m_strTemp.GetBuffer(0);
            break;
        case 6:
            {
                CTime time(pTimer->dwTimeStamp);
                m_strTemp = time.Format("%d/%m/%y");
                lvItem.pszText = m_strTemp.GetBuffer(0);
            }
            break;
        case 7:
            {
                CTime time(pTimer->dwTimeStamp);
                m_strTemp = time.Format("%H:%M:%S");
                lvItem.pszText = m_strTemp.GetBuffer(0);
            }
            break;
        }
    }

    *pResult = 0;
}

int CTimerList::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CListCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	SetRedraw(FALSE);
    
	ModifyStyle(0,LVS_REPORT);
    // Get the current window style.
    SetExtendedStyle(LVS_EX_FULLROWSELECT);

	// Set the column headings
    InsertColumn(enTest,"Test Name",LVCFMT_LEFT,70,enTest);
    InsertColumn(enThread,"User ID",LVCFMT_LEFT,100,enThread);
    InsertColumn(enScript,"Script Name",LVCFMT_LEFT,70,enScript);
    InsertColumn(enTimer,"Timer Name",LVCFMT_LEFT,70,enTimer);
    InsertColumn(enNumUsers,"Active Users",LVCFMT_LEFT,70,enNumUsers);
    InsertColumn(enElapsed,"Elapsed Time",LVCFMT_LEFT,70,enElapsed);
    InsertColumn(enDate,"Start Date",LVCFMT_LEFT,70,enDate);
    InsertColumn(enTime,"Start Time",LVCFMT_LEFT,70,enTime);

	int nNumTimers = m_arTimers.GetSize();
    SetItemCount(nNumTimers);

    CString strNames;
    int nRow = 0;
    for(int nTimer=0;nTimer<nNumTimers;nTimer++)
	{
        Timer *pTimer = m_arTimers[nTimer];
        if(!pTimer->bInFilter)
            continue;
        
        InsertItem(nRow++,LPSTR_TEXTCALLBACK);
    }

    SetRedraw();

	return 0;
}

// Sort the item in reverse alphabetical order.
int CTimerList::CompareTimer(const void *arg1, const void *arg2 )
{
	Timer* pTimer1 = *(Timer**)arg1;
	Timer* pTimer2 = *(Timer**)arg2;

	CString strItem1,strItem2;
	switch(m_nStaticSortColumn)
	{
	case enTest:
		strItem1 = pTimer1->strTestName;
		strItem2 = pTimer2->strTestName;
	break;
	case enThread:
		strItem1 = pTimer1->strThreadName;
		strItem2 = pTimer2->strThreadName;
	break;
	case enScript:
		strItem1 = pTimer1->strScriptName;
		strItem2 = pTimer2->strScriptName;
	break;
	case enTimer:
		strItem1 = pTimer1->strTimerName;
		strItem2 = pTimer2->strTimerName;
	break;
	case enNumUsers:
	{
		DWORD dwNum1 = pTimer1->dwActiveUsers;
		DWORD dwNum2 = pTimer2->dwActiveUsers;
		
		if(dwNum1 > dwNum2) return m_bStaticReverseSort ? -1 : 1;
		if(dwNum1 < dwNum2) return m_bStaticReverseSort ? 1 : -1;
		return 0;
	}
	break;
	case enElapsed:
	{
		double dVal1 = pTimer1->dValue;
		double dVal2 = pTimer2->dValue;

		if(dVal1 > dVal2) return m_bStaticReverseSort ? -1 : 1;
		if(dVal1 < dVal2) return m_bStaticReverseSort ? 1 : -1;
		return 0;
	}
	break;
	case enDate:
	{
		CTime time1(pTimer1->dwTimeStamp);
		CTime time2(pTimer2->dwTimeStamp);
		
		if(time1.GetYear() > time2.GetYear())	return m_bStaticReverseSort ? -1 : 1;
		if(time1.GetYear() < time2.GetYear())	return m_bStaticReverseSort ? 1 : -1;
		if(time1.GetMonth() > time2.GetMonth()) return m_bStaticReverseSort ? -1 : 1;
		if(time1.GetMonth() < time2.GetMonth()) return m_bStaticReverseSort ? 1 : -1;
		if(time1.GetDay() > time2.GetDay()) return m_bStaticReverseSort ? -1 : 1;
		if(time1.GetDay() < time2.GetDay()) return m_bStaticReverseSort ? 1 : -1;
		return 0;
	}
	break;
	case enTime:
	{
		CTime time1(pTimer1->dwTimeStamp);
		CTime time2(pTimer2->dwTimeStamp);

		if(time1.GetHour() > time2.GetHour())	return m_bStaticReverseSort ? -1 : 1;
		if(time1.GetHour() < time2.GetHour())	return m_bStaticReverseSort ? 1 : -1;
		if(time1.GetMinute() > time2.GetMinute()) return m_bStaticReverseSort ? -1 : 1;
		if(time1.GetMinute() < time2.GetMinute()) return m_bStaticReverseSort ? 1 : -1;
		if(time1.GetSecond() > time2.GetSecond()) return m_bStaticReverseSort ? -1 : 1;
		if(time1.GetSecond() < time2.GetSecond()) return m_bStaticReverseSort ? 1 : -1;
		return 0;
	}
	break;
	}

	if(m_bStaticReverseSort)
		return strcmp(strItem2,strItem1);
	else
		return strcmp(strItem1,strItem2);
}

void CTimerList::OnColumnclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

	if(m_nSortColumn == pNMListView->iSubItem)
		m_bReverseSort = !m_bReverseSort;
	else
		m_bReverseSort = false;

	m_nSortColumn = (ListColumn)pNMListView->iSubItem;

	m_nStaticSortColumn = m_nSortColumn;
	m_bStaticReverseSort = m_bReverseSort;
	
	qsort(m_arTimers.GetData(),m_arTimers.GetSize(),sizeof(Timer*),CompareTimer);

	Invalidate();
	
	*pResult = 0;
}

void CTimerList::OnCsvExport()
{
    CFileDialog dlg(FALSE,"csv","Timers.csv",OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,"Comma Separated Values (*.csv)|*.csv||",this);
    if(dlg.DoModal() != IDOK)
        return;

    CString strPath(dlg.GetPathName());

    CStdioFile f;
    if(!f.Open(strPath,CFile::modeWrite|CFile::modeCreate))
    {
        CString strMsg;
        strMsg.Format("Failed to open the export file \"%s\"",strPath);
        MessageBox(strMsg,"Timer List",MB_ICONEXCLAMATION|MB_OK);
        return;
    }

    //Create a header line in the csv file
    CHeaderCtrl* pHeader = GetHeaderCtrl();
    int nNumColumns = pHeader->GetItemCount();

    char szText[256];
    HDITEM hdItem;
    hdItem.mask = HDI_TEXT;
    hdItem.pszText = szText;
    hdItem.cchTextMax = 256;

    CString strLine;
    for(int nCol=0;nCol<nNumColumns;nCol++)
    {
        pHeader->GetItem(nCol,&hdItem);
        strLine += hdItem.pszText;
        strLine += ",,";
    }
    strLine += "\n";
    f.WriteString(strLine);

    int nNumItems = GetItemCount();
    for(int nItem=0;nItem<nNumItems;nItem++)
    {
		strLine.Empty();
        for(int nColumn=0;nColumn<nNumColumns;nColumn++)
        {
            strLine += GetItemText(nItem,nColumn);
            strLine += ",,";
        }
        strLine += "\n";
	    f.WriteString(strLine);
    }
    f.Close();
    MessageBox("Export complete","Timer List",MB_ICONINFORMATION|MB_OK);
}
