/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CaptureSaveDlg.cpp : implementation file
//

#include "stdafx.h"
#include "modeller.h"
#include "CaptureSaveDlg.h"
#include "htmlhelp.h"
#include "Help\CaptureSaveDlg.inl"
#include <ConstantesLib.h>
#include "OstaDefs.h"
#include "RegFunctions.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CModellerApp theApp ;

//---------------------------------------------------------------------------

CCaptureSaveDlg::CCaptureSaveDlg(DialogMode nMode,CWnd* pParent /*=NULL*/)
        : CDialog(CCaptureSaveDlg::IDD, pParent)
{
    m_nDialogMode = nMode;
}


//---------------------------------------------------------------------------

void CCaptureSaveDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CCaptureSaveDlg)
    DDX_Control(pDX, IDC_LIST_CAPTURE, m_CaptureList);
    DDX_Control(pDX, IDC_SCRIPT_NAME, m_ctlScriptEdit);
    //}}AFX_DATA_MAP
}

//---------------------------------------------------------------------------


BEGIN_MESSAGE_MAP(CCaptureSaveDlg, CDialog)
//{{AFX_MSG_MAP(CCaptureSaveDlg)
ON_WM_HELPINFO()
ON_NOTIFY(NM_CLICK, IDC_LIST_CAPTURE, OnClickListCapture)
ON_NOTIFY(NM_DBLCLK, IDC_LIST_CAPTURE, OnDblclkListCapture)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCaptureSaveDlg message handlers
//---------------------------------------------------------------------------

BOOL CCaptureSaveDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    CWaitCursor wc;

    if(m_nDialogMode == enScriptName)
        SetWindowText("Script Name");

    InitList() ;
    FillList() ;

    static_cast<CEdit *>(GetDlgItem(IDC_SCRIPT_NAME))->SetLimitText(16);

    return TRUE;
}

//---------------------------------------------------------------------------

void CCaptureSaveDlg::FillList()
{
	if(m_nDialogMode == enCaptureMode || m_nDialogMode == enScriptName)
        theApp.m_RepositoryAPI.GetProjectCaptureFiles(m_arstrCaptures);
    else
        theApp.m_RepositoryAPI.GetProjectTemplateFiles(m_arstrCaptures);

    for( int nIndex = 0 ; nIndex < m_arstrCaptures.GetSize() ; nIndex ++ )
    {
		CString strFile(m_arstrCaptures[nIndex]);
		int nSlashPos = strFile.ReverseFind('\\');
		if(nSlashPos==-1)
			nSlashPos = 0;
		strFile = strFile.Mid(nSlashPos+1);

		int nDotPos = strFile.ReverseFind('.');
		if(nDotPos == -1)
			nDotPos = 0;
		strFile = strFile.Left(nDotPos);

		int nNdx = m_CaptureList.InsertItem( nIndex , strFile) ;
		m_CaptureList.SetItemData(nNdx,nIndex);
	}
}

//---------------------------------------------------------------------------

BOOL CCaptureSaveDlg::InitList()
{
    CBitmap bmp ;

    //Image List
    if( ! m_imlNormal.Create( 16, 16, ILC_COLOR32|ILC_MASK, 0, 1 ) )
        return FALSE;


    VERIFY(bmp.LoadBitmap(IDB_PROJECT));
    m_imlNormal.Add( &bmp, RGB(0,128,128));
    bmp.DeleteObject();

    m_CaptureList.SetImageList( &m_imlNormal ,LVSIL_SMALL ) ;

    return TRUE ;

}

//---------------------------------------------------------------------------

void CCaptureSaveDlg::OnOK()
{
    m_ctlScriptEdit.GetWindowText(m_strScriptName);

    if(m_strScriptName.IsEmpty())
    {
        AfxMessageBox("Please enter a file name");
        return;
    }

    if(m_strScriptName.Right(strlen(STR_SCL_EXTENSION)) != STR_SCL_EXTENSION)
        m_strScriptName += STR_SCL_EXTENSION;

	m_strScriptName = GetRepositoryPath() + strScriptType + "\\" + m_strScriptName;

    for(int nCapture=0;nCapture<m_arstrCaptures.GetSize();nCapture++)
    {
        if(m_arstrCaptures[nCapture].CompareNoCase(m_strScriptName)==0)
        {
            int nResult = 0;
            if(m_nDialogMode == enScriptName)
                nResult = AfxMessageBox("This script already exists.\nAre you sure you want to call it this?",MB_YESNO);
            else
                nResult = AfxMessageBox("This script already exists.\nDo you want to overwrite it?",MB_YESNO);

            if(nResult == IDNO)
                return;
        }
    }

    CDialog::OnOK();
}

BOOL CCaptureSaveDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

	if(pWnd)
	    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

void CCaptureSaveDlg::OnClickListCapture(NMHDR* pNMHDR, LRESULT* pResult)
{
    POSITION pos = m_CaptureList.GetFirstSelectedItemPosition();

    if(pos)
    {
        int nItem = m_CaptureList.GetNextSelectedItem(pos);

        CString strItem = m_CaptureList.GetItemText(nItem,0);
        m_ctlScriptEdit.SetWindowText(strItem);
    }

    *pResult = 0;
}

void CCaptureSaveDlg::OnDblclkListCapture(NMHDR* pNMHDR, LRESULT* pResult)
{
    POSITION pos = m_CaptureList.GetFirstSelectedItemPosition();

    if(pos)
    {
        int nItem = m_CaptureList.GetNextSelectedItem(pos);

        CString strItem = m_CaptureList.GetItemText(nItem,0);
        m_ctlScriptEdit.SetWindowText(strItem);
        OnOK();
    }

    *pResult = 0;
}
