/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MonitorGraph.cpp : implementation file
//

#include "stdafx.h"
#include "MonitorGraph.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
static int DEFAULT_REFRESH_RATE = 500;
const int VtChSeriesType2dLine = 6;
#define IDT_POLL_DATA 999
extern HINSTANCE m_hInstance;

/////////////////////////////////////////////////////////////////////////////
// CMonitorGraph dialog


CMonitorGraph::CMonitorGraph(CWnd* pParent /*=NULL*/)
	: CDialog(CMonitorGraph::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMonitorGraph)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_pParent = pParent;


	m_nFilePos = 0;


}


void CMonitorGraph::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMonitorGraph)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMonitorGraph, CDialog)
	//{{AFX_MSG_MAP(CMonitorGraph)
	ON_WM_TIMER()
	ON_WM_SIZE()
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMonitorGraph message handlers

void CMonitorGraph::InitMonitor(CString strRunFolder, CStringArray& arrDesc)
{
	m_strRunFolder = strRunFolder;
	LoadHeaders(arrDesc);
}

BOOL CMonitorGraph::OnInitDialog() 
{
	CDialog::OnInitDialog();
	SetWindowText(m_strDesc);
	SetIcon(LoadIcon( m_hInstance, MAKEINTRESOURCE(IDI_MONITOR)), FALSE);

	for (int i = 0; i<m_arrGraphs.GetSize(); i++)
	{
		GRAPH_INFO inf = m_arrGraphs.GetAt(i);
		inf.pChart = CreateDisplay(m_arrGraphs.GetAt(i).strDesc);
		m_arrGraphs.SetAt(i, inf);
	}
	SetTimer(IDT_POLL_DATA,DEFAULT_REFRESH_RATE,NULL);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

CXChartCtrl* CMonitorGraph::CreateDisplay(CString strTitle)
{
    CXChartCtrl *pChart = new CXChartCtrl;

    if(!pChart)
        return NULL;

	CRect rcGraph;
	GetClientRect(rcGraph);

    pChart->Create(NULL,WS_CHILD|WS_VISIBLE,rcGraph,this,0);

    pChart->SetPlotBkColor(RGB(155,155,155));

    pChart->SetLabelColor(RGB(255,255,255));

    pChart->SetAxisTextColor(RGB(90,90,90));

    pChart->SetGridColor(RGB(90,90,90));

    pChart->AddDataPoint(1,0.0);
    pChart->SetSeriesStyle(1,VtChSeriesType2dLine);
    pChart->EnableZoom(FALSE);

    const int nMaxGraphs = 7;
    const COLORREF colLines[nMaxGraphs]=
        {
            RGB(255,0,0),   RGB(0,255,0),
            RGB(0,0,255),   RGB(255,255,0),
            RGB(255,0,255), RGB(0,255,255),
            RGB(255,255,255)
        };

    pChart->SetYAxisLabel(strTitle);
    return pChart;
}

void CMonitorGraph::OnTimer(UINT nIDEvent) 
{
    try
    {
		CString StatPath = 	m_strRunFolder + "\\NTPStat.txt";
		CStdioFile f(StatPath, CFile::modeRead | CFile::typeText | CFile::shareDenyNone );
		f.Seek(m_nFilePos, CFile::begin);

        CString strData;
		while(f.ReadString(strData))
		{
            CString node, index, start, stop, value;
			GetNextField(',', strData, node);
			GetNextField(',', strData, index);
			GetNextField(',', strData, start);
			GetNextField(',', strData, stop);
			GetNextField(',', strData, value);
            double dVal = atof(value);

			for (int i =0; i<m_arrGraphs.GetSize(); i++)
			{
				if (atoi(index) == m_arrGraphs.GetAt(i).nDataIndex)
				{
					m_arrGraphs.GetAt(i).pChart->AddDataPoint(1,dVal);
					m_arrGraphs.GetAt(i).pChart->Invalidate();
				}
			}
        }

		m_nFilePos = f.GetPosition();
    }
    catch(...)
    {
        GetParent()->PostMessage(WM_CLOSE);
    }		
	CDialog::OnTimer(nIDEvent);
}

int CMonitorGraph::GetNextField(char sep, CString &inStr, CString &field)
{
    if (inStr == "")
    {
        return -1;
    }

    int len = inStr.GetLength();
    int nPos = inStr.Find(sep, 0);

    if (nPos == -1)
    {
        field = inStr;
        inStr = "";
        return 0;
    }
    else
    {
        field = inStr.Left(nPos);
        inStr = inStr.Right(len - nPos - 1);
        return 0;
    }
}

void CMonitorGraph::OnSize(UINT nType, int cx, int cy) 
{
	CDialog::OnSize(nType, cx, cy);

	int nHeight = cy/m_arrGraphs.GetSize();
	for (int i = 0; i<m_arrGraphs.GetSize(); i++)
	{
		GRAPH_INFO inf = m_arrGraphs.GetAt(i);
		if(inf.pChart && inf.pChart->m_hWnd)
		{
			inf.pChart->MoveWindow(0,i*nHeight,cx, nHeight);			
		}
	}
}

BOOL CMonitorGraph::OnEraseBkgnd(CDC *pDC)
{
	return true;
}


void CMonitorGraph::LoadHeaders(CStringArray& arrDesc)
{
	try
	{
		// Find the data index of this description
		CString HeaderPath = m_strRunFolder + "\\NTPHeader.txt";
		CStdioFile f(HeaderPath, CFile::modeRead | CFile::typeText );

		CString headerInfo;
		while(f.ReadString(headerInfo))
		{
			GRAPH_INFO inf;

	        CString node = strtok(headerInfo.GetBuffer(0),",");
	        CString strIndex = strtok(NULL,",");
	        inf.nDataIndex = atoi(strIndex);
	        inf.strDesc = strtok(NULL,",");
	        inf.strDesc.TrimRight(",");
			inf.pChart = NULL;

			for (int i = 0; i<arrDesc.GetSize(); i++)
			{
				if (inf.strDesc == arrDesc.GetAt(i))
				{
					m_arrGraphs.Add(inf);
				}
			}
		}
	}
	catch(...)
	{
	}

}
