// HTTPGraphs.cpp: implementation of the CNTPerfGraphs class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NTPerfGraphs.h"
#include "GraphTypes.h"
#include "GraphWnd.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CNTPerfGraphs::CNTPerfGraphs():
	m_strCustomNTP("Custom NT Performance")
{
}

CNTPerfGraphs::~CNTPerfGraphs()
{
	POSITION pos = m_listNTPData.GetHeadPosition();
	while(pos)
		delete m_listNTPData.GetNext(pos);
	m_listNTPData.RemoveAll();
}

void CNTPerfGraphs::LoadRun(const CString& strRunFolder)
{
	m_strLoadedRun = strRunFolder;
	LoadStats();

	int nSlashPos = m_strLoadedRun.ReverseFind('\\');
	CString strRunName(m_strLoadedRun.Mid(nSlashPos+1));
	
	int nDay,nMonth,nYear,nHour,nMinute,nSecond;
	char *pszToken = strtok(strRunName.GetBuffer(0),"-");
	if(pszToken)
		nDay = atoi(pszToken);

	if(pszToken = strtok(NULL,"-"))
		nMonth = atoi(pszToken);

	if(pszToken = strtok(NULL," "))
		nYear = atoi(pszToken);

	if(pszToken = strtok(NULL,"-"))
		nHour = atoi(pszToken);

	if(pszToken = strtok(NULL,"-"))
		nMinute = atoi(pszToken);

	if(pszToken = strtok(NULL,"."))
		nSecond = atoi(pszToken);

	CTime T(nYear,nMonth,nDay,nHour,nMinute,nSecond);
	m_dwTestStartTime = T.GetTime();
}

void CNTPerfGraphs::GetAvailableGraphs(CStringArray& arstrGraphs)
{
	arstrGraphs.RemoveAll();

	if(m_listNTPData.GetCount())
	{
		arstrGraphs.Add(m_strCustomNTP);
	}
}

CWnd* CNTPerfGraphs::DisplayGraph(const CString& strName,CWnd *pParent,const CRect& rcBounds)
{
	CGraphWnd *pGraphWnd = new CGraphWnd;
	pGraphWnd->Create(NULL,"",WS_CHILD|WS_VISIBLE,rcBounds,pParent,0);
	
	CNTPGraph *pGraph = NULL;
	if(strName == m_strCustomNTP)
		pGraph = new CNTPGraph(pGraphWnd->m_pChart);
	
	pGraphWnd->m_pGraph = pGraph;

	if(!pGraph || !pGraph->AddTestRun(&m_listNTPData,m_dwTestStartTime))
	{
		delete pGraphWnd;
		return NULL;
	}
	pGraphWnd->SetGraphStyles();
		
	return pGraphWnd;
}

void CNTPerfGraphs::LoadStats()
{
	POSITION pos = m_listNTPData.GetHeadPosition();
	while(pos)
		delete m_listNTPData.GetNext(pos);
	m_listNTPData.RemoveAll();

	CString strHeaderPath(m_strLoadedRun + "\\NTPHeader.txt");
	CString strStatsPath(m_strLoadedRun + "\\NTPStat.txt");
    
    try
    {
        CStdioFile NTPFile(strHeaderPath,CFile::modeRead);

		CString strHeader;
		while(NTPFile.ReadString(strHeader))
		{
			NTPHeader *pHeader = new NTPHeader;
			char *pszToken = strtok(strHeader.GetBuffer(0),"_");
			if(!pszToken)
				continue;
			((LPBYTE)(&pHeader->dwIP))[0] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,"_"))) continue;
			((LPBYTE)(&pHeader->dwIP))[1] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,"_"))) continue;
			((LPBYTE)(&pHeader->dwIP))[2] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,","))) continue;
			((LPBYTE)(&pHeader->dwIP))[3] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,",")))
				continue;
			pHeader->nIndex = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,",")))
				continue;
			pHeader->strQuery = pszToken;

			m_listNTPData.AddTail(pHeader);
		}

		NTPFile.Close();

        NTPFile.Open(strStatsPath,CFile::modeRead);

		CString strStat;
		while(NTPFile.ReadString(strHeader))
		{
			NTPStat stat;
			char *pszToken = strtok(strHeader.GetBuffer(0),"_");
			if(!pszToken)
				continue;
			((LPBYTE)(&stat.dwIP))[0] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,"_"))) continue;
			((LPBYTE)(&stat.dwIP))[1] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,"_"))) continue;
			((LPBYTE)(&stat.dwIP))[2] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,","))) continue;
			((LPBYTE)(&stat.dwIP))[3] = atoi(pszToken);
			
			if(!(pszToken = strtok(NULL,",")))
				continue;
			stat.nIndex = atoi(pszToken);

			if(!(pszToken = strtok(NULL,",")))
				continue;
			stat.dwStartTime = atoi(pszToken);

			if(!(pszToken = strtok(NULL,",")))
				continue;
			stat.dwEndTime = atoi(pszToken); 

			if(!(pszToken = strtok(NULL,",")))
				continue;
			stat.dwValue = atoi(pszToken);

			//Find header for this stat
			POSITION pos = m_listNTPData.GetHeadPosition();
			while(pos)
			{
				NTPHeader *pHeader = m_listNTPData.GetNext(pos);
				if((pHeader->dwIP == stat.dwIP) && (pHeader->nIndex == stat.nIndex))
				{
					pHeader->listStats.AddTail(stat);
					pos = NULL;
				}
			}
		}
	}
	catch(...)
	{
	}
}

