/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyranoOTA
///////////////////////////////////////////////////////////////////////////////

#ifndef _OtaDefs_h
#define _OtaDefs_h

///////////////////////////////////////////////////////////////////////////////
// Standard includes
#include <string>

#ifndef VMS
using std::string;
#endif

// Tracing levels
#define TRACE_CONFIG        10      // Configuration
#define TRACE_RESULTS       20      // Results
#define TRACE_PARTIALDIAG   30      // Partial diagnostics
#define TRACE_FULLDIAG      40      // Full diagnostics

void OtaAddTrace( string category, long level, string msg );

enum OtaQueryTypes { QUERY_SQL, QUERY_EXEC, QUERY_NTPERF, QUERY_SNMP };

// macro to catch all the exceptions I can think of.
// need to set the string catchmsg, and bool caught to false
// before the try block
#define OTACATCHALL \
catch(string es) \
{ /* Detected error */ \
string msg = catchmsg; \
msg += "string: "; \
msg += es; \
OtaAddTrace(STARTUP, ERRORLEVEL, msg); \
caught=true; \
} \
catch(mdbIOException & e) \
{ /* derived from mdbRuntimeError */ \
string msg = catchmsg; \
msg += "mdbIOException: "; \
msg += e.what() + ": " + e.ioname(); \
\
OtaAddTrace(STARTUP, ERRORLEVEL, msg); \
caught=true; \
} \
catch(mdbParseException & e) \
{ /* derived from mdbRuntimeError */ \
string msg = catchmsg; \
msg += "mdbParseException: "; \
msg += e.what(); \
OtaAddTrace(STARTUP, ERRORLEVEL, msg); \
caught=true; \
} \
catch(mdbRuntimeError & e) \
{ /* derived from runtime_error */ \
string msg = catchmsg; \
msg += "mdbRuntimeError: "; \
msg += e.what(); \
\
OtaAddTrace(STARTUP, ERRORLEVEL, msg); \
caught=true; \
} \
catch(CyrNoFreeThreadException &) \
{ /* derived from CORBA::UserException */ \
string msg = catchmsg; \
msg += "CyrNoFreeThreadException: "; \
OtaAddTrace(STARTUP, ERRORLEVEL, msg); \
caught=true; \
} \
catch(CyrProcessException & /* TODO e */) \
{ /* derived from CORBA::UserException */ \
string msg = catchmsg; \
msg += "CyrProcessException: "; /* TODO + e.what() + ":" + e.where() + ":" + e.status(); */ \
OtaAddTrace(STARTUP, ERRORLEVEL, msg); \
caught=true; \
} \
catch(CyrRepositoryException & e) \
{ \
        /* derived from CyrException */ \
string msg = catchmsg; \
msg += "CyrRepositoryException: "; \
msg += e.what(); \
OtaAddTrace(STARTUP, ERRORLEVEL, msg ); \
caught=true; \
} \
catch(CyrException & e) \
{ \
        /* no derivation */ \
string msg = catchmsg; \
msg += "CyrException: "; \
msg += string(e.what()) + ":" + string(e.where()) + ":" + string(int_to_string(e.status())); \
OtaAddTrace(STARTUP, ERRORLEVEL, msg); \
caught=true; \
} \
catch(CORBA::Exception&) \
{ \
string msg = catchmsg; \
msg += "CORBA::Exception: "; \
OtaAddTrace(STARTUP, ERRORLEVEL, msg); \
caught=true; \
} \
catch(runtime_error & e) \
{ \
        /* derived from exception */ \
string msg = catchmsg; \
msg += "runtime_error: "; \
msg += e.what(); \
OtaAddTrace(STARTUP, ERRORLEVEL, msg ); \
caught=true; \
} \
catch(exception & e) \
{ \
string msg = catchmsg; \
msg += "exception: "; \
msg += e.what(); \
OtaAddTrace(STARTUP, ERRORLEVEL, msg ); \
caught=true; \
} \
catch(...) \
{ \
        /* Some other unexpected exception caught */ \
string msg = catchmsg; \
msg += "...: "; \
OtaAddTrace(STARTUP, ERRORLEVEL, msg ); \
caught=true; \
}
#endif
