/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// SNMPScanDlg.cpp : implementation file
//

#include "stdafx.h"
#include "SNMPScanDlg.h"
#include "winsock.h"
#include "CECSNMPQuery.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSNMPScanDlg dialog


CSNMPScanDlg::CSNMPScanDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CSNMPScanDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSNMPScanDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	WORD      VersionRequested;
	WSADATA   stWSAData;

	VersionRequested = MAKEWORD(1, 1);
	WSAStartup(VersionRequested, &stWSAData);

}


void CSNMPScanDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSNMPScanDlg)
	DDX_Control(pDX, IDC_IP_END, m_IPEnd);
	DDX_Control(pDX, IDC_IP_START, m_IPStart);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSNMPScanDlg, CDialog)
	//{{AFX_MSG_MAP(CSNMPScanDlg)
	ON_BN_CLICKED(IDC_SCAN, OnScan)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSNMPScanDlg message handlers

BOOL CSNMPScanDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// Get the host IP of this machine
    char hostname[256];
    gethostname(hostname,256);
    hostent* host=gethostbyname(hostname);
	CString strIP = "";
    in_addr* addr;
    if(host)
    {
        addr=((struct in_addr*)((host->h_addr_list)[0]));
		strIP = inet_ntoa(*addr);
    }
	
	int nPos = strIP.ReverseFind('.');
	long startIP = inet_addr(strIP.Left(nPos) + ".0");
	long endIP = inet_addr(strIP.Left(nPos) + ".255");
	
	m_IPStart.SetAddress(ntohl(startIP));
	m_IPEnd.SetAddress(ntohl(endIP));

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CSNMPScanDlg::OnScan() 
{
	BYTE b1,b2,b3,b4;
	m_IPStart.GetAddress(b1,b2,b3,b4);
	
	m_addrStart.byte1=(int)b1;
	m_addrStart.byte2=(int)b2;
	m_addrStart.byte3=(int)b3;
	m_addrStart.byte4=(int)b4;

	m_IPEnd.GetAddress(b1,b2,b3,b4);
	m_addrEnd.byte1=(int)b1;
	m_addrEnd.byte2=(int)b2;
	m_addrEnd.byte3=(int)b3;
	m_addrEnd.byte4=(int)b4;

	BOOL bOKToStart = IsRangeOK();
	if (!bOKToStart)
	{
		AfxMessageBox("Invalid Range");
		return;
	}

	RunScan();
	CDialog::OnOK();
}

BOOL CSNMPScanDlg::IsRangeOK()
{
	if (m_addrEnd.byte1 < m_addrStart.byte1)
		return FALSE;
	
	if (m_addrEnd.byte2 < m_addrStart.byte2)
		return FALSE;

	if (m_addrEnd.byte3 < m_addrStart.byte3)
		return FALSE;

	if (m_addrEnd.byte4 < m_addrStart.byte4)
		return FALSE;

	return TRUE;
}

void CSNMPScanDlg::RunScan()
{
	CWaitCursor wc;

	for (int i=m_addrStart.byte1; i<=m_addrEnd.byte1 && i<=255; i++)
	{
		for (int j=m_addrStart.byte2; j<=m_addrEnd.byte2 && j<=255; j++)
		{
			for (int k=m_addrStart.byte3; k<=m_addrEnd.byte3 && k<=255; k++)
			{
				for (int l=m_addrStart.byte4; l<=m_addrEnd.byte4 && l<=255; l++)
				{
					CString strIP;
					strIP.Format("%d.%d.%d.%d",i,j,k,l);

					QUERY_DEF def;
					def.header = strIP;
					def.community = "public";
					def.server = strIP;
					def.query = "system.sysName.0";
					def.nTimeout = 5000;
		
					CECSNMPQuery query;
					CString result;

					if(query.ExecuteDetailed(def, result))
					{
						m_arrSNMPServers.Add(strIP);
					}
				}
			}
		}
	}
}

void CSNMPScanDlg::GetServers(CStringArray &arrServers)
{
	for (int i=0; i<m_arrSNMPServers.GetSize(); i++)
	{
		arrServers.Add(m_arrSNMPServers.GetAt(i));
	}
}
